/*
 *  (C) Copyright 2007-2009 by FreeShoppa Development Team
 *                        (http://chomzee.ethernet.pl/freeshoppa/)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <GL/gl.h>
#ifdef WIN32
#include <gl\glext.h>
#endif

#include "freeshoppa.h"

/* thread and socket for client */
static SDL_Thread *thread;
static TCPsocket tcpsock;

/* static variables for storing data recieved from server */
static Uint8 buf[BUFLEN];
static int len;
static Uint8 *argv[MAX_LINES_PACKET];
static int argc;

static char mapname[81]; /* mapname */

/* textures */
int client_texArrow = 0;
int client_texZookaLabel = 0;
int client_texGrenadeLabel = 0;
int client_texDynamiteLabel = 0;
int client_texClusterLabel = 0;
int client_texKillerBeeLabel = 0;
int client_texHamstersLabel = 0;
int client_texUp = 0;
int client_texDown = 0;
int client_texLeft = 0;
int client_texRight = 0;
int client_texRopeShoot = 0;
int client_texLag = 0;
int client_texZooka = 0;
int client_texGrenade = 0;
int client_texDynamite = 0;
int client_texCluster = 0;
int client_texCluster2 = 0;
int client_texHamster = 0;
int client_texKillerBee = 0;
int client_texBall0 = 0;
int client_texBall1 = 0;
int client_texBall2 = 0;
int client_texBall3 = 0;
int client_texBall4 = 0;
int client_texBall5 = 0;
int client_texBox = 0;
int client_texBack = 0;

void *client_sndColision1 = NULL;
void *client_sndColision2 = NULL;
void *client_sndColision3 = NULL;
void *client_sndRope1 = NULL;
void *client_sndRope2 = NULL;
void *client_sndHamster1 = NULL;
void *client_sndHamster2 = NULL;
void *client_sndHamster3 = NULL;
void *client_sndHamster4 = NULL;
void *client_sndBoom = NULL;
void *client_sndJump = NULL;
void *client_sndThrow = NULL;
void *client_sndWater = NULL;
static void *sndChat = NULL;
static void *sndCow = NULL;
static void *sndOneAndOnly = NULL;

/* client_isError is 1 when some error ocurred */
int client_isError;
char client_errormsg[81]; /* error message */

/* players' structures */
ClientPlayer client_players[6];
static int max_players;
static int count_players;

char client_gameInProgress; /* 1 = game in progress, 0 = wating for players */
char client_serverName[81];
char client_motd[9][81];
Map *client_map;
int client_turn; /* contains index of player currently doing the turn */
int client_turnBall; /* contains index of ball which is in use now */
char client_doTheJob; /* 1 = we have to play */
char client_playTheJob; /* 1 = we just have to replay enemy's turn */
int client_secondsRecieved; /* number of SECOND packets already recieved */
char client_connected; /* 1 = connected, 0 = not connected */
int client_gametype; /* game type: 0=shoppa, 1=fly, 2=wxw, 3=proper, 4=rr */
int client_timelimit;
int client_fall; /* falling damage */
int client_balls;
int client_health;
char client_local; /* 1 = we are the host or playing practice */
char client_exitConfirm; /* Are you sure you want to quit? */

int client_cameraFollow; /* 0 = camera follows currently playing ball */
int client_cameraFocus; /* 1=ball, 2=drop */

ESS client_ess[80]; /* Every Second Sync */
char client_lag; /* lag icon */

char client_rcBuf[20][81]; /* stuff needed for Remote Console */
int client_rcCount;

static char oneandonly;

/* Houston, we have a problem */
static void error(char *reason)
{
	console_print("  client: %s", reason);
	snprintf(client_errormsg, 81, reason);
	client_isError = 1;

	return;
}

/* Houston reports some problems */
static void errorSrv(char *reason)
{
	console_print("  client: server said: %s", reason);
	snprintf(client_errormsg, 81, reason);
	client_isError = 1;

	return;
}

/* pack recieved data into argv[] and update argc */
static int preparePacket()
{
	int p;

	argc = 1;
	argv[0] = &buf[0];

	for (p=0; p<len; p++)
	{
		if (p>0 && p<(len-1)
			&& buf[p] == '\n'
			&& buf[p-1] == '\r')
		{
			argv[argc] = &buf[p+1];
			argc++;

			buf[p-1] = 0;

			if (argc>=MAX_LINES_PACKET)
			{
				error("too many lines in one packet!");
				return 1;
			}
		}
	}

	return 0;
}

/* recieve packet from the server */
static int recievePacket()
{
	int p = 0;
	int result;
	for (;;)
	{
		if (p >= BUFLEN)
		{
			error("too much data!.");
			return 1;
		}
		result = SDLNet_TCP_Recv(tcpsock, &buf[p], 1);
		if (result<1)
		{
			error("server closed connection.");
			return 1;
		}
		p++;
		if (p>=4 && buf[p-1]=='\n' && buf[p-2]=='\r'
			 && buf[p-3]=='\n' && buf[p-4]=='\r')
		{
			buf[p-4] = 0;
			len = p-4;
			p = 0;
			if (preparePacket())
				return 1;
			else
				return 0;
		}

	}
}

static int processServerState()
{

	if (strcmp((char*)argv[0], PACKET_SERVER_STATE))
	{
		if (strcmp((char*)argv[0], PACKET_ERROR)==0 && (argc == 2))
			errorSrv((char*)argv[1]);
		else
			error("Protocol incompability.");
		return 1;
	}

	if (argc < 11)
	{
		error("Protocol incompability.");
		return 1;
	}
	snprintf(client_serverName, 81, (char*)argv[1]);
	
	sscanf((char*)argv[2], "%d", &client_gametype);
	if (client_gametype<0 || client_gametype>4)
	{
		error("Protocol incompability.");
		return 1;
	}

	sscanf((char*)argv[3], "%d", &client_timelimit);
	if (client_timelimit<5 || client_timelimit>60)
	{
		error("Protocol incompability.");
		return 1;
	}
	sscanf((char*)argv[4], "%d", &client_fall);
	if (client_fall<0 || client_fall>1)
	{
		error("Protocol incompability.");
		return 1;
	}


	snprintf(mapname, 81, (char*)argv[5]);
	sscanf((char*)argv[7], "%d", &client_balls);
	sscanf((char*)argv[8], "%d", &client_health);
	sscanf((char*)argv[9], "%d", &max_players);
	sscanf((char*)argv[10], "%d", &count_players);

	if (max_players <= count_players)
	{
		error("Server full.");
		return 1;
	}
	
	return 0;
}

static int processWelcome()
{
	if (!strcmp((char*)argv[0], PACKET_WELCOME) && (argc == 10))
	{
		int p;
		memset(client_motd, 0, 9*81);
		for (p=0; p<9; p++)
			snprintf(client_motd[p], 81, (char*)argv[p+1]);
		return 0;
	}
	else if (!strcmp((char*)argv[0], PACKET_ERROR) && (argc == 2))
	{
		errorSrv((char*)argv[1]);
		return 1;
	}
	else
	{
		error("Protocol incompability.");
		return 1;
	}
}

static void sendHello()
{
	char buf[BUFLEN];

	if (client_isError)
		return;

	memset(buf, 0, BUFLEN);
	sprintf(&buf[strlen(buf)], "%s\r\n\r\n", PACKET_HELLO);

	SDLNet_TCP_Send(tcpsock, buf, strlen(buf));
	return;
}

static void sendJoin()
{
	char buf[BUFLEN];

	if (client_isError)
		return;

	memset(buf, 0, BUFLEN);
	sprintf(&buf[strlen(buf)], "%s\r\n", PACKET_JOIN);
	sprintf(&buf[strlen(buf)], "%s\r\n", commands_name);
	sprintf(&buf[strlen(buf)], "%s\r\n", client_map->md5);
	sprintf(&buf[strlen(buf)], "%s\r\n", VERSION);
	sprintf(&buf[strlen(buf)], "%s\r\n", PROTOCOL_VERSION);
	if (commands_cl_passwd[0])
		sprintf(&buf[strlen(buf)], "%s\r\n", commands_cl_passwd);
	sprintf(&buf[strlen(buf)], "\r\n");

	SDLNet_TCP_Send(tcpsock, buf, strlen(buf));
	return;
}

/* check what packet we've recieved and react */
static int processPacket()
{
	if (strcmp((char*)argv[0], PACKET_ERROR)==0 && (argc == 2))
	{
		errorSrv((char*)argv[1]);
		return 1;
	}
	else if (strcmp((char*)argv[0], PACKET_COW)==0 && (argc == 2))
	{
		audio_play(sndCow);
		console_messagePrint("COW! %s^0 will loose his turn!",
			argv[1]);
		return 0;
	}
	else if (strcmp((char*)argv[0], PACKET_SAY)==0 && (argc == 3))
	{
		audio_play(sndChat);
		console_print("%s^0: ^2%s", argv[1], argv[2]);
		if (strlen((char*)argv[1]) + strlen((char*)argv[2]) + 2 > 70)
		{
			char buf[BUFLEN];
			char tmp;

			memset(buf, 0, BUFLEN);
			snprintf(buf, BUFLEN, "%s^0: ^2%s", argv[1], argv[2]);
			tmp = buf[70];
			buf[70] = 0;
			console_messagePrint("%s", buf);
			buf[70] = tmp;
			console_messagePrint("     ^2%s", &buf[70]);
		}
		else
			console_messagePrint("%s^0: ^2%s", argv[1], argv[2]);
		return 0;
	}
	else if (strcmp((char*)argv[0], PACKET_NAME)==0 && (argc == 3))
	{
		console_print("%s^0 renamed to %s", argv[1], argv[2]);
		console_messagePrint("%s^0 renamed to %s", argv[1], argv[2]);
		return 0;
	}
	else if (strcmp((char*)argv[0], PACKET_CONNECTED)==0 && (argc == 2))
	{
		console_print("%s^0 connected", argv[1]);
		console_messagePrint("%s^0 connected", argv[1]);
		return 0;
	}
	else if (strcmp((char*)argv[0], PACKET_DISCONNECTED)==0 && (argc == 2))
	{
		console_print("%s^0 disconnected", argv[1]);
		console_messagePrint("%s^0 disconnected", argv[1]);
		return 0;
	}
	else if (strcmp((char*)argv[0], PACKET_READY)==0 && (argc == 2))
	{
		console_messagePrint("%s^0 is ready!", argv[1]);
		return 0;
	}
	else if (strcmp((char*)argv[0], PACKET_UNREADY)==0 && (argc == 2))
	{
		console_messagePrint("%s^0 is not ready!", argv[1]);
		return 0;
	}
	else if (strcmp((char*)argv[0], PACKET_INGAME)==0 && (argc == 2))
	{
		console_messagePrint("%s^0 joined the game", argv[1]);
		return 0;
	}
	else if (strcmp((char*)argv[0], PACKET_SPECTATOR)==0 && (argc == 2))
	{
		console_messagePrint("%s^0 joined spectators", argv[1]);
		return 0;
	}
	else if (strcmp((char*)argv[0], PACKET_SPEC)==0 && (argc == 1))
	{
		console_print("^1There is a game in progress. "
			"Please wait for next turn...");
		console_messagePrint("^1There is a game in progress. "
			"Please wait for next turn...");
		return 0;
	}
	else if (strcmp((char*)argv[0], PACKET_STATE)==0 && !((argc-2)%4))
	{
		int p;

		if (strcmp((char*)argv[1], "IN PROGRESS")==0)
			client_gameInProgress = 1;
		else if (strcmp((char*)argv[1], "WAITING FOR PLAYERS")==0)
			client_gameInProgress = 0;

		/* do some cleaning before copying data */
		for (p=0; p<6; p++)
		{
			client_players[p].status = 0;
			client_players[p].ready = 0;
			client_players[p].ingame = 0;
			memset(&(client_players[p].name), 0, 81);

		}

		for (p=0; p<((argc-2)/4); p++)
		{
			int id;

			sscanf((char*)argv[2+p*4], "%d", &id);

			if (id<0 || id>5)
				/* Protocol incompability! */
				break;

			client_players[id].status = 1;

			if (strcmp((char*)argv[2+p*4+1], "READY")==0)
				client_players[id].ready = 1;
			else if (strcmp((char*)argv[2+p*4+1], "UNREADY")==0)
				client_players[id].ready = 0;

			if (strcmp((char*)argv[2+p*4+2], "INGAME")==0)
				client_players[id].ingame = 1;
			else if (strcmp((char*)argv[2+p*4+2], "SPECTATOR")==0)
				client_players[id].ingame = 0;

			snprintf(client_players[id].name, 81, "%s",
				(char*)argv[2+p*4+3]);
		}

		return 0;
	}
	else if (strcmp((char*)argv[0], PACKET_GAME_START)==0 && (argc == 1))
	{
		int p, q;
		for (p=0; p<6; p++)
			for (q=0; q<4; q++)
			{
				client_players[p].ball[q].status = 0;
				client_players[p].ball[q].eyes = rand()%2;
			}

		client_doTheJob = 0;
		client_playTheJob = 0;

		client_gameInProgress = 1;
		client_turn = -1;
		console_messagePrint(
			"All players are ready. ^2Game is starting!");
		if (client_gametype == 4)
			oneandonly = 1;
		else
			oneandonly = 0;
		return 0;
	}
	else if (strcmp((char*)argv[0], PACKET_GAME_END)==0 && (argc == 2))
	{
		int p, q;
		int won;


		client_gameInProgress = 0;
		client_turn = -1;
		game_wind = 0;
		client_doTheJob = 0;
		client_playTheJob = 0;

		for (p=0; p<6; p++)
			for (q=0; q<4; q++)
			{
				client_players[p].ball[q].health = 0;
			}
		game_frame = 0;
		
		sscanf((char*)argv[1], "%d", &won);

		if (won >= 0 && won <= 6)
		{
			console_messagePrint("%s ^0has won the game!",
				client_players[won].name);
		}
		else
			console_messagePrint("The game was drawn.");
		return 0;
	}
	else if (strcmp((char*)argv[0], PACKET_COORDS)==0 && (argc > 2))
	{
		int pl;
		int balls;
		int p, q;

		sscanf((char*)argv[1], "%d", &pl);
		sscanf((char*)argv[2], "%d", &balls);
		if (pl<1 || pl>6 || (argc != ((3*balls+1)*pl+3)))
		{
			/* Protocol incompability! */
			return 0;
		}
		if (client_playTheJob)
		{
			while (client_secondsRecieved*50 != game_frame)
				SDL_Delay(1);
			client_playTheJob = 0;
		}

		for (p=0; p<pl; p++)
		{
			int id;
			sscanf((char*)argv[3+p*(3*balls+1)], "%d", &id);
			if (id<0 || id>5)
			{
				/* Protocol incompability! */
				return 0;
			}
			for (q=0; q<balls; q++)
			{
				int tmp;
				sscanf((char*)argv[3+p*(3*balls+1)+q*3+1],
					"%d",
					&(client_players[id].ball[q].health));
				if (client_players[id].ball[q].health)
					client_players[id].ball[q].status = 1;
				else
					client_players[id].ball[q].status = 0;

				sscanf((char*)argv[3+p*(3*balls+1)+q*3+2],
					"%d", &tmp);
				memcpy(&(client_players[id].ball[q].pos.x),
					&tmp, 4);

				sscanf((char*)argv[3+p*(3*balls+1)+q*3+3],
					"%d", &tmp);
				memcpy(&(client_players[id].ball[q].pos.y),
					&tmp, 4);
			}
		}

		return 0;
	}
	else if (strcmp((char*)argv[0], PACKET_TURN)==0
		&& (argc == 4 || argc == 7))
	{
		sscanf((char*)argv[1], "%d", &client_turn);
		sscanf((char*)argv[2], "%d", &client_turnBall);
		if (client_turn<0 || client_turn >5
			|| !client_players[client_turn].status
			|| client_turnBall<0 || client_turnBall>3)
		{
			/* Protocol incompability! */
			return 0;
		}

		sscanf((char*)argv[3], "%d", &game_wind);
		if (game_wind<-100 || game_wind>100)
		{
			/* Protocol incompability! */
			return 0;
		}

		if (argc == 7)
		{
			if (client_gametype != 4)
			{
				sscanf((char*)argv[4], "%d", &game_dropPos.x);
				sscanf((char*)argv[5], "%d", &game_dropPos.y);
				sscanf((char*)argv[6], "%d", &game_dropType);

				if (game_dropType<0 || game_dropType>9)
				{
					/* Protocol incompability! */
					return 0;
				}
			}
			else
			{
				/* Protocol incompability! */
				return 0;
			}
		}

		game_frame = 0;
		client_secondsRecieved = 0;
		client_lag = 0;
		client_playTheJob = 1;
		if (client_gametype != 4)
			client_cameraFocus = 2;
		console_messagePrint("%s^0 is doing his turn...",
			client_players[client_turn].name, argv[2]);

		return 0;
	}
	else if (strcmp((char*)argv[0], PACKET_GO_TURN)==0
		&& (argc == 4 || argc == 7))
	{
		int count;
		int p;

		sscanf((char*)argv[1], "%d", &client_turn);
		sscanf((char*)argv[2], "%d", &client_turnBall);
		if (client_turn<0 || client_turn >5
			|| !client_players[client_turn].status
			|| client_turnBall<0 || client_turnBall>3)
		{
			/* Protocol incompability! */
			return 0;
		}

		sscanf((char*)argv[3], "%d", &game_wind);
		if (game_wind<-100 || game_wind>100)
		{
			/* Protocol incompability! */
			return 0;
		}

		if (argc == 7)
		{
			if (client_gametype != 4)
			{
				sscanf((char*)argv[4], "%d", &game_dropPos.x);
				sscanf((char*)argv[5], "%d", &game_dropPos.y);
				sscanf((char*)argv[6], "%d", &game_dropType);

				if (game_dropType<0 || game_dropType>9)
				{
					/* Protocol incompability! */
					return 0;
				}
			}
			else
			{
				/* Protocol incompability! */
				return 0;
			}
		}

		game_frame = 0;
		game_prepare = 1;
		client_doTheJob = 1;
		if (client_gametype != 4)
			client_cameraFocus = 2;
		console_messagePrint("^1It's our turn! GO GO GO!");

		/* One and only :> */
		count = 0;
		for (p=0; p<4; p++)
			if (client_players[client_turn].ball[p].status)
				count++;
		if (count == 1 && !oneandonly)
		{
			audio_play(sndOneAndOnly);
			oneandonly = 1;
		}
		return 0;
	}
	else if (strcmp((char*)argv[0], PACKET_SECOND)==0 && (argc == 224))
	{
		int sec;
		int p, q, tmp;
		sscanf((char*)argv[1], "%d", &sec);

		if (sec==0)
			client_cameraFocus = 1;

		if (sec != client_secondsRecieved)
		{
			return 0;
		}

		for (p=0; p<50; p++)
		{
			if (strlen((char*)argv[p+2]) > 7)
			{
				/* Protocol incompability! */
				return 0;
			}
			strncpy(game_keys[50*client_secondsRecieved+p],
				(char*)argv[p+2], 8);
		}

		/* ess */
		for (p=0; p<6; p++)
			for (q=0; q<4; q++)
			{
				sscanf((char*)argv[52+p*20 + q*5], "%d",
					&client_ess[sec].ball[p][q].health);

				sscanf((char*)argv[52+p*20 + q*5 + 1],
					"%d", &tmp);
				memcpy(&client_ess[sec].ball[p][q].pos.x,
					&tmp, 4);

				sscanf((char*)argv[52+p*20 + q*5 + 2],
					"%d", &tmp);
				memcpy(&client_ess[sec].ball[p][q].pos.y,
					&tmp, 4);

				sscanf((char*)argv[52+p*20 + q*5 + 3],
					"%d", &tmp);
				memcpy(&client_ess[sec].ball[p][q].vel.x,
					&tmp, 4);

				sscanf((char*)argv[52+p*20 + q*5 + 4],
					"%d", &tmp);
				memcpy(&client_ess[sec].ball[p][q].vel.y,
					&tmp, 4);
			}
		for (p=0; p<MAXCLUSTERS; p++)
		{
			sscanf((char*)argv[172+p*8], "%d",
				&client_ess[sec].weap[p].status);

			sscanf((char*)argv[172+p*8 + 1], "%d",
				&client_ess[sec].weap[p].boom);

			sscanf((char*)argv[172+p*8 + 2], "%d",
				&client_ess[sec].weap[p].fuse);

			sscanf((char*)argv[172+p*8 + 3], "%d",
				(int*)&client_ess[sec].weap[p].side);
			
			sscanf((char*)argv[172+p*8 + 4], "%d", &tmp);
			memcpy(&client_ess[sec].weap[p].pos.x, &tmp, 4);

			sscanf((char*)argv[172+p*8 + 5], "%d", &tmp);
			memcpy(&client_ess[sec].weap[p].pos.y, &tmp, 4);
			
			sscanf((char*)argv[172+p*8 + 6], "%d", &tmp);
			memcpy(&client_ess[sec].weap[p].vel.x, &tmp, 4);

			sscanf((char*)argv[172+p*8 + 7], "%d", &tmp);
			memcpy(&client_ess[sec].weap[p].vel.y, &tmp, 4);
		}

		sscanf((char*)argv[212], "%d", (int*)&client_ess[sec].side);

		sscanf((char*)argv[213], "%d", &tmp);
		memcpy(&client_ess[sec].aim, &tmp, 4);

		sscanf((char*)argv[214], "%d", (int*)&client_ess[sec].rope);

		sscanf((char*)argv[215], "%d", &tmp);
		memcpy(&client_ess[sec].ropeang, &tmp, 4);

		sscanf((char*)argv[216], "%d", &tmp);
		memcpy(&client_ess[sec].ropelen, &tmp, 4);

		sscanf((char*)argv[217], "%d", (int*)&client_ess[sec].hooked);

		sscanf((char*)argv[218], "%d", &tmp);
		memcpy(&client_ess[sec].hookPos.x, &tmp, 4);

		sscanf((char*)argv[219], "%d", &tmp);
		memcpy(&client_ess[sec].hookPos.y, &tmp, 4);

		sscanf((char*)argv[220], "%d", (int*)&client_ess[sec].roperep);

		sscanf((char*)argv[221], "%d", &tmp);
		memcpy(&client_ess[sec].omega, &tmp, 4);

		sscanf((char*)argv[222], "%d", (int*)&client_ess[sec].drop);

		sscanf((char*)argv[223], "%d", &client_ess[sec].walls2go);

		client_secondsRecieved++;
		return 0;
	}
	else if (strcmp((char*)argv[0], PACKET_RCON_WRONG_PASSWORD)==0
		&& (argc == 1))
	{
		console_print("Remote console password is incorrect!");
		return 0;
	}
	else /* unknown packet type */
	{
		error("Protocol incompability.");
		return 1;
	}
	return 0;
}

/* client's loop. Connect to server and recieve data */
static int clientLoop(void *ptr)
{
	IPaddress ip;

	if (!client_local)
	{
		console_print("  client: connecting...");
		console_messagePrint("Connecting...");
	}

	if (client_local)
	{
		if (SDLNet_ResolveHost(&ip, "localhost", 22200) == -1)
		{
			error("couldn't resolve hostname.");
			return 0;
		}
	}
	else
	{
		if (SDLNet_ResolveHost(&ip, commands_cl_hostname,
			commands_cl_port) == -1)
		{
			error("couldn't resolve hostname.");
			return 0;
		}
	}

	tcpsock = SDLNet_TCP_Open(&ip);
	if (!tcpsock)
	{
		error("couldn't connect.");
		return 0;
	}
	
	sendHello();

	if (recievePacket())
		return 0;
	
	if (processServerState())
		return 0;

	client_map = map_load(mapname, 0);
	if (!client_map)
	{
		char buffer[256];
		snprintf(buffer, 256, "couldn't load map: %s.", mapname);
		error(buffer);
		return 0;
	}

	sendJoin();

	if (recievePacket())
		return 0;
	
	if (processWelcome())
		return 0;

	if (!client_local)
	{
		console_print("  client: connected to %s.",
			commands_cl_hostname);
		console_messagePrint("Succesfully connected to %s",
			commands_cl_hostname);
	}
	client_connected = 1;

	/* recieve and process until thread is terminated */
	for (;;)
	{
		if (recievePacket())
			return 0;
		if (processPacket())
			return 1;
	}
	return 0;
}

void client_loadTextures()
{
	client_texArrow = textures_loadBMPAlpha("graphics/game/arrow.bmp");

	client_texZookaLabel =
		textures_loadBMP("graphics/game/zooka.bmp");
	client_texGrenadeLabel =
		textures_loadBMP("graphics/game/grenade.bmp");
	client_texDynamiteLabel =
		textures_loadBMP("graphics/game/dynamite.bmp");
	client_texClusterLabel =
		textures_loadBMP("graphics/game/cluster.bmp");
	client_texKillerBeeLabel =
		textures_loadBMP("graphics/game/killerbee.bmp");
	client_texHamstersLabel =
		textures_loadBMP("graphics/game/hamsters.bmp");
	client_texUp = textures_loadBMP("graphics/game/up.bmp");
	client_texDown = textures_loadBMP("graphics/game/down.bmp");
	client_texLeft = textures_loadBMP("graphics/game/left.bmp");
	client_texRight = textures_loadBMP("graphics/game/right.bmp");
	client_texRopeShoot = textures_loadBMP("graphics/game/ropeshoot.bmp");
	client_texLag = textures_loadBMP("graphics/game/lag.bmp");

	client_texZooka =
		textures_loadBMPWithAlpha("graphics/game/weapons/zooka.bmp",
		"graphics/game/weapons/zooka_.bmp");
	client_texGrenade =
		textures_loadBMPWithAlpha("graphics/game/weapons/grenade.bmp",
		"graphics/game/weapons/grenade_.bmp");
	client_texDynamite =
		textures_loadBMPWithAlpha("graphics/game/weapons/dynamite.bmp",
		"graphics/game/weapons/dynamite_.bmp");
	client_texCluster =
		textures_loadBMPWithAlpha("graphics/game/weapons/cluster.bmp",
		"graphics/game/weapons/cluster_.bmp");
	client_texCluster2 =
		textures_loadBMPWithAlpha("graphics/game/weapons/cluster2.bmp",
		"graphics/game/weapons/cluster2_.bmp");
	client_texHamster =
		textures_loadBMPWithAlpha("graphics/game/weapons/hamster.bmp",
		"graphics/game/weapons/hamster_.bmp");
	client_texKillerBee =
		textures_loadBMPWithAlpha("graphics/game/weapons/killerbee.bmp",
		"graphics/game/weapons/killerbee_.bmp");

	client_texBall0 =
		textures_loadBMPWithAlpha("graphics/game/balls/ball0.bmp",
		"graphics/game/balls/ball_.bmp");
	client_texBall1 =
		textures_loadBMPWithAlpha("graphics/game/balls/ball1.bmp",
		"graphics/game/balls/ball_.bmp");
	client_texBall2 =
		textures_loadBMPWithAlpha("graphics/game/balls/ball2.bmp",
		"graphics/game/balls/ball_.bmp");
	client_texBall3 =
		textures_loadBMPWithAlpha("graphics/game/balls/ball3.bmp",
		"graphics/game/balls/ball_.bmp");
	client_texBall4 =
		textures_loadBMPWithAlpha("graphics/game/balls/ball4.bmp",
		"graphics/game/balls/ball_.bmp");
	client_texBall5 =
		textures_loadBMPWithAlpha("graphics/game/balls/ball5.bmp",
		"graphics/game/balls/ball_.bmp");
	client_texBox =
		textures_loadBMPWithAlpha("graphics/game/box.bmp",
		"graphics/game/box_.bmp");
	client_texBack = textures_loadBMP("graphics/game/world/back.bmp");

	return;
}

void client_unloadTextures()
{
	textures_unloadBMP(&client_texArrow);
	textures_unloadBMP(&client_texZookaLabel);
	textures_unloadBMP(&client_texGrenadeLabel);
	textures_unloadBMP(&client_texDynamiteLabel);
	textures_unloadBMP(&client_texClusterLabel);
	textures_unloadBMP(&client_texKillerBeeLabel);
	textures_unloadBMP(&client_texHamstersLabel);
	textures_unloadBMP(&client_texUp);
	textures_unloadBMP(&client_texDown);
	textures_unloadBMP(&client_texLeft);
	textures_unloadBMP(&client_texRight);
	textures_unloadBMP(&client_texRopeShoot);
	textures_unloadBMP(&client_texLag);

	textures_unloadBMP(&client_texZooka);
	textures_unloadBMP(&client_texGrenade);
	textures_unloadBMP(&client_texDynamite);
	textures_unloadBMP(&client_texCluster);
	textures_unloadBMP(&client_texCluster2);
	textures_unloadBMP(&client_texHamster);
	textures_unloadBMP(&client_texKillerBee);

	textures_unloadBMP(&client_texBall0);
	textures_unloadBMP(&client_texBall1);
	textures_unloadBMP(&client_texBall2);
	textures_unloadBMP(&client_texBall3);
	textures_unloadBMP(&client_texBall4);
	textures_unloadBMP(&client_texBall5);

	textures_unloadBMP(&client_texBox);

	textures_unloadBMP(&client_texBack);
	return;
}

int client_init()
{
	int p, q;

	console_print("Initializing client...");
	client_connected = 0;

	client_rcCount = 0;

	client_exitConfirm = 0;

	client_cameraFollow = 0;
	client_cameraFocus = 1;
	console_messagesClear();
	input_mouse.x = 0.0f;
	input_mouse.y = 0.0f;

	client_isError = 0;
	tcpsock = NULL;
	memset(client_players, 0, sizeof(ClientPlayer)*6);

	thread = SDL_CreateThread(clientLoop, NULL);
	if (thread == NULL)
	{
		console_print("^1FAILED!!!");
		return 1;
	}

	client_loadTextures();
	
	client_sndColision1 = audio_loadWAV("sounds/colision1.wav");
	client_sndColision2 = audio_loadWAV("sounds/colision2.wav");
	client_sndColision3 = audio_loadWAV("sounds/colision3.wav");
	client_sndRope1 = audio_loadWAV("sounds/rope1.wav");
	client_sndRope2 = audio_loadWAV("sounds/rope2.wav");
	client_sndHamster1 = audio_loadWAV("sounds/hamster1.wav");
	client_sndHamster2 = audio_loadWAV("sounds/hamster2.wav");
	client_sndHamster3 = audio_loadWAV("sounds/hamster3.wav");
	client_sndHamster4 = audio_loadWAV("sounds/hamster4.wav");
	client_sndBoom = audio_loadWAV("sounds/boom.wav");
	client_sndJump = audio_loadWAV("sounds/jump.wav");
	client_sndThrow = audio_loadWAV("sounds/throw.wav");
	client_sndWater = audio_loadWAV("sounds/water.wav");
	sndChat = audio_loadWAV("sounds/chat.wav");
	sndCow = audio_loadWAV("sounds/moo.wav");
	sndOneAndOnly = audio_loadWAV("sounds/oneandonly.wav");

	for (p=0; p<6; p++)
		for (q=0; q<4; q++)
			client_players[p].ball[q].status = 0;

	client_doTheJob = 0;
	client_playTheJob = 0;
	ui_clientRunning = 1;
	SDL_EnableKeyRepeat(0, 0);
	console_print("^2DONE");
	return 0;
}

void client_cleanup()
{
	console_print("Cleaning client...");
	client_connected = 0;

	if (tcpsock)
	{
		SDLNet_TCP_Close(tcpsock);
		tcpsock = NULL;
	}
	client_unloadTextures();
	audio_unloadWAV(client_sndColision1);
	audio_unloadWAV(client_sndColision2);
	audio_unloadWAV(client_sndColision3);
	audio_unloadWAV(client_sndRope1);
	audio_unloadWAV(client_sndRope2);
	audio_unloadWAV(client_sndHamster1);
	audio_unloadWAV(client_sndHamster2);
	audio_unloadWAV(client_sndHamster3);
	audio_unloadWAV(client_sndHamster4);
	audio_unloadWAV(client_sndBoom);
	audio_unloadWAV(client_sndJump);
	audio_unloadWAV(client_sndThrow);
	audio_unloadWAV(client_sndWater);
	audio_unloadWAV(sndChat);
	audio_unloadWAV(sndCow);
	audio_unloadWAV(sndOneAndOnly);

	ui_clientRunning = 0;
	ui_place = 0;

	client_isError = 0;

	if (client_map)
	{
		map_unload(client_map);
		client_map = NULL;
	}

	console_print("^2DONE");
	if (thread)
	{
		SDL_KillThread(thread);
		thread = NULL;
	}

	SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY,
		SDL_DEFAULT_REPEAT_INTERVAL);
	return;
}

/* handle mouse click event */
void client_click(float x, float y)
{
	if (client_isError)
		client_cleanup();
	return;
}

/* handle keyboard event */
void client_keyDown(SDLKey key)
{
	if (client_isError)
		client_cleanup();
	
	if (client_exitConfirm)
	{
		if (key == SDLK_RETURN)
			client_cleanup();
		else if (key == SDLK_ESCAPE)
			client_exitConfirm = 0;

		return;
	}

	if (key == SDLK_ESCAPE)
		client_exitConfirm = 1;
	
	if (key == 't')
	{
		console_chatterActive = 1;
		SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY,
			SDL_DEFAULT_REPEAT_INTERVAL);
	}

	if (!client_gameInProgress)
	{
		if (key == SDLK_F1)
			client_ingame();
		else if (key == SDLK_F2)
			client_spectator();
		else if (key == SDLK_F3)
			client_ready();
		else if (key == SDLK_F4)
			client_unready();
	}
	if (game_prepare)
		game_prepare = 0;
	if (game_drop && (key == SDLK_RSHIFT || key == SDLK_LSHIFT))
		client_cameraFocus = 2;

	return;
}

/* send packet SAY (send text messages to other players) */
void client_say(char *msg)
{
	char buf[BUFLEN];

	if (client_isError || !client_connected)
		return;

	memset(buf, 0, BUFLEN);
	sprintf(&buf[strlen(buf)], "%s\r\n", PACKET_SAY);
	sprintf(&buf[strlen(buf)], "%s\r\n\r\n", msg);

	SDLNet_TCP_Send(tcpsock, buf, strlen(buf));
	return;
}

/* send packet NAME (change name during the game) */
void client_name(char *msg)
{
	char buf[BUFLEN];

	if (client_isError || !client_connected)
		return;

	memset(buf, 0, BUFLEN);
	sprintf(&buf[strlen(buf)], "%s\r\n", PACKET_NAME);
	sprintf(&buf[strlen(buf)], "%s\r\n\r\n", msg);

	SDLNet_TCP_Send(tcpsock, buf, strlen(buf));
	return;
}

/* send packet INGAME (join to game) */
void client_ingame()
{
	char buf[BUFLEN];

	if (client_isError || !client_connected)
		return;
	
	memset(buf, 0, BUFLEN);
	sprintf(&buf[strlen(buf)], "%s\r\n\r\n", PACKET_INGAME);

	SDLNet_TCP_Send(tcpsock, buf, strlen(buf));
	return;
}

/* send packet SPECTATOR (join spectators) */
void client_spectator()
{
	char buf[BUFLEN];

	if (client_isError || !client_connected)
		return;
	
	memset(buf, 0, BUFLEN);
	sprintf(&buf[strlen(buf)], "%s\r\n\r\n", PACKET_SPECTATOR);

	SDLNet_TCP_Send(tcpsock, buf, strlen(buf));
	return;
}

/* send READY packet (become ready) */
void client_ready()
{
	char buf[BUFLEN];

	if (client_isError || !client_connected)
		return;
	
	memset(buf, 0, BUFLEN);
	sprintf(&buf[strlen(buf)], "%s\r\n\r\n", PACKET_READY);

	SDLNet_TCP_Send(tcpsock, buf, strlen(buf));
	return;
}

/* send UNREADY packet (become unready) */
void client_unready()
{
	char buf[BUFLEN];

	if (client_isError || !client_connected)
		return;
	
	memset(buf, 0, BUFLEN);
	sprintf(&buf[strlen(buf)], "%s\r\n\r\n", PACKET_UNREADY);

	SDLNet_TCP_Send(tcpsock, buf, strlen(buf));
	return;
}

/* send SECOND packet (tell server about what we have done during the last
 * second) */
void client_second(int second)
{
	int p;
	char buf[BUFLEN];

	if (client_isError || !client_connected)
		return;

	memset(buf, 0, BUFLEN);
	sprintf(&buf[strlen(buf)], PACKET_SECOND);
	sprintf(&buf[strlen(buf)], "\r\n");
	sprintf(&buf[strlen(buf)], "%d", second);
	sprintf(&buf[strlen(buf)], "\r\n");
	for (p=0; p<50; p++)
	{
		sprintf(&buf[strlen(buf)], game_keys[50*second+p]);
		sprintf(&buf[strlen(buf)], "\r\n");
	}
	SDLNet_TCP_Send(tcpsock, buf, strlen(buf));

	memset(buf, 0, BUFLEN);
	for (p=0; p<6; p++)
	{
		int q;

		for (q=0; q<4; q++)
		{
			int tmp;
			sprintf(&buf[strlen(buf)], "%d\r\n",
				client_players[p].ball[q].health);

			memcpy(&tmp, &(client_players[p].ball[q].pos.x), 4);
			sprintf(&buf[strlen(buf)], "%d\r\n", tmp);
			memcpy(&tmp, &(client_players[p].ball[q].pos.y), 4);
			sprintf(&buf[strlen(buf)], "%d\r\n", tmp);
			memcpy(&tmp, &(client_players[p].ball[q].vel.x), 4);
			sprintf(&buf[strlen(buf)], "%d\r\n", tmp);
			memcpy(&tmp, &(client_players[p].ball[q].vel.y), 4);
			sprintf(&buf[strlen(buf)], "%d\r\n", tmp);
		}
		
	}
	for (p=0; p<MAXCLUSTERS; p++)
	{
		int tmp;
		sprintf(&buf[strlen(buf)], "%d\r\n",
			game_weap[p].status);
		sprintf(&buf[strlen(buf)], "%d\r\n",
			game_weap[p].boom);
		sprintf(&buf[strlen(buf)], "%d\r\n",
			game_weap[p].fuse);
		sprintf(&buf[strlen(buf)], "%d\r\n",
			(int)game_weap[p].side);

		memcpy(&tmp, &game_weap[p].pos.x, 4);
		sprintf(&buf[strlen(buf)], "%d\r\n", tmp);
		memcpy(&tmp, &game_weap[p].pos.y, 4);
		sprintf(&buf[strlen(buf)], "%d\r\n", tmp);
		memcpy(&tmp, &game_weap[p].vel.x, 4);
		sprintf(&buf[strlen(buf)], "%d\r\n", tmp);
		memcpy(&tmp, &game_weap[p].vel.y, 4);
		sprintf(&buf[strlen(buf)], "%d\r\n", tmp);
		
	}

	{
		int tmp;

		sprintf(&buf[strlen(buf)], "%d\r\n", (int)game_side);

		memcpy(&tmp, &game_aim, 4);
		sprintf(&buf[strlen(buf)], "%d\r\n", tmp);

		sprintf(&buf[strlen(buf)], "%d\r\n", (int)game_rope);

		memcpy(&tmp, &game_ropeang, 4);
		sprintf(&buf[strlen(buf)], "%d\r\n", tmp);

		memcpy(&tmp, &game_ropelen, 4);
		sprintf(&buf[strlen(buf)], "%d\r\n", tmp);

		sprintf(&buf[strlen(buf)], "%d\r\n", (int)game_hooked);

		memcpy(&tmp, &game_hookPos.x, 4);
		sprintf(&buf[strlen(buf)], "%d\r\n", tmp);

		memcpy(&tmp, &game_hookPos.y, 4);
		sprintf(&buf[strlen(buf)], "%d\r\n", tmp);

		sprintf(&buf[strlen(buf)], "%d\r\n", (int)game_roperep);

		memcpy(&tmp, &game_omega, 4);
		sprintf(&buf[strlen(buf)], "%d\r\n", tmp);

		sprintf(&buf[strlen(buf)], "%d\r\n", (int)game_drop);

		sprintf(&buf[strlen(buf)], "%d\r\n", game_walls2go);

	}

	sprintf(&buf[strlen(buf)], "\r\n");

	SDLNet_TCP_Send(tcpsock, buf, strlen(buf));
	return;
}

/* send SYNC packet (after end of our turn) */
void client_sync()
{
	int p;
	char buf[BUFLEN];

	if (client_isError || !client_connected)
		return;
	
	sprintf(buf, PACKET_SYNC);
	sprintf(&buf[strlen(buf)], "\r\n");
	for (p=0; p<6; p++)
	{
		int q;

		for (q=0; q<4; q++)
		{
			int tmp;
			sprintf(&buf[strlen(buf)], "%d\r\n",
				client_players[p].ball[q].health);

			memcpy(&tmp, &(client_players[p].ball[q].pos.x), 4);
			sprintf(&buf[strlen(buf)], "%d\r\n", tmp);
			memcpy(&tmp, &(client_players[p].ball[q].pos.y), 4);
			sprintf(&buf[strlen(buf)], "%d\r\n", tmp);
		}
		
	}

	sprintf(&buf[strlen(buf)], "\r\n");

	SDLNet_TCP_Send(tcpsock, buf, strlen(buf));
	return;
}

void client_rcQueue(char *msg)
{
	if (client_isError || !client_connected || client_local)
	{
		console_print("You must be connected to dedicated server "
			"to use remote console");
		return;
	}

	if (client_rcCount >= 20)
	{
		console_print("You can queue only 20 commands.");
		return;
	}

	sprintf(client_rcBuf[client_rcCount], "%s", msg);
	client_rcCount++;

	console_print("Job %d queued. Use rc_restart to apply changes.",
		client_rcCount);
	return;
}

void client_rcRestart()
{
	char buf[BUFLEN];
	int p;

	if (client_isError || !client_connected || client_local)
	{
		console_print("You must be connected to dedicated server "
			"to use remote console");
		return;
	}
	if (commands_rc_clpasswd[0] == 0)
	{
		console_print("Set your remote console password with "
			"rc_clpasswd");
		return;
	}

	memset(buf, 0, BUFLEN);
	sprintf(&buf[strlen(buf)], "%s\r\n", PACKET_RCON_RESTART);
	sprintf(&buf[strlen(buf)], "%s\r\n", commands_rc_clpasswd);
	for (p=0; p<client_rcCount; p++)
		sprintf(&buf[strlen(buf)], "%s\r\n", client_rcBuf[p]);
	sprintf(&buf[strlen(buf)], "\r\n");

	SDLNet_TCP_Send(tcpsock, buf, strlen(buf));
	client_rcCount = 0;
	return;
}

void client_rcKick(char *msg)
{
	char buf[BUFLEN];
	int index;

	if (client_isError || !client_connected || client_local)
	{
		console_print("You must be connected to dedicated server "
			"to use remote console");
		return;
	}
	if (commands_rc_clpasswd[0] == 0)
	{
		console_print("Set your remote console password with "
			"rc_clpasswd");
		return;
	}

	if (!strcmp(msg, "red"))
		index = 0;
	else if (!strcmp(msg, "blue"))
		index = 1;
	else if (!strcmp(msg, "green"))
		index = 2;
	else if (!strcmp(msg, "yellow"))
		index = 3;
	else if (!strcmp(msg, "magenta"))
		index = 4;
	else if (!strcmp(msg, "cyan"))
		index = 5;
	else
	{
		console_print("No such color!");
		return;
	}

	memset(buf, 0, BUFLEN);
	sprintf(&buf[strlen(buf)], "%s\r\n", PACKET_RCON_KICK);
	sprintf(&buf[strlen(buf)], "%s\r\n", commands_rc_clpasswd);
	sprintf(&buf[strlen(buf)], "%d\r\n\r\n", index);

	SDLNet_TCP_Send(tcpsock, buf, strlen(buf));
	return;
	
}
