/*
 *  (C) Copyright 2007-2009 by FreeShoppa Development Team
 *                        (http://chomzee.ethernet.pl/freeshoppa/)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <GL/gl.h>
#ifdef WIN32
#include <gl\glext.h>
#endif

#include "freeshoppa.h"

/* text messages used in the game */
static const char txtInGame[] = "In game:";
static const char txtSpectators[] = "Spectators:";
static const char txtReady[] = "^4READY";
static const char txtUnready[] = "^1NOT READY";
static const char txtColor[] = "Color:";
static const char txtAskReady[] = "Ready?";
static const char txtExitConfirm[] = "^4Are you sure you want to quit?";
static const char txtExitConfirm2[] = " ^2Enter = Yes      ^1Escape = No";

/* draw line of text (can use colors) */
static void printLine(int p, float h, char *txt)
{
	int q, s;
	s = 0;
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	for (q=0; q<strlen(txt); q++)
	{
		/* white */
		if (txt[q] == '^' && txt[q+1] == '0')
		{
			q++;
			glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
			continue;
		}
		/* red */
		if (txt[q] == '^' && txt[q+1] == '1')
		{
			q++;
			glColor4f(1.0f, 0.0f, 0.0f, 1.0f);
			continue;
		}
		/* green */
		if (txt[q] == '^' && txt[q+1] == '2')
		{
			q++;
			glColor4f(0.0f, 1.0f, 0.0f, 1.0f);
			continue;
		}
		/* blue */
		if (txt[q] == '^' && txt[q+1] == '3')
		{
			q++;
			glColor4f(0.0f, 0.0f, 1.0f, 1.0f);
			continue;
		}
		/* yellow */
		if (txt[q] == '^' && txt[q+1] == '4')
		{
			q++;
			glColor4f(1.0f, 1.0f, 0.0f, 1.0f);
			continue;
		}
		/* cyan */
		if (txt[q] == '^' && txt[q+1] == '5')
		{
			q++;
			glColor4f(0.0f, 1.0f, 1.0f, 1.0f);
			continue;
		}
		/* magenta */
		if (txt[q] == '^' && txt[q+1] == '6')
		{
			q++;
			glColor4f(1.0f, 0.0f, 1.0f, 1.0f);
			continue;
		}
		/* black */
		if (txt[q] == '^' && txt[q+1] == '7')
		{
			q++;
			glColor4f(0.0f, 0.0f, 0.0f, 1.0f);
			continue;
		}

		fonts_drawChar(console_texFont, txt[q],
				h + s*10.0f, h + 10.0f + s*10.0f,
				485.0f-p*20, 500.0f-p*20);
		s++;
	}

	return;
}

static void drawWorld(void)
{
	int p;

	for (p=0; p<(client_map->elementsCount); p++)
	{
		if (client_map->elements[p].type == 0)
		{
			glColor3f(1.0f, 1.0f, 1.0f);
			glBegin(GL_LINES);
			glVertex2f((float)client_map->elements[p].arg[0],
				   (float)client_map->elements[p].arg[1]);
			glVertex2f((float)client_map->elements[p].arg[2],
				   (float)client_map->elements[p].arg[3]);
			glEnd();
		}
		if (client_map->elements[p].type == 6) /* wxw walls */
		{
			if (client_gametype != 2)
				glColor3f(1.0f, 1.0f, 1.0f);
			else if (game_walls[p])
				glColor3f(1.0f, 0.0f, 0.0f);
			else
				glColor3f(0.0f, 1.0f, 0.0f);
			glBegin(GL_LINES);
			glVertex2f((float)client_map->elements[p].arg[0],
				   (float)client_map->elements[p].arg[1]);
			glVertex2f((float)client_map->elements[p].arg[2],
				   (float)client_map->elements[p].arg[3]);
			glEnd();
		}
		/*
		 * Draw Spawn points...
		else if (client_map->elements[p].type == 1)
		{
			glColor4f(0.0f, 0.0f, 0.0f, 0.25f);
			glBegin(GL_TRIANGLE_FAN);
			glVertex2f((float)client_map->elements[p].arg[0]-10.0f,
				   (float)client_map->elements[p].arg[1]+10.0f);
			glVertex2f((float)client_map->elements[p].arg[0]-10.0f,
				   (float)client_map->elements[p].arg[1]-10.0f);
			glVertex2f((float)client_map->elements[p].arg[0]+10.0f,
				   (float)client_map->elements[p].arg[1]-10.0f);
			glVertex2f((float)client_map->elements[p].arg[0]+10.0f,
				   (float)client_map->elements[p].arg[1]+10.0f);
			glEnd();
		}
		*/
		else if (client_map->elements[p].type == 3
			&& client_gametype == 4)
		{
			glColor4f(0.75f, 0.0f, 0.0f, 0.25f);
			glBegin(GL_TRIANGLE_FAN);
			glVertex2f((float)client_map->elements[p].arg[0],
				   (float)client_map->elements[p].arg[3]);
			glVertex2f((float)client_map->elements[p].arg[0],
				   (float)client_map->elements[p].arg[1]);
			glVertex2f((float)client_map->elements[p].arg[2],
				   (float)client_map->elements[p].arg[1]);
			glVertex2f((float)client_map->elements[p].arg[2],
				   (float)client_map->elements[p].arg[3]);
			glEnd();
		}
	}

	return;
}

static void drawBack()
{
	float p;
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	glEnable(GL_TEXTURE_2D);

	glBindTexture(GL_TEXTURE_2D, client_texBack);
	glBegin(GL_TRIANGLES);

	/*---*/
	for (p=0.0f; p<client_map->xMax+video_xOffset+400.0f; p+=800.0f)
	{
		glTexCoord2f(0.0f, 0.99f);
		glVertex2f(p, 500.0f);
		glTexCoord2f(0.0f, 0.25f);
		glVertex2f(p, -100.0f);
		glTexCoord2f(1.0f, 0.25f);
		glVertex2f(p+800.0f, -100.0f);

		glTexCoord2f(0.0f, 0.99f);
		glVertex2f(p, 500.0f);
		glTexCoord2f(1.0f, 0.25f);
		glVertex2f(p+800.0f, -100.0f);
		glTexCoord2f(1.0f, 0.99f);
		glVertex2f(p+800.0f, 500.0f);
	}
	for (p=0.0f; p>client_map->xMin-video_xOffset-400.0f; p-=800.0f)
	{
		glTexCoord2f(0.0f, 0.99f);
		glVertex2f(p-800.0f, 500.0f);
		glTexCoord2f(0.0f, 0.25f);
		glVertex2f(p-800.0f, -100.0f);
		glTexCoord2f(1.0f, 0.25f);
		glVertex2f(p, -100.0f);

		glTexCoord2f(0.0f, 0.99f);
		glVertex2f(p-800.0f, 500.0f);
		glTexCoord2f(1.0f, 0.25f);
		glVertex2f(p, -100.0f);
		glTexCoord2f(1.0f, 0.99f);
		glVertex2f(p, 500.0f);
	}

	glEnd();
	glDisable(GL_TEXTURE_2D);

	return;
}

static void drawWater()
{
	int p;
	static int time = 0;

	glColor4f(0.118f, 0.078f, 0.114f, 1.0f);

	glBegin(GL_TRIANGLE_FAN);
	glVertex2f(client_map->xMin-(400.0f+video_xOffset), -5.0f);
	glVertex2f(client_map->xMin-(400.0f+video_xOffset),
		-300.0f-video_yOffset);
	glVertex2f(client_map->xMax+(400.0f+video_xOffset),
		-300.0f-video_yOffset);
	glVertex2f(client_map->xMax+(400.0f+video_xOffset), -5.0f);
	glEnd();

	if (!commands_cl_water)
		return;

	/* draw the waves */
	glBegin(GL_TRIANGLE_STRIP);
	for (p=0; client_map->xMin-(400.0f+video_xOffset)+8.0*p <=
		client_map->xMax+(400.0f+video_xOffset); p++)
	{
		glColor3f(0.359f, 0.218f, 0.316f);
		glVertex2f(client_map->xMin-(400.0f+video_xOffset)+8.0*p,
			5.0f*cos((double)time*0.2 + (double)p*0.2));
		glColor4f(0.118f, 0.078f, 0.114f, 1.0f);
		glVertex2f(client_map->xMin-(400.0f+video_xOffset)+8.0*p,
			5.0f*cos((double)time*0.2 + (double)p*0.2)-25.0);
	}
	glEnd();
	time++;

	return;
}

/* draw disc */
/*static void drawBall(int details, float radius)
{
	float p;
	int q;
	glBegin(GL_TRIANGLE_FAN);

	glVertex2f(0.0f, 0.0f);
	p = 0.0f;
	for (q=0; q<=details; q++)
	{
		glVertex2f(radius*cos(p), radius*sin(p));
		p += PI/((float)details/2.0f);
	}

	glEnd();

	return;
}
*/

/* draw simple explosion */
static void drawBoom(int details, float radius, int w)
{
	float p;
	int q;
	glBegin(GL_TRIANGLE_FAN);

	glColor4f(1.0f, 1.0f, 1.0f,
		(1.0f-(float)game_weap[w].boom/50.0f)/2.0f);
	glVertex2f(0.0f, 0.0f);
	glColor4f(1.0f, 1.0f, 1.0f, 0.0f);
	p = 0.0f;
	for (q=0; q<=details; q++)
	{
		glVertex2f(radius*cos(p), radius*sin(p));
		p += PI/((float)details/2.0f);
	}

	glEnd();
	
}

static void drawLabel(int p, int q, int health)
{
	glColor4f(0.0f, 0.0f, 0.0f, 1.0f);

	glBegin(GL_TRIANGLES);

	glVertex2f(-30.0f, 35.0f);
	glVertex2f(-30.0f, 20.0f);
	glVertex2f(30.0f, 20.0f);

	glVertex2f(-30.0f, 35.0f);
	glVertex2f(30.0f, 20.0f);
	glVertex2f(30.0f, 35.0f);

	glEnd();

	if (p == 0)
		glColor4f(1.0f, 0.0f, 0.0f, 1.0f);
	else if (p==1)
		glColor4f(0.0f, 0.0f, 1.0f, 1.0f);
	else if (p==2)
		glColor4f(0.0f, 1.0f, 0.0f, 1.0f);
	else if (p==3)
		glColor4f(1.0f, 1.0f, 0.0f, 1.0f);
	else if (p==4)
		glColor4f(1.0f, 0.0f, 1.0f, 1.0f);
	else if (p==5)
		glColor4f(0.0f, 1.0f, 1.0f, 1.0f);
	
	glBegin(GL_LINES);

	glVertex2f(-30.0f, 35.0f);
	glVertex2f(30.0f, 35.0f);

	glVertex2f(-30.0f, 20.0f);
	glVertex2f(30.0f, 20.0f);

	glVertex2f(-30.0f, 20.0f);
	glVertex2f(-30.0f, 35.0f);

	glVertex2f(30.0f, 20.0f);
	glVertex2f(30.0f, 35.0f);

	glEnd();
	
	glEnable(GL_TEXTURE_2D);
	fonts_drawChar(console_texFont, '#',
		-30.0f, -20.0f,
		20.0f, 35.0f);
	fonts_drawChar(console_texFont, (char)(q+49),
		-20.0f, -10.0f,
		20.0f, 35.0f);

	fonts_drawChar(console_texFont, (health/100)%10 + 48,
		0.0f, 10.0f,
		20.0f, 35.0f);
	fonts_drawChar(console_texFont, (health/10)%10 + 48,
		10.0f, 20.0f,
		20.0f, 35.0f);
	fonts_drawChar(console_texFont, health%10 + 48,
		20.0f, 30.0f,
		20.0f, 35.0f);
	glDisable(GL_TEXTURE_2D);

	return;
}

static void drawFuseTime(int w)
{
	glColor4f(0.0f, 0.0f, 0.0f, 1.0f);

	glBegin(GL_TRIANGLES);

	glVertex2f(-20.0f, 35.0f);
	glVertex2f(-20.0f, 20.0f);
	glVertex2f(20.0f, 20.0f);

	glVertex2f(-20.0f, 35.0f);
	glVertex2f(20.0f, 20.0f);
	glVertex2f(20.0f, 35.0f);

	glEnd();

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	
	glBegin(GL_LINES);

	glVertex2f(-20.0f, 35.0f);
	glVertex2f(20.0f, 35.0f);

	glVertex2f(-20.0f, 20.0f);
	glVertex2f(20.0f, 20.0f);

	glVertex2f(-20.0f, 20.0f);
	glVertex2f(-20.0f, 35.0f);

	glVertex2f(20.0f, 20.0f);
	glVertex2f(20.0f, 35.0f);

	glEnd();


	glEnable(GL_TEXTURE_2D);
	fonts_drawChar(console_texFont, (game_weap[w].fuse/50%10)+48,
		-20.0f, -10.0f,
		20.0f, 35.0f);

	fonts_drawChar(console_texFont, '.',
		-10.0f, 0.0f,
		20.0f, 35.0f);
	fonts_drawChar(console_texFont, (game_weap[w].fuse%50*2/10)+48,
		0.0f, 10.0f,
		20.0f, 35.0f);
	fonts_drawChar(console_texFont, (game_weap[w].fuse%50*2%10)+48,
		10.0f, 20.0f,
		20.0f, 35.0f);
	glDisable(GL_TEXTURE_2D);

	return;
}

/* "It's our turn" arrow */
static void drawArrow(void)
{
	glColor4f(0.8f, 0.8f, 0.0f, 1.0f);
	
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, client_texArrow);

	glBegin(GL_TRIANGLES);

	glTexCoord2f(0.0f, 1.0f);
	glVertex2f(-20.0f, 80.0f);
	glTexCoord2f(0.0f, 0.0f);
	glVertex2f(-20.0f, 40.0f);
	glTexCoord2f(1.0f, 0.0f);
	glVertex2f(20.0f, 40.0f);

	glTexCoord2f(0.0f, 1.0f);
	glVertex2f(-20.0f, 80.0f);
	glTexCoord2f(1.0f, 0.0f);
	glVertex2f(20.0f, 40.0f);
	glTexCoord2f(1.0f, 1.0f);
	glVertex2f(20.0f, 80.0f);

	glEnd();
	glDisable(GL_TEXTURE_2D);
	return;
}

static void drawWeaponLabel(void)
{
	glPushMatrix();
	glTranslatef(-video_xOffset, video_yOffset, 0.0f);
	if (console_chatterActive)
		glTranslatef(0.0f, -15.0f, 0.0f);

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	
	glEnable(GL_TEXTURE_2D);
	if (game_dropType == 0)
		glBindTexture(GL_TEXTURE_2D, client_texZookaLabel);
	else if (game_dropType == 1)
		glBindTexture(GL_TEXTURE_2D, client_texGrenadeLabel);
	else if (game_dropType == 2)
		glBindTexture(GL_TEXTURE_2D, client_texDynamiteLabel);
	else if (game_dropType == 3)
		glBindTexture(GL_TEXTURE_2D, client_texClusterLabel);
	else if (game_dropType == 4)
		glBindTexture(GL_TEXTURE_2D, client_texHamstersLabel);
	else if (game_dropType == 5)
		glBindTexture(GL_TEXTURE_2D, client_texKillerBeeLabel);

	glBegin(GL_TRIANGLES);

	glTexCoord2f(0.0f, 1.0f);
	glVertex2f(10.0f, 590.0f);
	glTexCoord2f(0.0f, 0.0f);
	glVertex2f(10.0f, 540.0f);
	glTexCoord2f(1.0f, 0.0f);
	glVertex2f(60.0f, 540.0f);

	glTexCoord2f(0.0f, 1.0f);
	glVertex2f(10.0f, 590.0f);
	glTexCoord2f(1.0f, 0.0f);
	glVertex2f(60.0f, 540.0f);
	glTexCoord2f(1.0f, 1.0f);
	glVertex2f(60.0f, 590.0f);

	glEnd();
	glDisable(GL_TEXTURE_2D);
	glPopMatrix();
	return;
}

static void drawBallWithEyes(int p, int q)
{

	float left, right;

	if ((client_players[p].ball[q].vel.x*client_players[p].ball[q].vel.x +
	    client_players[p].ball[q].vel.y*client_players[p].ball[q].vel.y
	    > 1000.0f
	    || (p == client_turn && q == client_turnBall && game_hooked))
	    && (client_players[p].ball[q].vel.x != 0.0f
	     && client_players[p].ball[q].vel.y != 0.0f))
	{

		if (client_players[p].ball[q].eyes)
		{
			left = 0.0f;
			right = 1.0f;
		}
		else
		{
			left = 1.0f;
			right = 0.0f;
		}

		glRotatef(misc_rad2deg(atan(client_players[p].ball[q].vel.y/
			client_players[p].ball[q].vel.x)),
			0.0f, 0.0f, 1.0f);

		if (p == 0)
			glBindTexture(GL_TEXTURE_2D,
				client_texBall0);
		else if (p==1)
			glBindTexture(GL_TEXTURE_2D,
				client_texBall1);
		else if (p==2)
			glBindTexture(GL_TEXTURE_2D,
				client_texBall2);
		else if (p==3)
			glBindTexture(GL_TEXTURE_2D,
				client_texBall3);
		else if (p==4)
			glBindTexture(GL_TEXTURE_2D,
				client_texBall4);
		else if (p==5)
			glBindTexture(GL_TEXTURE_2D,
				client_texBall5);
				
		glBegin(GL_TRIANGLES);

		glTexCoord2f(left, 1.0f);
		glVertex2f(-10.0f, 10.0f);
		glTexCoord2f(left, 0.0f);
		glVertex2f(-10.0f, -10.0f);
		glTexCoord2f(right, 0.0f);
		glVertex2f(10.0f, -10.0f);

		glTexCoord2f(left, 1.0f);
		glVertex2f(-10.0f, 10.0f);
		glTexCoord2f(right, 0.0f);
		glVertex2f(10.0f, -10.0f);
		glTexCoord2f(right, 1.0f);
		glVertex2f(10.0f, 10.0f);

		glEnd();
	}
	else
	{
		if (client_players[p].ball[q].eyes)
		{
			left = 0.0f;
			right = 1.0f;
		}
		else
		{
			left = 1.0f;
			right = 0.0f;
		}

		if (p == 0)
			glBindTexture(GL_TEXTURE_2D,
				client_texBall0);
		else if (p==1)
			glBindTexture(GL_TEXTURE_2D,
				client_texBall1);
		else if (p==2)
			glBindTexture(GL_TEXTURE_2D,
				client_texBall2);
		else if (p==3)
			glBindTexture(GL_TEXTURE_2D,
				client_texBall3);
		else if (p==4)
			glBindTexture(GL_TEXTURE_2D,
				client_texBall4);
		else if (p==5)
			glBindTexture(GL_TEXTURE_2D,
				client_texBall5);

		glBegin(GL_TRIANGLES);

		glTexCoord2f(left, 1.0f);
		glVertex2f(-10.0f, 10.0f);
		glTexCoord2f(left, 0.0f);
		glVertex2f(-10.0f, -10.0f);
		glTexCoord2f(right, 0.0f);
		glVertex2f(10.0f, -10.0f);

		glTexCoord2f(left, 1.0f);
		glVertex2f(-10.0f, 10.0f);
		glTexCoord2f(right, 0.0f);
		glVertex2f(10.0f, -10.0f);
		glTexCoord2f(right, 1.0f);
		glVertex2f(10.0f, 10.0f);
		glEnd();
	}
	return;
}

static void drawBalls(void)
{
	int p, q;

	glEnable(GL_TEXTURE_2D);

	for (p=0; p<6; p++)
		if (client_players[p].status && client_players[p].ingame)
		{
			for (q=0; q<4; q++)
			{
				float alpha;
				if (client_players[p].ball[q].water)
				{
					alpha = (float)(250-
						client_players[p].ball[q].water)
						/250.0f;
					misc_minMaxLimit(&alpha, 0.0f, 1.0f);
				}
				else
					alpha = 1.0f;

				glColor4f(1.0f, 1.0f, 1.0f, alpha);

				if (client_players[p].ball[q].status)
				{
					glPushMatrix();
					glTranslatef(
						client_players[p].ball[q].pos.x,
						client_players[p].ball[q].pos.y,
						0.0f);

					drawBallWithEyes(p, q);
					glPopMatrix();
				}
			}
		}
	glDisable(GL_TEXTURE_2D);
	return;
}

static void drawLabels(void)
{
	int p, q;

	for (p=0; p<6; p++)
		if (client_players[p].status && client_players[p].ingame)
		{
			for (q=0; q<4; q++)
			{
				if (client_players[p].ball[q].status &&
					!client_players[p].ball[q].water)
				{
					glPushMatrix();
					glTranslatef(
						client_players[p].ball[q].pos.x,
						client_players[p].ball[q].pos.y,
						0.0f);

					if (!(client_turn == p
						&& client_turnBall == q)
						|| game_prepare
						|| game_preparetoend)
						drawLabel(p, q, client_players
							[p].ball[q].health);

					if ((client_turn == p
						&& client_turnBall == q)
						&& game_prepare)
						drawArrow();
						

					glPopMatrix();
				}
			}
		}

	return;
}

static void drawDrop(void)
{
	glColor4f(0.5f, 0.25f, 0.0f, 1.0f);

	glPushMatrix();

	glTranslatef((float)game_dropPos.x, (float)game_dropPos.y, 0.0f);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, client_texBox);

	glBegin(GL_TRIANGLES);

	glTexCoord2f(0.0f, 1.0f);
	glVertex2f(-10.0f, 10.0f);
	glTexCoord2f(0.0f, 0.0f);
	glVertex2f(-10.0f, -10.0f);
	glTexCoord2f(1.0f, 0.0f);
	glVertex2f(10.0f, -10.0f);

	glTexCoord2f(0.0f, 1.0f);
	glVertex2f(-10.0f, 10.0f);
	glTexCoord2f(1.0f, 0.0f);
	glVertex2f(10.0f, -10.0f);
	glTexCoord2f(1.0f, 1.0f);
	glVertex2f(10.0f, 10.0f);

	glEnd();
	glDisable(GL_TEXTURE_2D);
	glPopMatrix();
	return;
}

static void drawTimer(void)
{
	int toDraw;

	if (!client_gameInProgress)
		return;
	if (game_prepare)
		toDraw = 250-game_prepare;
	else if (game_escape)
	{
		toDraw = 500-game_escape;
		if (toDraw < 0)
			toDraw = 0;
	}
	else if (game_frame > 50*client_timelimit)
		toDraw = 0;
	else
		toDraw = (50*client_timelimit)-game_frame;

	glPushMatrix();
	glTranslatef(-video_xOffset, -video_yOffset, 0.0f);

	glColor4f(1.0f, 1.0f, 1.0f, 0.5f);
	glBegin(GL_TRIANGLES);

	glVertex2f(10.0f, 60.0f);
	glVertex2f(10.0f, 10.0f);
	glVertex2f(90.0f, 10.0f);

	glVertex2f(10.0f, 60.0f);
	glVertex2f(90.0f, 10.0f);
	glVertex2f(90.0f, 60.0f);

	glEnd();
	glEnable(GL_TEXTURE_2D);
	if (game_prepare)
		glColor4f(1.0f, 0.0f, 0.0f, 1.0f);
	else if (game_escape)
		glColor4f(1.0f, 1.0f, 0.0f, 1.0f);
	else if (client_playTheJob && client_lag && game_frame == 0)
		glColor4f(1.0f, 0.0f, 0.0f, 1.0f);
	else
		glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

	fonts_drawChar(console_texFont, (toDraw/50/10)+48,
		20.0f, 40.0f,
		20.0f, 50.0f);
	fonts_drawChar(console_texFont, (toDraw/50%10)+48,
		40.0f, 60.0f,
		20.0f, 50.0f);
	fonts_drawChar(console_texFont, (toDraw%50*2/10)+48,
		60.0f, 70.0f,
		35.0f, 50.0f);
	fonts_drawChar(console_texFont, (toDraw%50*2%10)+48,
		70.0f, 80.0f,
		35.0f, 50.0f);
	glDisable(GL_TEXTURE_2D);
	glPopMatrix();
}

static void drawLag(void)
{
	glPushMatrix();
	glTranslatef(-video_xOffset, -video_yOffset, 0.0f);

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	glEnable(GL_TEXTURE_2D);

	glBindTexture(GL_TEXTURE_2D, client_texLag);
	glBegin(GL_TRIANGLES);

	glTexCoord2f(0.0f, 1.0f);
	glVertex2f(25.0f, 120.0f);
	glTexCoord2f(0.0f, 0.0f);
	glVertex2f(25.0f, 70.0f);
	glTexCoord2f(1.0f, 0.0f);
	glVertex2f(75.0f, 70.0f);

	glTexCoord2f(0.0f, 1.0f);
	glVertex2f(25.0f, 120.0f);
	glTexCoord2f(1.0f, 0.0f);
	glVertex2f(75.0f, 70.0f);
	glTexCoord2f(1.0f, 1.0f);
	glVertex2f(75.0f, 120.0f);

	glEnd();
	glDisable(GL_TEXTURE_2D);
	glPopMatrix();
	return;
}
static void drawKeys(void)
{
	int frame;

	if (client_doTheJob)
		frame = game_frame-1;
	else if (client_playTheJob)
	{
		frame = game_frame-1;

		if (game_frame == 0 || client_lag)
			return;
	}
	else
		return;

	glPushMatrix();
	glTranslatef(-video_xOffset, -video_yOffset, 0.0f);

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	glEnable(GL_TEXTURE_2D);

	if (strchr(game_keys[frame], 'U'))
	{
		glBindTexture(GL_TEXTURE_2D, client_texUp);
		glBegin(GL_TRIANGLES);

		glTexCoord2f(0.0f, 1.0f);
		glVertex2f(40.0f, 120.0f);
		glTexCoord2f(0.0f, 0.0f);
		glVertex2f(40.0f, 100.0f);
		glTexCoord2f(1.0f, 0.0f);
		glVertex2f(60.0f, 100.0f);

		glTexCoord2f(0.0f, 1.0f);
		glVertex2f(40.0f, 120.0f);
		glTexCoord2f(1.0f, 0.0f);
		glVertex2f(60.0f, 100.0f);
		glTexCoord2f(1.0f, 1.0f);
		glVertex2f(60.0f, 120.0f);
		glEnd();
	}
	if (strchr(game_keys[frame], 'D'))
	{
		glBindTexture(GL_TEXTURE_2D, client_texDown);
		glBegin(GL_TRIANGLES);

		glTexCoord2f(0.0f, 1.0f);
		glVertex2f(40.0f, 90.0f);
		glTexCoord2f(0.0f, 0.0f);
		glVertex2f(40.0f, 70.0f);
		glTexCoord2f(1.0f, 0.0f);
		glVertex2f(60.0f, 70.0f);

		glTexCoord2f(0.0f, 1.0f);
		glVertex2f(40.0f, 90.0f);
		glTexCoord2f(1.0f, 0.0f);
		glVertex2f(60.0f, 70.0f);
		glTexCoord2f(1.0f, 1.0f);
		glVertex2f(60.0f, 90.0f);
		glEnd();
	}
	if (strchr(game_keys[frame], 'L'))
	{
		glBindTexture(GL_TEXTURE_2D, client_texLeft);
		glBegin(GL_TRIANGLES);

		glTexCoord2f(0.0f, 1.0f);
		glVertex2f(10.0f, 90.0f);
		glTexCoord2f(0.0f, 0.0f);
		glVertex2f(10.0f, 70.0f);
		glTexCoord2f(1.0f, 0.0f);
		glVertex2f(30.0f, 70.0f);

		glTexCoord2f(0.0f, 1.0f);
		glVertex2f(10.0f, 90.0f);
		glTexCoord2f(1.0f, 0.0f);
		glVertex2f(30.0f, 70.0f);
		glTexCoord2f(1.0f, 1.0f);
		glVertex2f(30.0f, 90.0f);
		glEnd();
	}
	if (strchr(game_keys[frame], 'R'))
	{
		glBindTexture(GL_TEXTURE_2D, client_texRight);
		glBegin(GL_TRIANGLES);

		glTexCoord2f(0.0f, 1.0f);
		glVertex2f(70.0f, 90.0f);
		glTexCoord2f(0.0f, 0.0f);
		glVertex2f(70.0f, 70.0f);
		glTexCoord2f(1.0f, 0.0f);
		glVertex2f(90.0f, 70.0f);

		glTexCoord2f(0.0f, 1.0f);
		glVertex2f(70.0f, 90.0f);
		glTexCoord2f(1.0f, 0.0f);
		glVertex2f(90.0f, 70.0f);
		glTexCoord2f(1.0f, 1.0f);
		glVertex2f(90.0f, 90.0f);
		glEnd();
	}
	if (strchr(game_keys[frame], 'S'))
	{
		glBindTexture(GL_TEXTURE_2D, client_texRopeShoot);
		glBegin(GL_TRIANGLES);

		glTexCoord2f(0.0f, 1.0f);
		glVertex2f(10.0f, 145.0f);
		glTexCoord2f(0.0f, 0.5f);
		glVertex2f(10.0f, 130.0f);
		glTexCoord2f(1.0f, 0.5f);
		glVertex2f(40.0f, 130.0f);

		glTexCoord2f(0.0f, 1.0f);
		glVertex2f(10.0f, 145.0f);
		glTexCoord2f(1.0f, 0.5f);
		glVertex2f(40.0f, 130.0f);
		glTexCoord2f(1.0f, 1.0f);
		glVertex2f(40.0f, 145.0f);
		glEnd();
	}
	if (strchr(game_keys[frame], 'E'))
	{
		glBindTexture(GL_TEXTURE_2D, client_texRopeShoot);
		glBegin(GL_TRIANGLES);

		glTexCoord2f(0.0f, 0.5f);
		glVertex2f(60.0f, 145.0f);
		glTexCoord2f(0.0f, 0.0f);
		glVertex2f(60.0f, 130.0f);
		glTexCoord2f(1.0f, 0.0f);
		glVertex2f(90.0f, 130.0f);

		glTexCoord2f(0.0f, 0.5f);
		glVertex2f(60.0f, 145.0f);
		glTexCoord2f(1.0f, 0.0f);
		glVertex2f(90.0f, 130.0f);
		glTexCoord2f(1.0f, 0.5f);
		glVertex2f(90.0f, 145.0f);
		glEnd();
	}

	glDisable(GL_TEXTURE_2D);
	glPopMatrix();
}

static void drawWind(void)
{
	glPushMatrix();
	glTranslatef(video_xOffset, -video_yOffset, 0.0f);
	glColor4f(1.0f, 0.0f, 0.0f, 0.5f);

	if (game_wind > 0)
	{
		glBegin(GL_TRIANGLES);
		glVertex2f(690.0f, 30.0f);
		glVertex2f(690.0f, 10.0f);
		glVertex2f(690.0f + (float)game_wind, 10.0f);

		glVertex2f(690.0f, 30.0f);
		glVertex2f(690.0f + (float)game_wind, 10.0f);
		glVertex2f(690.0f + (float)game_wind, 30.0f);
		glEnd();
	}
	else if (game_wind < 0)
	{
		glBegin(GL_TRIANGLES);
		glVertex2f(690.0f + (float)game_wind, 30.0f);
		glVertex2f(690.0f + (float)game_wind, 10.0f);
		glVertex2f(690.0f, 10.0f);

		glVertex2f(690.0f + (float)game_wind, 30.0f);
		glVertex2f(690.0f, 10.0f);
		glVertex2f(690.0f, 30.0f);
		glEnd();
	}
	glBegin(GL_LINES);

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	glVertex2f(590.0f, 10.0f);
	glVertex2f(790.0f, 10.0f);

	glVertex2f(590.0f, 30.0f);
	glVertex2f(790.0f, 30.0f);

	glVertex2f(590.0f, 10.0f);
	glVertex2f(590.0f, 30.0f);

	glVertex2f(690.0f, 10.0f);
	glVertex2f(690.0f, 30.0f);

	glVertex2f(790.0f, 10.0f);
	glVertex2f(790.0f, 30.0f);
	glEnd();

	glPopMatrix();

	return;
}

static void drawScore(void)
{
	int playersCount = 0;
	int scores[6];
	int colors[6];
	int p, q;
	int maxscore = client_balls * client_health;
	float alpha = 0.5f;

	glPushMatrix();
	glTranslatef(video_xOffset, video_yOffset, 0.0f);
	/* get scores */
	for (p=0; p<6; p++)
		if (client_players[p].status && client_players[p].ingame)
		{
			scores[playersCount] = client_players[p].ball[0].health;
			for (q=1; q<client_balls; q++)
				scores[playersCount]+=
					client_players[p].ball[q].health;
			if (scores[playersCount] > maxscore)
				scores[playersCount] = maxscore;
			colors[playersCount] = p;
			playersCount++;
		}
	
	/* sort them */
	for (p=1; p<playersCount; p++)
		for (q=0; q<p; q++)
	{
		if (scores[p] > scores[q])
		{
			int buf = scores[p];
			scores[p] = scores[q];
			scores[q] = buf;

			buf = colors[p];
			colors[p] = colors[q];
			colors[q] = buf;
		}
	}

	for (p=0; p<playersCount; p++)
	{
		int s;

		if (colors[p] == 0)
			glColor4f(1.0f, 0.0f, 0.0f, alpha);
		else if (colors[p] == 1)
			glColor4f(0.0f, 0.0f, 1.0f, alpha);
		else if (colors[p] == 2)
			glColor4f(0.0f, 1.0f, 0.0f, alpha);
		else if (colors[p] == 3)
			glColor4f(1.0f, 1.0f, 0.0f, alpha);
		else if (colors[p] == 4)
			glColor4f(1.0f, 0.0f, 1.0f, alpha);
		else if (colors[p] == 5)
			glColor4f(0.0f, 1.0f, 1.0f, alpha);

		glBegin(GL_TRIANGLES);
		glVertex2f(690.0f, 590.0f-p*10.0f);
		glVertex2f(690.0f, 580.0f-p*10.0f);
		glVertex2f(690.0f + (float)scores[p]/(float)maxscore*100.0f,
			580.0f-p*10.0f);

		glVertex2f(690.0f, 590.0f-p*10.0f);
		glVertex2f(690.0f + (float)scores[p]/(float)maxscore*100.0f,
			580.0f-p*10.0f);
		glVertex2f(690.0f + (float)scores[p]/(float)maxscore*100.0f,
			590.0f-p*10.0f);
		glEnd();

		glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
		glBegin(GL_LINES);
		glVertex2f(690.0f, 590.0f-p*10.0f);
		glVertex2f(790.0f, 590.0f-p*10.0f);

		glVertex2f(690.0f, 580.0f-p*10.0f);
		glVertex2f(790.0f, 580.0f-p*10.0f);

		glVertex2f(690.0f, 580.0f-p*10.0f);
		glVertex2f(690.0f, 590.0f-p*10.0f);

		glVertex2f(790.0f, 580.0f-p*10.0f);
		glVertex2f(790.0f, 590.0f-p*10.0f);

		s = 0;
		for (q=0; q<client_balls; q++)
		{
			s += client_players[colors[p]].ball[q].health;

			if (s <= maxscore)
			{
				glVertex2f(690.0f +
					(float)s/(float)maxscore*100.0f,
					580.0f-p*10.0f);
				glVertex2f(690.0f +
					(float)s/(float)maxscore*100.0f,
					590.0f-p*10.0f);
			}
		}

		glEnd();

	}

	glPopMatrix();

	return;
}

static void drawExitConfirm(void)
{
	glBegin(GL_TRIANGLES);

	glColor4f(0.75f, 0.75f, 0.75f, 0.75f);
	glVertex2f(240.0f, 350.0f);
	glVertex2f(240.0f, 270.0f);
	glVertex2f(560.0f, 270.0f);

	glVertex2f(240.0f, 350.0f);
	glVertex2f(560.0f, 270.0f);
	glVertex2f(560.0f, 350.0f);

	glEnd();

	glEnable(GL_TEXTURE_2D);

	printLine(8, 250.0f, (char*)txtExitConfirm);
	printLine(10, 250.0f, (char*)txtExitConfirm2);

	glDisable(GL_TEXTURE_2D);
	return;
}
static void drawTabScore(void)
{
	int p, r;

	glBegin(GL_TRIANGLES);

	glColor4f(0.75f, 0.75f, 0.75f, 0.75f);
	glVertex2f(100.0f, 500.0f);
	glVertex2f(100.0f, 100.0f);
	glVertex2f(700.0f, 100.0f);

	glVertex2f(100.0f, 500.0f);
	glVertex2f(700.0f, 100.0f);
	glVertex2f(700.0f, 500.0f);

	glEnd();

	glEnable(GL_TEXTURE_2D);
	p = 0;

	printLine(p, 100.0f, (char*)txtInGame);
	printLine(p, 500.0f, (char*)txtColor);
	printLine(p, 600.0f, (char*)txtAskReady);
	p++;
	for (r=0; r<6; r++)
	{
		if (client_players[r].status && client_players[r].ingame)
		{
			printLine(p, 100.0f, client_players[r].name);
			printLine(p, 500.0f, colors[r]);
			if (client_players[r].ready)
				printLine(p, 600.0f, (char*)txtReady);
			else
				printLine(p, 600.0f, (char*)txtUnready);
			p++;
		}
	}
	p++;
	printLine(p, 100.0f, (char*)txtSpectators);
	printLine(p, 500.0f, (char*)txtColor);
	p++;
	for (r=0; r<6; r++)
	{
		if (client_players[r].status && !client_players[r].ingame)
		{
			printLine(p, 100.0f, client_players[r].name);
			printLine(p, 500.0f, colors[r]);
			p++;
		}
	}
	p++;
	printLine(p, 100.0f, client_serverName); p++;
	printLine(p, 100.0f, client_motd[0]); p++;
	printLine(p, 100.0f, client_motd[1]); p++;
	printLine(p, 100.0f, client_motd[2]); p++;
	printLine(p, 100.0f, client_motd[3]); p++;
	printLine(p, 100.0f, client_motd[4]); p++;
	printLine(p, 100.0f, client_motd[5]); p++;
	printLine(p, 100.0f, client_motd[6]); p++;
	printLine(p, 100.0f, client_motd[7]); p++;
	printLine(p, 100.0f, client_motd[8]);


	glDisable(GL_TEXTURE_2D);
	return;
}

static void drawCroshair(void)
{
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	glPushMatrix();
	glTranslatef(client_players[client_turn].ball[client_turnBall].pos.x,
		client_players[client_turn].ball[client_turnBall].pos.y,
		0.0f);
	
	if (game_side)
	{
		glRotatef(game_aim, 0.0f, 0.0f, 1.0f);
		glTranslatef(100.0f, 0.0f, 0.0f);
	}	
	else
	{
		glRotatef(-game_aim, 0.0f, 0.0f, 1.0f);
		glTranslatef(-100.0f, 0.0f, 0.0f);
	}
	glBegin(GL_LINES);

	glVertex2f(-5.0f, 0.0f);
	glVertex2f(5.0f, 0.0f);

	glVertex2f(0.0f, -5.0f);
	glVertex2f(0.0f, 5.0f);

	glEnd();

	glPopMatrix();
	return;
}

static void drawRope(void)
{
	glColor4f(1.0f, 1.0f, 0.0f, 1.0f);
	if (game_hooked)
	{
		glBegin(GL_LINES);
		glVertex2f(
			client_players[client_turn].ball[client_turnBall].pos.x,
			client_players[client_turn].ball[client_turnBall].pos.y
			);
		glVertex2f(game_hookPos.x, game_hookPos.y);
		glEnd();
	}
	else
	{
		glPushMatrix();
		glTranslatef(
			client_players[client_turn].ball[client_turnBall].pos.x,
			client_players[client_turn].ball[client_turnBall].pos.y,
			0.0f);
		glRotatef(game_ropeang, 0.0f, 0.0f, 1.0f);

		glBegin(GL_LINES);
		glVertex2f(0.0f, 0.0f);
		glVertex2f(game_ropelen, 0.0f);
		glEnd();

		glPopMatrix();
	}
	return;
}

static void drawZooka(int p)
{
	float left, right;

	if (game_weap[p].vel.x > 0)
	{
		left = 0.0f;
		right = 1.0f;
	}
	else
	{
		left = 1.0f;
		right = 0.0f;
	}

	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, client_texZooka);

	glRotatef(misc_rad2deg(atan(game_weap[p].vel.y/game_weap[p].vel.x)),
		0.0f, 0.0f, 1.0f);

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	glBegin(GL_TRIANGLES);

	glTexCoord2f(left, 1.0f);
	glVertex2f(-10.0f, 10.0f);
	glTexCoord2f(left, 0.0f);
	glVertex2f(-10.0f, -10.0f);
	glTexCoord2f(right, 0.0f);
	glVertex2f(10.0f, -10.0f);

	glTexCoord2f(left, 1.0f);
	glVertex2f(-10.0f, 10.0f);
	glTexCoord2f(right, 0.0f);
	glVertex2f(10.0f, -10.0f);
	glTexCoord2f(right, 1.0f);
	glVertex2f(10.0f, 10.0f);

	glEnd();
	glDisable(GL_TEXTURE_2D);
}
static void drawGrenade(int p)
{
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, client_texGrenade);

	glRotatef(game_weap[p].rotation, 0.0f, 0.0f, 1.0f);

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	glBegin(GL_TRIANGLES);

	glTexCoord2f(0.0f, 1.0f);
	glVertex2f(-10.0f, 10.0f);
	glTexCoord2f(0.0f, 0.0f);
	glVertex2f(-10.0f, -10.0f);
	glTexCoord2f(1.0f, 0.0f);
	glVertex2f(10.0f, -10.0f);

	glTexCoord2f(0.0f, 1.0f);
	glVertex2f(-10.0f, 10.0f);
	glTexCoord2f(1.0f, 0.0f);
	glVertex2f(10.0f, -10.0f);
	glTexCoord2f(1.0f, 1.0f);
	glVertex2f(10.0f, 10.0f);

	glEnd();
	glDisable(GL_TEXTURE_2D);
}
static void drawDynamite(int p)
{
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, client_texDynamite);

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	glBegin(GL_TRIANGLES);

	glTexCoord2f(0.0f, 1.0f);
	glVertex2f(-10.0f, 15.0f);
	glTexCoord2f(0.0f, 0.0f);
	glVertex2f(-10.0f, -5.0f);
	glTexCoord2f(1.0f, 0.0f);
	glVertex2f(10.0f, -5.0f);

	glTexCoord2f(0.0f, 1.0f);
	glVertex2f(-10.0f, 15.0f);
	glTexCoord2f(1.0f, 0.0f);
	glVertex2f(10.0f, -5.0f);
	glTexCoord2f(1.0f, 1.0f);
	glVertex2f(10.0f, 15.0f);

	glEnd();
	glDisable(GL_TEXTURE_2D);
}
static void drawCluster(int p)
{
	float size = 10.0f;
	glEnable(GL_TEXTURE_2D);
	if (p==0)
	{
		glBindTexture(GL_TEXTURE_2D, client_texCluster);
		glRotatef(game_weap[p].rotation, 0.0f, 0.0f, 1.0f);
	}
	else
	{
		glBindTexture(GL_TEXTURE_2D, client_texCluster2);
		glRotatef(game_weap[p].rotation + (p-1)*90.0f,
			0.0f, 0.0f, 1.0f);
		size = 14.0f;
	}

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	glBegin(GL_TRIANGLES);

	glTexCoord2f(0.0f, 1.0f);
	glVertex2f(-size, size);
	glTexCoord2f(0.0f, 0.0f);
	glVertex2f(-size, -size);
	glTexCoord2f(1.0f, 0.0f);
	glVertex2f(size, -size);

	glTexCoord2f(0.0f, 1.0f);
	glVertex2f(-size, size);
	glTexCoord2f(1.0f, 0.0f);
	glVertex2f(size, -size);
	glTexCoord2f(1.0f, 1.0f);
	glVertex2f(size, size);

	glEnd();
	glDisable(GL_TEXTURE_2D);
/*

	if (p==0)
	{
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, client_texCluster);

		glRotatef(game_weap[p].rotation, 0.0f, 0.0f, 1.0f);

		glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
		glBegin(GL_TRIANGLES);

		glTexCoord2f(0.0f, 1.0f);
		glVertex2f(-10.0f, 10.0f);
		glTexCoord2f(0.0f, 0.0f);
		glVertex2f(-10.0f, -10.0f);
		glTexCoord2f(1.0f, 0.0f);
		glVertex2f(10.0f, -10.0f);

		glTexCoord2f(0.0f, 1.0f);
		glVertex2f(-10.0f, 10.0f);
		glTexCoord2f(1.0f, 0.0f);
		glVertex2f(10.0f, -10.0f);
		glTexCoord2f(1.0f, 1.0f);
		glVertex2f(10.0f, 10.0f);

		glEnd();
		glDisable(GL_TEXTURE_2D);
	}
	else
	{
		glColor4f(1.0f, 0.0f, 1.0f, 1.0f);
		drawBall(16, 5.0f);
	}
*/

	return;
}
static void drawHamster(int p)
{
	float left, right;

	if ((game_frame/10)%2)
	{
		if (game_weap[p].side)
		{
			left = 0.5f;
			right = 0.0f;
		}
		else
		{
			left = 0.0f;
			right = 0.5f;
		}
	}
	else
	{
		if (game_weap[p].side)
		{
			left = 1.0f;
			right = 0.5f;
		}
		else
		{
			left = 0.5f;
			right = 1.0f;
		}
	}

	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, client_texHamster);

	glRotatef(misc_rad2deg(atan(game_weap[p].vel.y/game_weap[p].vel.x)),
		0.0f, 0.0f, 1.0f);

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	glBegin(GL_TRIANGLES);

	glTexCoord2f(left, 1.0f);
	glVertex2f(-10.0f, 12.5f);
	glTexCoord2f(left, 0.0f);
	glVertex2f(-10.0f, -7.5f);
	glTexCoord2f(right, 0.0f);
	glVertex2f(10.0f, -7.5f);

	glTexCoord2f(left, 1.0f);
	glVertex2f(-10.0f, 12.5f);
	glTexCoord2f(right, 0.0f);
	glVertex2f(10.0f, -7.5f);
	glTexCoord2f(right, 1.0f);
	glVertex2f(10.0f, 12.5f);

	glEnd();
	glDisable(GL_TEXTURE_2D);
}
static void drawKillerBee(int p)
{
	float left, right;

	if ((game_frame/10)%2)
	{
		if (game_weap[p].side)
		{
			left = 0.5f;
			right = 0.0f;
		}
		else
		{
			left = 0.0f;
			right = 0.5f;
		}
	}
	else
	{
		if (game_weap[p].side)
		{
			left = 1.0f;
			right = 0.5f;
		}
		else
		{
			left = 0.5f;
			right = 1.0f;
		}
	}

	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, client_texKillerBee);

	glRotatef(misc_rad2deg(atan(game_weap[p].vel.y/game_weap[p].vel.x)),
		0.0f, 0.0f, 1.0f);

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	glBegin(GL_TRIANGLES);

	glTexCoord2f(left, 1.0f);
	glVertex2f(-10.0f, 10.0f);
	glTexCoord2f(left, 0.0f);
	glVertex2f(-10.0f, -10.0f);
	glTexCoord2f(right, 0.0f);
	glVertex2f(10.0f, -10.0f);

	glTexCoord2f(left, 1.0f);
	glVertex2f(-10.0f, 10.0f);
	glTexCoord2f(right, 0.0f);
	glVertex2f(10.0f, -10.0f);
	glTexCoord2f(right, 1.0f);
	glVertex2f(10.0f, 10.0f);

	glEnd();
	glDisable(GL_TEXTURE_2D);
}

static void drawWeapon(int p)
{
	glPushMatrix();
	glTranslatef(game_weap[p].pos.x, game_weap[p].pos.y, 0.0f);

	if (game_dropType == 0)
		drawZooka(p);
	else if (game_dropType == 1)
		drawGrenade(p);
	else if (game_dropType == 2)
		drawDynamite(p);
	else if (game_dropType == 3)
		drawCluster(p);
	else if (game_dropType == 4)
		drawHamster(p);
	else if (game_dropType == 5)
		drawKillerBee(p);


	glPopMatrix();
	return;
}

static void drawBooms(void)
{
	int p;
	for (p=0; p<MAXCLUSTERS; p++)
		if (game_weap[p].boom)
		{
			glPushMatrix();
			glTranslatef(game_weap[p].pos.x,
					game_weap[p].pos.y,
					0.0f);
			if (game_dropType == 0)
				drawBoom(32.0f, ZOOKABOOM, p);
			else if (game_dropType == 1)
				drawBoom(32.0f, GRENADEBOOM, p);
			else if (game_dropType == 2)
				drawBoom(32.0f, DYNAMITEBOOM, p);
			else if (game_dropType == 3 && p == 0)
				drawBoom(32.0f, CLUSTERBOOM, p);
			else if (game_dropType == 3)
				drawBoom(32.0f, CLUSTERBOOM2, p);
			else if (game_dropType == 4)
				drawBoom(32.0f, HAMSTERBOOM, p);
			else if (game_dropType == 5)
				drawBoom(32.0f, KILLERBEEBOOM, p);
			glPopMatrix();
		}
		return;
}
static void drawWeapons(void)
{
	int p;
	for (p=0; p<MAXCLUSTERS; p++)
		if (game_weap[p].status)
			drawWeapon(p);
	return;
}

/* do all the drawing stuff */
void client_render_draw(void)
{
	glLineWidth((int)commands_cl_linewidth);

	/* camera movement */
	if (client_cameraFollow)
		client_cameraFollow++;
	
	else if (client_gameInProgress && client_turn != -1)
	{
		/* follow 'drop' */
		if (client_cameraFocus == 2)
		{
			input_mouse.x += (game_dropPos.x - input_mouse.x)/10;
			input_mouse.y += (game_dropPos.y - input_mouse.y)/10;
		}
		/* follow ball */
		else if (client_cameraFocus == 1)
		{
			input_mouse.x += (client_players[client_turn].
				ball[client_turnBall].pos.x - input_mouse.x)/10;
			input_mouse.y += (client_players[client_turn].
				ball[client_turnBall].pos.y - input_mouse.y)/10;
		}
	}

	if (client_map && client_map->ready)
	{
		misc_minMaxLimit(&input_mouse.x,
			client_map->xMin,
			client_map->xMax);
		misc_minMaxLimit(&input_mouse.y,
			0.0f,
			client_map->yMax);
	}

	if (client_cameraFollow == 120)
		client_cameraFollow = 0;
	
	/* prepare everything... */
	glDisable(GL_DEPTH_TEST);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glClearColor(0.23f, 0.16f, 0.24f, 1.0f);

	glOrtho(0.0-video_xOffset, 800.0+video_xOffset,
		0.0-video_yOffset, 600.0+video_yOffset, -1.0f, 1.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	if (client_isError || !client_connected)
	{
		glClearColor(0.0f, 0.0f, 0.0f, 1.0f);

		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
			glEnable(GL_TEXTURE_2D);

	}
	if (client_isError)
	{
		int q;
		for (q=0; q<strlen(client_errormsg); q++)
		{
			fonts_drawChar(console_texFont, client_errormsg[q],
				q*10.0f, 10.0f+ q*10.0f,
				10.0f, 25.0f);
		}
		glDisable(GL_TEXTURE_2D);
		return;
	}
	
	glPushMatrix();
	glTranslatef(-input_mouse.x+400.0f, -input_mouse.y+300.0f, 0.0f);
	if (client_map && client_map->ready)
	{
		if (commands_cl_background)
			drawBack();
		drawWater();
		if (client_gameInProgress)
		{
			if (client_playTheJob || client_doTheJob)
			{
				if (game_rope)
					drawRope();
				else if (!game_roperep)
					drawCroshair();
			}

			drawBalls();

			drawWeapons();

			if (client_gametype != 4 && client_turn != -1
				&& game_drop)
				drawDrop();
		}
		drawWorld();
		if (client_gameInProgress)
		{
			int w;

			drawBooms();
			drawLabels();
			for (w=0; w<MAXCLUSTERS; w++)
				if (game_weap[w].status && game_weap[w].fuse)
				{
					glPushMatrix();
					glTranslatef(game_weap[w].pos.x,
						game_weap[w].pos.y,
						0.0f);
					drawFuseTime(w);
					glPopMatrix();
				}
		}
	}
	glPopMatrix();

	if (client_exitConfirm)
		drawExitConfirm();
	else if ((!client_gameInProgress || input_keys[SDLK_TAB])
		&& !console_active && client_connected)
		drawTabScore();
	
	drawTimer();
	drawKeys();
	if (client_playTheJob && client_lag && game_frame != 0)
		drawLag();

	if (client_gameInProgress)
		drawWind();
	
	if (client_gametype != 4 && client_gameInProgress)
		drawScore();

	if (client_gametype != 4 && client_gameInProgress && !game_drop
		&& !game_escape && !game_preparetoend
		&& (client_doTheJob || client_playTheJob))
		drawWeaponLabel();

	console_messagesDraw();

	return;
}
