/*
 *  (C) Copyright 2007-2009 by FreeShoppa Development Team
 *                        (http://chomzee.ethernet.pl/freeshoppa/)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <sys/stat.h>
#include <sys/types.h>

#include "freeshoppa.h"

/* global variables changeable with console */

#define DEFAULT_NAME "Player"
char commands_name[81] = DEFAULT_NAME;
#define DEFAULT_CO_BLINK 40
int commands_co_blink = DEFAULT_CO_BLINK;
#define DEFAULT_CO_ALPHA 200
int commands_co_alpha = DEFAULT_CO_ALPHA;
#define DEFAULT_VD_WIDTH 800
int commands_vd_width = DEFAULT_VD_WIDTH;
#define DEFAULT_VD_HEIGHT 600
int commands_vd_height = DEFAULT_VD_HEIGHT;
#define DEFAULT_VD_FULLSCREEN 0
int commands_vd_fullscreen = DEFAULT_VD_FULLSCREEN;
#define DEFAULT_VD_VSYNC 0
int commands_vd_vsync = DEFAULT_VD_VSYNC;
#define DEFAULT_SV_MAX_CLIENTS 8
int commands_sv_max_clients = DEFAULT_SV_MAX_CLIENTS;
#define DEFAULT_SV_MAX_PLAYERS 2
int commands_sv_max_players = DEFAULT_SV_MAX_PLAYERS;
#define DEFAULT_SV_NAME "Unnamed Server"
char commands_sv_name[81] = DEFAULT_SV_NAME;
#define DEFAULT_SV_PASSWD ""
char commands_sv_passwd[81] = DEFAULT_SV_PASSWD;
#define DEFAULT_CL_PASSWD ""
char commands_cl_passwd[81] = DEFAULT_CL_PASSWD;
#define DEFAULT_SV_MAP "shoppa1"
char commands_sv_map[81] = DEFAULT_SV_MAP;
#define DEFAULT_SV_BALLS 4
int commands_sv_balls = DEFAULT_SV_BALLS;
#define DEFAULT_CL_HOSTNAME "localhost"
char commands_cl_hostname[81] = DEFAULT_CL_HOSTNAME;
#define DEFAULT_SV_HOSTNAME "localhost"
char commands_sv_hostname[81] = DEFAULT_SV_HOSTNAME;
#define DEFAULT_SV_PORT 22200
int commands_sv_port = DEFAULT_SV_PORT;
#define DEFAULT_CL_PORT 22200
int commands_cl_port = DEFAULT_CL_PORT;
#define DEFAULT_SV_TIMELIMIT 45
int commands_sv_timelimit = DEFAULT_SV_TIMELIMIT;
#define DEFAULT_SV_GAMETYPE 0
int commands_sv_gametype = DEFAULT_SV_GAMETYPE;
#define DEFAULT_SV_FALL 1
int commands_sv_fall = DEFAULT_SV_FALL;
#define DEFAULT_SV_HEALTH 100
int commands_sv_health = DEFAULT_SV_HEALTH;
#define DEFAULT_DB_FRAMETIME 0
int commands_db_frametime = DEFAULT_DB_FRAMETIME;

/* message of the day */
#define DEFAULT_SV_MOTD1 "^1F1                 ^0-- ^4Join Game"
#define DEFAULT_SV_MOTD2 "^1F2                 ^0-- ^4Join Spectators"
#define DEFAULT_SV_MOTD3 "^1F3                 ^0-- ^4Ready"
#define DEFAULT_SV_MOTD4 "^1F4                 ^0-- ^4Unready"
#define DEFAULT_SV_MOTD5 "^1AWSD ^0or ^1Arrows     ^0-- ^4Movement"
#define DEFAULT_SV_MOTD6 "^1Spacebar           ^0-- ^4Rope"
#define DEFAULT_SV_MOTD7 "^1Enter              ^0-- ^4Shoot ^0or ^4jump"
#define DEFAULT_SV_MOTD8 "^112345              ^0-- ^4Change fuse time"
#define DEFAULT_SV_MOTD9 "^1T                  ^0-- ^4Chat"
char commands_sv_motd1[81] = DEFAULT_SV_MOTD1;
char commands_sv_motd2[81] = DEFAULT_SV_MOTD2;
char commands_sv_motd3[81] = DEFAULT_SV_MOTD3;
char commands_sv_motd4[81] = DEFAULT_SV_MOTD4;
char commands_sv_motd5[81] = DEFAULT_SV_MOTD5;
char commands_sv_motd6[81] = DEFAULT_SV_MOTD6;
char commands_sv_motd7[81] = DEFAULT_SV_MOTD7;
char commands_sv_motd8[81] = DEFAULT_SV_MOTD8;
char commands_sv_motd9[81] = DEFAULT_SV_MOTD9;

#define DEFAULT_CONFIGFILE "config.cfg"
char commands_configFile[81] = DEFAULT_CONFIGFILE;
#define DEFAULT_VD_BMPMOVIE 0
int commands_vd_bmpmovie = DEFAULT_VD_BMPMOVIE;
#define DEFAULT_SV_STATUSFILE "status.txt"
char commands_sv_statusfile[81] = DEFAULT_SV_STATUSFILE;
#define DEFAULT_SV_STATUS 0
int commands_sv_status = DEFAULT_SV_STATUS;
#define DEFAULT_CL_LINEWIDTH 1
int commands_cl_linewidth = DEFAULT_CL_LINEWIDTH;
#define DEFAULT_CL_WATER 1
int commands_cl_water = DEFAULT_CL_WATER;
#define DEFAULT_CL_BACKGROUND 1
int commands_cl_background = DEFAULT_CL_BACKGROUND;
#define DEFAULT_RC_SVPASSWD ""
char commands_rc_svpasswd[81] = DEFAULT_RC_SVPASSWD;
#define DEFAULT_RC_CLPASSWD ""
char commands_rc_clpasswd[81] = DEFAULT_RC_CLPASSWD;

#ifndef DED
static void vd_reload     (char *arg);
static void say           (char *arg);
static void quit          (char *arg);
static void name          (char *arg);
static void rc_cmd        (char *arg);
static void rc_restart    (char *arg);
static void rc_kick       (char *arg);
#endif
static void help          (char *arg);
static void echo          (char *arg);
static void clear         (char *arg);
static void version       (char *arg);
static void save          (char *arg);
static void co_blink      (char *arg);
static void co_alpha      (char *arg);
static void vd_width      (char *arg);
static void vd_height     (char *arg);
static void vd_fullscreen (char *arg);
static void vd_vsync      (char *arg);
static void sv_max_clients(char *arg);
static void sv_max_players(char *arg);
static void sv_name       (char *arg);
static void sv_passwd     (char *arg);
static void sv_delpasswd  (char *arg);
static void cl_passwd     (char *arg);
static void cl_delpasswd  (char *arg);
static void sv_map        (char *arg);
static void sv_balls      (char *arg);
static void cl_hostname   (char *arg);
static void sv_hostname   (char *arg);
static void sv_port       (char *arg);
static void cl_port       (char *arg);
static void sv_timelimit  (char *arg);
static void sv_gametype   (char *arg);
static void sv_fall       (char *arg);
static void sv_health     (char *arg);
static void db_frametime  (char *arg);
static void sv_motd1      (char *arg);
static void sv_motd2      (char *arg);
static void sv_motd3      (char *arg);
static void sv_motd4      (char *arg);
static void sv_motd5      (char *arg);
static void sv_motd6      (char *arg);
static void sv_motd7      (char *arg);
static void sv_motd8      (char *arg);
static void sv_motd9      (char *arg);
static void vd_bmpmovie   (char *arg);
static void sv_statusfile (char *arg);
static void sv_status     (char *arg);
static void cl_linewidth  (char *arg);
static void cl_water      (char *arg);
static void cl_background (char *arg);
static void rc_svpasswd   (char *arg);
static void rc_svdelpasswd(char *arg);
static void rc_clpasswd   (char *arg);
static void rc_cldelpasswd(char *arg);

static struct _cmds
{
	void (*ptr)(char *);
	const char *name;
	const char *usage;
	const char *desc;
} cmds[] = {
#ifndef DED
{vd_reload, "vd_reload", "vd_reload", "Reloads the video settings"},
{say, "say", "say [text]", "Sends message to other players"},
{quit, "quit", "quit", "Quits the game"},
{name, "name", "name [string]", "Sets name of the player"},
{rc_cmd, "rc_cmd", "rc_cmd [string]", "Queues command to execution with "
	"rc_restart"},
{rc_restart, "rc_restart", "rc_restart", "Execute commands queued by "
	"rc_cmd and restart server"},
{rc_kick, "rc_kick", "rc_kick [red/blue/green/yellow/magenta/cyan]", "Kicks "
	"player from the server"},
#endif
{help, "help", "help [command]", "Shows short command's description"},
{echo, "echo", "echo [text]", "Prints text to console"},
{clear, "clear", "clear", "Clears ingame console"},
{version, "version", "version", "Shows version of FreeShoppa"},
{save, "save", "save", "Saves the config"},
{co_blink, "co_blink", "co_blink [integer]", "No help available yet"},
{co_alpha, "co_alpha", "co_alpha [integer]", "No help available yet"},
{vd_width, "vd_width", "vd_width [integer]", "Sets width of the screen"},
{vd_height, "vd_height", "vd_height [integer]", "Sets height of the screen"},
{vd_fullscreen, "vd_fullscreen", "vd_fullscreen [integer]",
						"Sets fullscreen mode"},
{vd_vsync, "vd_vsync", "vd_vsync [boolean]",
	"Set to 0 to gain performance, set to 1 to gain better visual effect"},
{sv_max_clients, "sv_max_clients", "sv_max_clients [integer]",
		     "Sets the maximum number of clients connected to server"},
{sv_max_players, "sv_max_players", "sv_max_players [integer]",
		     "Sets the maximum number of players"},
{sv_name, "sv_name", "sv_name [string]", "Sets the server name"},
{sv_passwd, "sv_passwd", "sv_passwd [string]", "Sets the server password"},
{sv_delpasswd, "sv_delpasswd", "sv_delpasswd", "Deletes server password"},
{cl_passwd, "cl_passwd", "cl_passwd [string]",
		"Sets password which client uses to connect to a server"},
{cl_delpasswd, "cl_delpasswd", "cl_delpasswd", "Deletes cl_passwd"},
{sv_map, "sv_map", "sv_map [string]", "Sets map for server"},
{sv_balls, "sv_balls", "sv_balls [integer]",
					"Number of balls every player has"},
{cl_hostname, "cl_hostname", "cl_hostname [string]",
			"Hostname of server that client will connect to"},
{sv_hostname, "sv_hostname", "sv_hostname [string]",
			"Hostname of the server"},
{sv_port, "sv_port", "sv_port [integer]",
				"Port which server should listen on"},
{cl_port, "cl_port", "cl_port [integer]",
				"Port which client should connect to"},
{sv_timelimit, "sv_timelimit", "sv_timelimit [integer]",
				"Sets the timelimit"},
{sv_gametype, "sv_gametype", "sv_gametype [integer]",
	"Sets the game type. 0=Shoppa 1=FlyShoppa 2=WxW 3=Proper 4=RopeRace"},
{sv_fall, "sv_fall", "sv_fall [boolean]", "Falling damage"},
{sv_health, "sv_health", "sv_health [integer]",
	"Sets default health of each ball"},
{db_frametime, "db_frametime", "db_frametime [boolean]",
	"Shows how long does one frame drawing last"},
{sv_motd1, "sv_motd1", "sv_motd1 [string]", "1st line of 'Message Of The Day'"},
{sv_motd2, "sv_motd2", "sv_motd2 [string]", "2nd line of 'Message Of The Day'"},
{sv_motd3, "sv_motd3", "sv_motd3 [string]", "3rd line of 'Message Of The Day'"},
{sv_motd4, "sv_motd4", "sv_motd4 [string]", "4th line of 'Message Of The Day'"},
{sv_motd5, "sv_motd5", "sv_motd5 [string]", "5th line of 'Message Of The Day'"},
{sv_motd6, "sv_motd6", "sv_motd6 [string]", "6th line of 'Message Of The Day'"},
{sv_motd7, "sv_motd7", "sv_motd7 [string]", "7th line of 'Message Of The Day'"},
{sv_motd8, "sv_motd8", "sv_motd8 [string]", "8th line of 'Message Of The Day'"},
{sv_motd9, "sv_motd9", "sv_motd9 [string]", "9th line of 'Message Of The Day'"},
{vd_bmpmovie, "vd_bmpmovie", "vd_bmpmovie [boolean]",
	"takes screenshot every 40ms. You need to have much disk space."},
{sv_statusfile, "sv_statusfile", "sv_statusfile [string]", "Sets filename to"
	" save server's status"},
{sv_status, "sv_status", "sv_status [boolean]", "Sets if server's status "
	"should be saved in file"},
{cl_linewidth, "cl_linewidth", "cl_linewidth [integer]", "Sets width of the "
	"lines"},
{cl_water, "cl_water", "cl_water [integer]", "Switches waving water"},
{cl_background, "cl_background", "cl_nackground [boolean]", "Switches "
	"textured background"},
{rc_svpasswd, "rc_svpasswd", "rc_svpasswd [string]", "Remote console password "
	"on the server's side"},
{rc_svdelpasswd, "rc_svdelpasswd", "rc_svdelpasswd", "Deletes "
	"server's side remote console password"},
{rc_clpasswd, "rc_clpasswd", "rc_clpasswd [string]", "Remote console password "
	"on the client's side"},
{rc_cldelpasswd, "rc_cldelpasswd", "rc_cldelpasswd", "Deletes "
	"server's side remote console password"}
};

static const int cmdsCount = sizeof(cmds)/sizeof(struct _cmds);

static void usage(const char *command)
{
	int p;

	for (p=0; p<cmdsCount; p++)
		if (strcmp(command, cmds[p].name) == 0)
		{
			console_print("Usage: %s", cmds[p].usage);
			console_print(cmds[p].desc);
			return;
		}
	return;
}

static int setStrVar(char *var, char *name, char *arg, char *def)
{
	if (strcmp(arg, "") == 0)
	{
		console_print("%s is \"%s^0\", default: \"%s^0\"",
			name, var, def);
		return 0;
	}
	else
	{
		strncpy(var, arg, 81);
		return 1;
	}
}

static void setIntVar(int *var, char *name, char *arg, int def,
	int min, int max)
{
	int number;
	char *endptr;

	if (strcmp(arg, "") == 0)
	{
		console_print("%s is %d, default: %d",
			name, *var, def);
	}
	else
	{
		number = strtol(arg, &endptr, 10);

		if (*endptr)
			usage(name);
		else
			if ((number < min) || (number > max))
				console_print("%s range is %d-%d",
					name, min, max);
			else
				*var = number;
	}
	return;
}
#ifndef DED
static void vd_reload     (char *arg)
{
	int flags;

	console_print("Unloading textures...");
	ui_unloadTextures();
	console_unloadTextures();
	if (ui_clientRunning)
		client_unloadTextures();
	console_print("^2DONE");

	SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 5);
	SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 5);
	SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 5);
	SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 16);
	SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);

	if (commands_vd_vsync)
		SDL_GL_SetAttribute(SDL_GL_SWAP_CONTROL, 1);
	else
		SDL_GL_SetAttribute(SDL_GL_SWAP_CONTROL, 0);
	
	if (commands_vd_fullscreen)
		flags = SDL_OPENGL | SDL_FULLSCREEN;
	else
		flags = SDL_OPENGL;
	
	console_print("Setting the video mode...");
	if (SDL_SetVideoMode(commands_vd_width,
		commands_vd_height, 0, flags)== NULL)
		console_print("^1FAILED!!!");
	else
	{
		video_setupOGL(commands_vd_width, commands_vd_height);
		console_print("^2DONE");
	}

	console_print("Reloading textures...");
	ui_loadTextures();
	console_loadTextures();
	if (ui_clientRunning)
		client_loadTextures();
	console_print("^2DONE");

	return;
}
static void say(char *arg)
{
	if (strlen(arg))
		client_say(arg);
	return;
}
static void quit(char *arg)
{
	ui_quitNow = 1;
	return;
}
static void name(char *arg)
{
	if (setStrVar(commands_name, "name", arg, DEFAULT_NAME))
		client_name(commands_name);
	return;
}
static void rc_cmd(char *arg)
{
	if (strlen(arg))
		client_rcQueue(arg);
	else
		help("rc_cmd");
	return;
}
static void rc_restart(char *arg)
{
	client_rcRestart();
	return;
}
static void rc_kick(char *arg)
{
	if (strlen(arg))
		client_rcKick(arg);
	else
		help("rc_kick");
	return;
}
#endif

static void help(char *arg)
{
	int p;
	char notFound;

	if (strcmp(arg, "") == 0)
	{
		console_print("Here will be the overall help");
	}
	else
	{
		notFound = 1;
		for (p=0; p<cmdsCount; p++)
			if (strcmp(arg, cmds[p].name) == 0)
			{
				usage(cmds[p].name);
				notFound = 0;
				break;
			}
		if (notFound)
			console_print("No such command: %s", arg);
	}
	return;
}
static void echo(char *arg)
{
	console_print(arg);
	return;
}
static void clear(char *arg)
{
	console_preInit();
	return;
}
static void version(char *arg)
{
	console_print(VERSION);
	return;
}
static void save(char *arg)
{
	commands_saveConfig();
	return;
}
static void co_blink(char *arg)
{
	setIntVar(&commands_co_blink, "co_blink", arg, DEFAULT_CO_BLINK,
		0, 200);
	return;
}
static void co_alpha(char *arg)
{
	setIntVar(&commands_co_alpha, "co_alpha", arg, DEFAULT_CO_ALPHA,
		0, 255);
	return;
}
static void vd_width      (char *arg)
{
	setIntVar(&commands_vd_width, "vd_width", arg, DEFAULT_VD_WIDTH,
		1, 7680);
	return;
}
static void vd_height     (char *arg)
{
	setIntVar(&commands_vd_height, "vd_height", arg, DEFAULT_VD_HEIGHT,
		1, 4800);
	return;
}
static void vd_fullscreen (char *arg)
{
	setIntVar(&commands_vd_fullscreen, "vd_fullscreen", arg,
		DEFAULT_VD_FULLSCREEN, 0, 1);
	return;
}
static void vd_vsync (char *arg)
{
	setIntVar(&commands_vd_vsync, "vd_vsync", arg,
		DEFAULT_VD_VSYNC, 0, 1);
	return;
}
static void sv_max_clients(char *arg)
{
	setIntVar(&commands_sv_max_clients, "sv_max_clients", arg,
		DEFAULT_SV_MAX_CLIENTS, 0, 255);
}
static void sv_max_players(char *arg)
{
	setIntVar(&commands_sv_max_players, "sv_max_players", arg,
		DEFAULT_SV_MAX_PLAYERS, 1, 6);
}
static void sv_name(char *arg)
{
	setStrVar(commands_sv_name, "sv_name", arg, DEFAULT_SV_NAME);
	return;
}
static void sv_passwd(char *arg)
{
	setStrVar(commands_sv_passwd, "sv_passwd", arg, DEFAULT_SV_PASSWD);
	return;
}
static void sv_delpasswd(char *arg)
{
	commands_sv_passwd[0] = 0;
	return;
}
static void cl_passwd(char *arg)
{
	setStrVar(commands_cl_passwd, "cl_passwd", arg, DEFAULT_CL_PASSWD);
	return;
}
static void cl_delpasswd(char *arg)
{
	commands_cl_passwd[0] = 0;
	return;
}
static void sv_map(char *arg)
{
	setStrVar(commands_sv_map, "sv_map", arg, DEFAULT_SV_MAP);
	return;
}
static void sv_balls(char *arg)
{
	setIntVar(&commands_sv_balls, "sv_balls", arg,
		DEFAULT_SV_BALLS, 1, 4);
}
static void cl_hostname(char *arg)
{
	setStrVar(commands_cl_hostname, "cl_hostname", arg,
		DEFAULT_CL_HOSTNAME);
	return;
}
static void sv_hostname(char *arg)
{
	setStrVar(commands_sv_hostname, "sv_hostname", arg,
		DEFAULT_SV_HOSTNAME);
	return;
}
static void sv_port(char *arg)
{
	setIntVar(&commands_sv_port, "sv_port", arg,
		DEFAULT_SV_PORT, 1, 65535);
}
static void cl_port(char *arg)
{
	setIntVar(&commands_cl_port, "cl_port", arg,
		DEFAULT_CL_PORT, 1, 65535);
}
static void sv_timelimit(char *arg)
{
	setIntVar(&commands_sv_timelimit, "sv_timelimit", arg,
		DEFAULT_SV_TIMELIMIT, 5, 60);
}
static void sv_gametype(char *arg)
{
	setIntVar(&commands_sv_gametype, "sv_gametype", arg,
		DEFAULT_SV_GAMETYPE, 0, 4);
}
static void sv_fall(char *arg)
{
	setIntVar(&commands_sv_fall, "sv_fall", arg,
		DEFAULT_SV_FALL, 0, 1);
}
static void sv_health(char *arg)
{
	setIntVar(&commands_sv_health, "sv_health", arg,
		DEFAULT_SV_HEALTH, 1, 999);
}
static void db_frametime (char *arg)
{
	setIntVar(&commands_db_frametime, "db_frametime", arg,
		DEFAULT_DB_FRAMETIME, 0, 1);
	return;
}
static void sv_motd1      (char *arg)
{
	setStrVar(commands_sv_motd1, "sv_motd1", arg, DEFAULT_SV_MOTD1);
	return;
}
static void sv_motd2      (char *arg)
{
	setStrVar(commands_sv_motd2, "sv_motd2", arg, DEFAULT_SV_MOTD2);
	return;
}
static void sv_motd3      (char *arg)
{
	setStrVar(commands_sv_motd3, "sv_motd3", arg, DEFAULT_SV_MOTD3);
	return;
}
static void sv_motd4      (char *arg)
{
	setStrVar(commands_sv_motd4, "sv_motd4", arg, DEFAULT_SV_MOTD4);
	return;
}
static void sv_motd5      (char *arg)
{
	setStrVar(commands_sv_motd5, "sv_motd5", arg, DEFAULT_SV_MOTD5);
	return;
}
static void sv_motd6      (char *arg)
{
	setStrVar(commands_sv_motd6, "sv_motd6", arg, DEFAULT_SV_MOTD6);
	return;
}
static void sv_motd7      (char *arg)
{
	setStrVar(commands_sv_motd7, "sv_motd7", arg, DEFAULT_SV_MOTD7);
	return;
}
static void sv_motd8      (char *arg)
{
	setStrVar(commands_sv_motd8, "sv_motd8", arg, DEFAULT_SV_MOTD8);
	return;
}
static void sv_motd9      (char *arg)
{
	setStrVar(commands_sv_motd9, "sv_motd9", arg, DEFAULT_SV_MOTD9);
	return;
}
static void vd_bmpmovie   (char *arg)
{
	setIntVar(&commands_vd_bmpmovie, "vd_bmpmovie", arg,
		DEFAULT_VD_BMPMOVIE, 0, 1);
	return;
}
static void sv_statusfile (char *arg)
{
	setStrVar(commands_sv_statusfile, "sv_statusfile", arg,
		DEFAULT_SV_STATUSFILE);
	return;
}
static void sv_status     (char *arg)
{
	setIntVar(&commands_sv_status, "sv_status", arg,
		DEFAULT_SV_STATUS, 0, 1);
	return;
}

static void cl_linewidth  (char *arg)
{
	setIntVar(&commands_cl_linewidth, "cl_linewidth", arg,
		DEFAULT_CL_LINEWIDTH, 1, 5);
	return;
}

static void cl_water      (char *arg)
{
	setIntVar(&commands_cl_water, "cl_water", arg,
		DEFAULT_CL_WATER, 0, 1);
	return;
}
static void cl_background (char *arg)
{
	setIntVar(&commands_cl_background, "cl_background", arg,
		DEFAULT_CL_BACKGROUND, 0, 1);
	return;
}
static void rc_svpasswd   (char *arg)
{
	setStrVar(commands_rc_svpasswd, "rc_svpasswd", arg,
		DEFAULT_RC_SVPASSWD);
	return;
}
static void rc_svdelpasswd(char *arg)
{
	commands_rc_svpasswd[0] = 0;
	return;
}
static void rc_clpasswd   (char *arg)
{
	setStrVar(commands_rc_clpasswd, "rc_clpasswd", arg,
		DEFAULT_RC_CLPASSWD);
	return;
}
static void rc_cldelpasswd(char *arg)
{
	commands_rc_clpasswd[0] = 0;
	return;
}

/* load the config file */
int commands_loadConfig(void)
{
	FILE *config;
	char filenameBuf[256];

	console_print("Loading config...");

#ifdef WIN32
	snprintf(filenameBuf, 256, commands_configFile);
#else
	snprintf(filenameBuf, 256, "%s/.freeshoppa/%s",
		getenv("HOME"), commands_configFile);
#endif

	config = fopen(filenameBuf, "r");
	if (config)
	{
		char buffer[256]; /* just to be careful */
		while (fgets(buffer, 256, config) != NULL)
		{
			if (buffer[0] == '#')
				continue;

			if (strlen(buffer) > 0 &&
			buffer[strlen(buffer)-1] == '\n')
				buffer[strlen(buffer)-1] = 0;
			commands_process(buffer);
		}
		
		fclose(config);
		console_print("^2DONE");
		return 0;
	}
	else
	{
		console_print("  commands: failed to load config file.");
		console_print("^1FAILED!!!");
		return 1;
	}

}

/* save the config file */
int commands_saveConfig(void)
{
	FILE *config;
	char filenameBuf[256];

	console_print("Saving config...");
#ifdef WIN32
	snprintf(filenameBuf, 256, commands_configFile);
	config = fopen(filenameBuf, "w");
#else
	snprintf(filenameBuf, 256, "%s/.freeshoppa/%s",
		getenv("HOME"), commands_configFile);
	config = fopen(filenameBuf, "w");
	if (!config)
	{
		snprintf(filenameBuf, 256, "%s/.freeshoppa",
			getenv("HOME"));
		mkdir(filenameBuf, 0755);

		snprintf(filenameBuf, 256, "%s/.freeshoppa/%s",
			getenv("HOME"), commands_configFile);
		config = fopen(filenameBuf, "w");
	}
#endif

	if (config)
	{
		fprintf(config, "# Generated by freeshoppa. Do not modify"
			" unless you know what you're doing.\n");
		fprintf(config, "name %s\n", commands_name);
		fprintf(config, "co_blink %d\n", commands_co_blink);
		fprintf(config, "co_alpha %d\n", commands_co_alpha);
		fprintf(config, "vd_width %d\n", commands_vd_width);
		fprintf(config, "vd_height %d\n", commands_vd_height);
		fprintf(config, "vd_fullscreen %d\n", commands_vd_fullscreen);
		fprintf(config, "vd_vsync %d\n", commands_vd_vsync);
		fprintf(config, "sv_max_clients %d\n", commands_sv_max_clients);
		fprintf(config, "sv_max_players %d\n", commands_sv_max_players);
		fprintf(config, "sv_name %s\n", commands_sv_name);
		if (commands_sv_passwd[0] != 0)
			fprintf(config, "sv_passwd %s\n", commands_sv_passwd);
		if (commands_cl_passwd[0] != 0)
			fprintf(config, "cl_passwd %s\n", commands_cl_passwd);
		fprintf(config, "sv_map %s\n", commands_sv_map);
		fprintf(config, "sv_balls %d\n", commands_sv_balls);
		fprintf(config, "cl_hostname %s\n", commands_cl_hostname);
		fprintf(config, "sv_hostname %s\n", commands_sv_hostname);
		fprintf(config, "sv_port %d\n", commands_sv_port);
		fprintf(config, "cl_port %d\n", commands_cl_port);
		fprintf(config, "sv_timelimit %d\n", commands_sv_timelimit);
		fprintf(config, "sv_gametype %d\n", commands_sv_gametype);
		fprintf(config, "sv_fall %d\n", commands_sv_fall);
		fprintf(config, "sv_health %d\n", commands_sv_health);
		fprintf(config, "db_frametime %d\n", commands_db_frametime);
		fprintf(config, "sv_motd1 %s\n", commands_sv_motd1);
		fprintf(config, "sv_motd2 %s\n", commands_sv_motd2);
		fprintf(config, "sv_motd3 %s\n", commands_sv_motd3);
		fprintf(config, "sv_motd4 %s\n", commands_sv_motd4);
		fprintf(config, "sv_motd5 %s\n", commands_sv_motd5);
		fprintf(config, "sv_motd6 %s\n", commands_sv_motd6);
		fprintf(config, "sv_motd7 %s\n", commands_sv_motd7);
		fprintf(config, "sv_motd8 %s\n", commands_sv_motd8);
		fprintf(config, "sv_motd9 %s\n", commands_sv_motd9);
		fprintf(config, "sv_statusfile %s\n", commands_sv_statusfile);
		fprintf(config, "sv_status %d\n", commands_sv_status);
		fprintf(config, "cl_linewidth %d\n", commands_cl_linewidth);
		fprintf(config, "cl_water %d\n", commands_cl_water);
		fprintf(config, "cl_background %d\n", commands_cl_background);
		if (commands_rc_svpasswd[0] != 0)
			fprintf(config, "rc_svpasswd %s\n",
				commands_rc_svpasswd);
		if (commands_rc_clpasswd[0] != 0)
			fprintf(config, "rc_clpasswd %s\n",
				commands_rc_clpasswd);

		fclose(config);
		console_print("^2DONE");
		return 0;
	}
	else
	{
		console_print("  commands: failed to save config file.");
		console_print("^1FAILED!!!");
		return 1;
	}
}

/* process command given by the console */
void commands_process(char *command)
{
	char cmd[81];
	char arg[81];
	char *spacebar;
	int p;
	char notFound;

	memset(cmd, 0, 81);
	memset(arg, 0, 81);

	spacebar = strchr(command, ' ');

	/* command length limit to 40 characters */
	if ((spacebar-command) > 40)
	{
		console_print("  commands: command is longer than"
			" 40 characters.");
		return;
	}

	if (spacebar == NULL)
	{
		memcpy(cmd, command, 81);
	}
	else
	{
		memcpy(cmd, command, spacebar-command);
		memcpy(arg, spacebar+1, 80-(spacebar-command));
	}

	notFound = 1;
	for (p=0; p<cmdsCount; p++)
		if (strcmp(cmd, cmds[p].name) == 0)
		{
			cmds[p].ptr(arg);
			notFound = 0;
			break;
		}
	if (notFound)
		console_print("No such command: %s", cmd);

	return;
}

/* when you press TAB in the console */
void commands_completion(char *input, int *current)
{
	int p;
	int count;
	int results[sizeof(cmds)/sizeof(struct _cmds)];

	count = 0;
	for (p=0; p<cmdsCount; p++)
		if (strncmp(input, cmds[p].name, strlen(input)) == 0)
		{
			results[count] = p;
			count++;
		}
	if (count == 1)
	{
		snprintf(input, 81, "%s ", cmds[results[0]].name);
		*current = strlen(input);
		return;
	}
	else if (count != 0)
	{
		char buf[81];
		int p, q;

		console_print("^4What do you mean?");
		for (p=0; p<count; p++)
			console_print(cmds[results[p]].name);

		memset(buf, 0, 81);


		for (q=0; q<81; q++)
		{
			for (p=1; p<count; p++)
			{
				if (cmds[results[p]].name[q] !=
					cmds[results[0]].name[q])
				{
					goto done;
				}
			}
		}
done:
		strncpy(input, cmds[results[0]].name, q);
		*current = q;
	}

	return;
}
