/*
 *  (C) Copyright 2007-2009 by FreeShoppa Development Team
 *                        (http://chomzee.ethernet.pl/freeshoppa/)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <GL/gl.h>
#ifdef WIN32
#include <gl\glext.h>
#endif

#include "freeshoppa.h"

#define MAX_CONSOLE_LINES 1000
#define MAX_MESSAGES_LINES 5

int console_texFont = 0; /* console font */
static int texBackground = 0; /* background texture */

char console_active = 0;

/* buffer for console chars */
static char consoletext[MAX_CONSOLE_LINES][81];
static int consoletextCurrent;
static int consoletextView;

static char consoleinput[81];
static int consoleinputCurrent;

static char consolehistory[81];
static int consolehistoryCurrent;

/* counter for blinking cursor */
static int cursor = 0;

/* messages */
static char messages[MAX_MESSAGES_LINES][81];
static int messagesCount;
static Uint32 messagesTime[MAX_MESSAGES_LINES];

/* chatter */
static char chatter[81];
static int chatterCurrent;
char console_chatterActive;

/* prepare and clean the console */
void console_preInit()
{
	memset(consoletext, 0, 81*MAX_CONSOLE_LINES);
	consoletextCurrent = 0;
	consoletextView = 0;

	memset(consoleinput, 0, 81);
	consoleinputCurrent = 0;

	memset(chatter, 0, 81);
	chatterCurrent = 0;
	console_chatterActive = 0;

	return;
}

void console_loadTextures()
{
	console_texFont = fonts_loadFont("graphics/fonts/console.bmp");
	texBackground =
		textures_loadBMP("graphics/console/background.bmp");
	return;
}
void console_unloadTextures()
{
	fonts_unloadFont(&console_texFont);
	textures_unloadBMP(&texBackground);
	return;
}

/* init console (load texture) */
int console_init(void)
{
	console_print("Initializing console...");

	console_loadTextures();
	console_print("^2DONE");
	return 0;
}

/* print data to stdout and to ingame console */
void console_print(const char *text, ...)
{
	va_list argptr, argptr2;
	va_start(argptr, text);
	va_start(argptr2, text);

	vprintf(text, argptr);
	printf("\n");
	vsnprintf(consoletext[consoletextCurrent], 81, text, argptr2);

	if (consoletextCurrent == consoletextView)
	{
		consoletextView ++;
		if (consoletextView == MAX_CONSOLE_LINES)
			consoletextView = 0;
	}
	consoletextCurrent ++;
	if (consoletextCurrent == MAX_CONSOLE_LINES)
		consoletextCurrent = 0;
	
	va_end(argptr);
	va_end(argptr2);

	return;
}

/* clear messages panel */
void console_messagesClear(void)
{
	memset(messages, 0, 81*MAX_MESSAGES_LINES);
	messagesCount = 0;
	memset(messagesTime, 0, sizeof(Uint32)*MAX_MESSAGES_LINES);

	return;
}

/* print data to messages panel */
void console_messagePrint(const char *text, ...)
{
	va_list argptr;
	int p;

	va_start(argptr, text);

	if (messagesCount == MAX_MESSAGES_LINES)
	{
		for (p=0; p<MAX_MESSAGES_LINES-1; p++)
			memcpy(messages[p], messages[p+1], 81);
		for (p=0; p<MAX_MESSAGES_LINES-1; p++)
			memcpy(&messagesTime[p], &messagesTime[p+1],
				sizeof(Uint32));
		messagesCount--;
	}

	vsnprintf(messages[messagesCount], 81, text, argptr);
	messagesTime[messagesCount] = SDL_GetTicks();
	messagesCount++;

	va_end(argptr);
	return;
}

/* makes the console active or not */
void console_swap(void)
{
	console_active = !console_active;

	if (console_active)
	{
		SDL_ShowCursor(SDL_ENABLE);
		SDL_WM_GrabInput(SDL_GRAB_OFF);
		SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY,
			SDL_DEFAULT_REPEAT_INTERVAL);
	}
	else
	{
		SDL_ShowCursor(SDL_DISABLE);
		SDL_WM_GrabInput(SDL_GRAB_ON);
		if (ui_clientRunning && !console_chatterActive)
			SDL_EnableKeyRepeat(0, 0);
	}
	
	return;
}

/* do the drawing stuff */
void console_draw(void)
{
	int p, q, r, s;

	/* prepare everything... */
	glDisable(GL_DEPTH_TEST);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glOrtho(0.0-video_xOffset, 800.0+video_xOffset,
		0.0-video_yOffset, 600.0+video_yOffset, -1.0f, 1.0f);
	glPushMatrix();
	glTranslatef(0.0f, video_yOffset, 0.0f);

	/* let's draw! */
	glColor4f(1.0f, 1.0f, 1.0f, (float)commands_co_alpha/255.0f);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, texBackground);

	glBegin(GL_TRIANGLES);

	glTexCoord2f(0.0f, 1.0f);
	glVertex2f(0.0f, 600.0f);
	glTexCoord2f(0.0f, 0.25f);
	glVertex2f(0.0f, 300.0f);
	glTexCoord2f(1.0f, 0.25f);
	glVertex2f(800.0f, 300.0f);

	glTexCoord2f(0.0f, 1.0f);
	glVertex2f(0.0f, 600.0f);
	glTexCoord2f(1.0f, 0.25f);
	glVertex2f(800.0f, 300.0f);
	glTexCoord2f(1.0f, 1.0f);
	glVertex2f(800.0f, 600.0f);
	glEnd();

	/* draw chars */

	p = consoletextView;

	for (q=0; q<18; q++)
	{
		p--;
		if (p == -1)
			p = MAX_CONSOLE_LINES-1;

		glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
		s = 0;
		for (r=0; r<80; r++)
		{
			if (consoletext[p][r] == 0)
				break;

			/* Console colors */

			/* white */
			if (consoletext[p][r] == '^' &&
				consoletext[p][r+1] == '0')
			{
				r++;
				glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
				continue;
			}
			/* red */
			if (consoletext[p][r] == '^' &&
				consoletext[p][r+1] == '1')
			{
				r++;
				glColor4f(1.0f, 0.0f, 0.0f, 1.0f);
				continue;
			}
			/* green */
			if (consoletext[p][r] == '^' &&
				consoletext[p][r+1] == '2')
			{
				r++;
				glColor4f(0.0f, 1.0f, 0.0f, 1.0f);
				continue;
			}
			/* blue */
			if (consoletext[p][r] == '^' &&
				consoletext[p][r+1] == '3')
			{
				r++;
				glColor4f(0.0f, 0.0f, 1.0f, 1.0f);
				continue;
			}
			/* yellow */
			if (consoletext[p][r] == '^' &&
				consoletext[p][r+1] == '4')
			{
				r++;
				glColor4f(1.0f, 1.0f, 0.0f, 1.0f);
				continue;
			}
			/* cyan */
			if (consoletext[p][r] == '^' &&
				consoletext[p][r+1] == '5')
			{
				r++;
				glColor4f(0.0f, 1.0f, 1.0f, 1.0f);
				continue;
			}
			/* magenta */
			if (consoletext[p][r] == '^' &&
				consoletext[p][r+1] == '6')
			{
				r++;
				glColor4f(1.0f, 0.0f, 1.0f, 1.0f);
				continue;
			}
			/* black */
			if (consoletext[p][r] == '^' &&
				consoletext[p][r+1] == '7')
			{
				r++;
				glColor4f(0.0f, 0.0f, 0.0f, 1.0f);
				continue;
			}
			fonts_drawChar(console_texFont, consoletext[p][r],
				s*10.0f, 10.0f+ s*10.0f,
				330.0f + q*15.0f, 345.0f + q*15.0f);
			s++;
		}
	}

	/* prompt */
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	fonts_drawChar(console_texFont, '#', 0.0f, 10.0f, 300.0f, 315.0f);

	/* input line */
	for (q=0; q<consoleinputCurrent; q++)
	{
		fonts_drawChar(console_texFont, consoleinput[q],
			20.0f + q*10.0f, 30.0f+ q*10.0f,
			300.0f, 315.0f);
	}
	
	
	glColor4f(1.0f, 1.0f, 0.0f, 1.0f);
	/* (more) */
	if (consoletextView != consoletextCurrent)
	for (q=0; q<strlen("(more)"); q++)
	{
		fonts_drawChar(console_texFont, "(more)"[q],
			q*10.0f, 10.0f+ q*10.0f,
			315.0f, 330.0f);
	}
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

	/* version */
	for (q=0; q<strlen(VERSION); q++)
	{
		fonts_drawChar(console_texFont, VERSION[strlen(VERSION)-q-1],
			790.0f -q*10.0f,
			800.0f -q*10.0f,
			315.0f, 330.0f);
	}

	/* blinking cursor */
	
	if((commands_co_blink == 0) || (!((cursor/commands_co_blink) % 2)))
		fonts_drawChar(console_texFont, '_',
		20.0f+(10.0f*consoleinputCurrent),
		30.0f+(10.0f*consoleinputCurrent), 300.0f, 315.0f);

	cursor ++;

	glDisable(GL_TEXTURE_2D);
	/* end of drawing chars */

	glPopMatrix();
	return;
}

/* do the drawing stuff */
void console_messagesDraw(void)
{
	int p, r, s;

	/* Before we draw, we have to check if we can delete some lines
	 * because of messages visibility time limit */
	if (messagesCount > 0 && SDL_GetTicks()-messagesTime[0] > 5000)
	{
		for (p=0; p<MAX_MESSAGES_LINES-1; p++)
			memcpy(messages[p], messages[p+1], 81);
		memset(messages[MAX_MESSAGES_LINES-1], 0, 81);

		for (p=0; p<MAX_MESSAGES_LINES-1; p++)
			memcpy(&messagesTime[p], &messagesTime[p+1],
				sizeof(Uint32));
		messagesTime[MAX_MESSAGES_LINES-1] = 0;

		messagesCount--;
	}

	/* prepare everything... */
	glDisable(GL_DEPTH_TEST);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glOrtho(0.0-video_xOffset, 800.0+video_xOffset,
		0.0-video_yOffset, 600.0+video_yOffset, -1.0f, 1.0f);
	glPushMatrix();
	glTranslatef(0.0f, -video_yOffset, 0.0f);

	/* draw chars */
	glEnable(GL_TEXTURE_2D);

	for (p=0; p<MAX_MESSAGES_LINES; p++)
	{
		glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
		s = 0;
		for (r=0; r<80; r++)
		{
			if (messages[p][r] == 0)
				break;

			/* Console colors */

			/* white */
			if (messages[p][r] == '^' &&
				messages[p][r+1] == '0')
			{
				r++;
				glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
				continue;
			}
			/* red */
			if (messages[p][r] == '^' &&
				messages[p][r+1] == '1')
			{
				r++;
				glColor4f(1.0f, 0.0f, 0.0f, 1.0f);
				continue;
			}
			/* green */
			if (messages[p][r] == '^' &&
				messages[p][r+1] == '2')
			{
				r++;
				glColor4f(0.0f, 1.0f, 0.0f, 1.0f);
				continue;
			}
			/* blue */
			if (messages[p][r] == '^' &&
				messages[p][r+1] == '3')
			{
				r++;
				glColor4f(0.0f, 0.0f, 1.0f, 1.0f);
				continue;
			}
			/* yellow */
			if (messages[p][r] == '^' &&
				messages[p][r+1] == '4')
			{
				r++;
				glColor4f(1.0f, 1.0f, 0.0f, 1.0f);
				continue;
			}
			/* cyan */
			if (messages[p][r] == '^' &&
				messages[p][r+1] == '5')
			{
				r++;
				glColor4f(0.0f, 1.0f, 1.0f, 1.0f);
				continue;
			}
			/* magenta */
			if (messages[p][r] == '^' &&
				messages[p][r+1] == '6')
			{
				r++;
				glColor4f(1.0f, 0.0f, 1.0f, 1.0f);
				continue;
			}
			/* black */
			if (messages[p][r] == '^' &&
				messages[p][r+1] == '7')
			{
				r++;
				glColor4f(0.0f, 0.0f, 0.0f, 1.0f);
				continue;
			}
			fonts_drawChar(console_texFont, messages[p][r],
				100.0f + s*10.0f, 110.0f+ s*10.0f,
				15.0f*(MAX_MESSAGES_LINES-p),
				15.0f + 15.0f*(MAX_MESSAGES_LINES-p));
			s++;
		}

	}
	glPopMatrix();
	/* chatter */
	glPushMatrix();
	glTranslatef(-video_xOffset, video_yOffset, 0.0f);
	if (console_chatterActive)
	{
		int q;
		glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
		
		fonts_drawChar(console_texFont, 's',
			0.0f, 10.0f, 585.0f, 600.0f);
		fonts_drawChar(console_texFont, 'a',
			10.0f, 20.0f, 585.0f, 600.0f);
		fonts_drawChar(console_texFont, 'y',
			20.0f, 30.0f, 585.0f, 600.0f);
		fonts_drawChar(console_texFont, ':',
			30.0f, 40.0f, 585.0f, 600.0f);

		for (q=0; q<chatterCurrent; q++)
		{

			fonts_drawChar(console_texFont, chatter[q],
				50.0f + q*10.0f, 60.0f+ q*10.0f,
				585.0f, 600.0f);
		}
	}
	glDisable(GL_TEXTURE_2D);
	/* end of drawing chars */
	glPopMatrix();

	return;
}

/* handle keyboard */
void console_chatterKeyDown(SDLKey key)
{
	if (key == SDLK_ESCAPE)
	{
		console_chatterActive = 0;
		SDL_EnableKeyRepeat(0, 0);
	}
	
	if ((key == SDLK_RETURN) && chatterCurrent)
	{
		memcpy(consolehistory, chatter, 81);
		consolehistoryCurrent = chatterCurrent;
	}

	misc_keyboardInput(key, chatter, &chatterCurrent, 74, 1);

	if ((key == SDLK_RETURN))
	{
		input_keys[SDLK_RETURN] = 0;
		memcpy(consolehistory, chatter, 81);
		consolehistoryCurrent = chatterCurrent;

		if (chatterCurrent)
			client_say(chatter);
		memset(chatter, 0, 81);
		chatterCurrent = 0;
		console_chatterActive = 0;
		SDL_EnableKeyRepeat(0, 0);
	}
	
	return;
}

/* handle keyboard */
void console_keyDown(SDLKey key)
{
	if (key == SDLK_ESCAPE)
		console_swap();
	if ((key == SDLK_RETURN) && consoleinputCurrent)
	{
		memcpy(consolehistory, consoleinput, 81);
		consolehistoryCurrent = consoleinputCurrent;
	}

	misc_keyboardInput(key, consoleinput, &consoleinputCurrent, 78, 1);

	if ((key == SDLK_RETURN) && consoleinputCurrent)
	{
		memcpy(consolehistory, consoleinput, 81);
		consolehistoryCurrent = consoleinputCurrent;

		console_print("# %s", consoleinput);
		commands_process(consoleinput);
		memset(consoleinput, 0, 81);
		consoleinputCurrent = 0;
	}
	if (key == SDLK_TAB)
		commands_completion(consoleinput, &consoleinputCurrent);
	if (key == SDLK_UP)
	{
		memcpy(consoleinput, consolehistory, 81);
		consoleinputCurrent = consolehistoryCurrent;
	}
	if (key == SDLK_PAGEUP)
	{
		consoletextView --;
		if (consoletextView == -1)
			consoletextView = MAX_CONSOLE_LINES;
	}
	if (key == SDLK_PAGEDOWN)
	{
		if (consoletextView != consoletextCurrent)
		{
			consoletextView ++;
			if (consoletextView == MAX_CONSOLE_LINES)
				consoletextView = 0;
		}
	}
	if (key == SDLK_END)
	{
		consoletextView = consoletextCurrent;
	}

	cursor = 0;
	
	return;
}

/* unload texture */
void console_cleanup(void)
{
	console_print("Cleaning console...");

	console_loadTextures();

	console_print("^2DONE");

	return;
}
