/*
 *  (C) Copyright 2007-2009 by FreeShoppa Development Team
 *                        (http://chomzee.ethernet.pl/freeshoppa/)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include "freeshoppa.h"

#define CONSTMOVE 0.02f
#define CONSTROPE 0.75f
#define GRAVITY 15.0f
#define WATER 5.0f
#define WATER2 0.04f
#define WATERX 1.02f
#define JUMP 350.0f
#define XJUMP 75.0f
#define LITTLEJUMP 200.0f
#define SPRING 0.5f
#define ROPESPRING 1.0f
#define AIMSPEED 3.0f
#define ROPESPEED 22.0f
#define ROPEMAXLEN 500.0f
#define ROPEMINLEN 20.0f
#define ROPEMOVESPEED 5.0f
#define SWING 75.0f
#define SWINGBOOST 1.1f
#define MASS 7.0f
#define RESISTANCE 0.011f
#define WINDZOOKA 0.16f
#define WINDCLUSTER 0.1f
#define BOOMCONST 6.1f
#define BOOMYVEL 1.0f
#define DAMAGE 0.1f
#define FALL 0.125f
#define MINFALL 265.0f
#define HAMSTERSPRING 0.25f
#define HAMSTERSPEED 100.0f
#define HAMSTERJUMP 120.0f
#define SLOPEBUGFIX 1.0f
#define ROTATION 0.1f

char game_keys[4000][8];

int game_frame; /* number of frame playing */
int game_prepare; /* 'prepare for turn' counter */

char game_side; /* 0 -- left
		 * 1 -- right */
float game_aim; /* rope aiming: -90.0f to 90.0f */

char game_rope;
float game_ropeang; /* rope angle */
float game_ropelen; /* rope length */

char game_hooked; /* 1 = rope hooked, 0 = rope not hooked */
Vector game_hookPos;
char game_roperep;

float game_omega; /* angular velocity */

char game_drop; /* 1 = weapon is on the map */
int game_dropType; /* type of weapon */
IntVector game_dropPos;

int game_wind; /* wind <-100.0f; 100.0f> */

Weapon game_weap[MAXCLUSTERS];

static char end;
int game_preparetoend; /* wait 5 seconds after every end of turn */

int game_escape; /* 10 seconds to escape after using a weapon */

static int fuseTime;
static int colIndex;
char game_walls[MAXELEMENTSCOUNT];
int game_walls2go;
static char flight;
static char flightSide;

static void prepareTurn()
{
	int p, q;

	/* clear velocities */
	for (p=0; p<6; p++)
		for (q=0; q<4; q++)
		{
			client_players[p].ball[q].vel.x = 0.0f;
			client_players[p].ball[q].vel.y = 0.0f;

			client_players[p].ball[q].fall = 0; /* falling damage
							     * bug */
			client_players[p].ball[q].water = 0;
		}
	game_side = 0;
	client_players[client_turn].ball[client_turnBall].eyes = 0;
	game_aim = 0.0f;

	game_rope = 0;
	game_ropeang = 0.0f;
	game_ropelen = 0.0f;

	game_hooked = 0;
	game_hookPos.x = 0.0f;
	game_hookPos.y = 0.0f;
	game_roperep = 0;

	for (p=0; p<MAXCLUSTERS; p++)
	{
		game_weap[p].status = 0;
		game_weap[p].boom = 0;
		game_weap[p].fuse = 0;
		game_weap[p].side = 0;
		game_weap[p].rotation = 0.0f;
		game_weap[p].sndtime = 0;
	}

	memset(game_walls, 0, MAXELEMENTSCOUNT);
	game_walls2go = client_map->wallsCount;

	game_escape = 0;
	fuseTime = 3;

	flight = 0;
	if (client_players[client_turn].ball[client_turnBall].pos.x > 0.0f)
		flightSide = 1;
	else
		flightSide = 0;

	if (client_gametype != 4)
		game_drop = 1;
	else
		game_drop = 0;
	return;
}

static void processInput(int frame)
{
	int p;

	memset(game_keys[frame], 0, 8);

	if (console_active || console_chatterActive)
	{
		game_keys[frame][0] = '.';
		game_keys[frame][1] = 0;
		return;
	}

	if (input_keys[SDLK_UP] || input_keys['w'])
		sprintf(&game_keys[frame][strlen(game_keys[frame])], "U");
	if (input_keys[SDLK_DOWN] || input_keys['s'])
		sprintf(&game_keys[frame][strlen(game_keys[frame])], "D");
	if (input_keys[SDLK_LEFT] || input_keys['a'])
		sprintf(&game_keys[frame][strlen(game_keys[frame])], "L");
	if (input_keys[SDLK_RIGHT] || input_keys['d'])
		sprintf(&game_keys[frame][strlen(game_keys[frame])], "R");
	
	for (p=1; p<6; p++)
		if (input_keys[p+48])
		{
			sprintf(&game_keys[frame][strlen(game_keys[frame])],
				"%d", p);
			break;
		}
	if (input_keys[SDLK_SPACE])
	{
		sprintf(&game_keys[frame][strlen(game_keys[frame])], "S");
		input_keys[SDLK_SPACE] = 0;
	}
	if (input_keys[SDLK_RETURN])
		sprintf(&game_keys[frame][strlen(game_keys[frame])], "E");

	if (strlen(game_keys[frame]) == 0)
	{
		game_keys[frame][0] = '.';
		game_keys[frame][1] = 0;
	}

	return;
}

static int checkBall2ballColisions(int p, int q, int r, int s)
{
	float d;

	d =  sqrtf(
	pow(client_players[p].ball[q].pos.x-client_players[r].ball[s].pos.x, 2)+
	pow(client_players[p].ball[q].pos.y-client_players[r].ball[s].pos.y, 2)
	);

	if (d > 20.0f)
		return 0;
	else
		return 1;
}

static int ball2ballColisions(int p, int q, int r, int s)
{
	float d, dvx2, a, x21, y21, vx21, vy21, fy21, sign, movx, movy;

	d =  sqrtf(
	pow(client_players[p].ball[q].pos.x-client_players[r].ball[s].pos.x, 2)+
	pow(client_players[p].ball[q].pos.y-client_players[r].ball[s].pos.y, 2)
	);

	if (d > 20.0f)
		return 0;

	x21  = client_players[r].ball[s].pos.x-client_players[p].ball[q].pos.x;
	y21  = client_players[r].ball[s].pos.y-client_players[p].ball[q].pos.y;
	vx21 = client_players[r].ball[s].vel.x-client_players[p].ball[q].vel.x;
	vy21 = client_players[r].ball[s].vel.y-client_players[p].ball[q].vel.y;

	fy21=1.0E-12*fabs(y21);
	if ( fabs(x21)<fy21 )
	{
		if (x21 < 0.0f)
			sign = -1;
		else
			sign = 1;
		x21 = fy21*sign;
	}

	a = y21/x21;
	dvx2= -2*(vx21 +a*vy21)/((1+a*a)*(2.0f));

	client_players[r].ball[s].vel.x += dvx2;
	client_players[r].ball[s].vel.y += a*dvx2;

	client_players[p].ball[q].vel.x -= dvx2;
	client_players[p].ball[q].vel.y -= a*dvx2;

	movx = (client_players[r].ball[s].pos.x
		 -client_players[p].ball[q].pos.x)
		* (20.0f/d - 1.0f);
	movy = (client_players[r].ball[s].pos.y
		 -client_players[p].ball[q].pos.y)
		* (20.0f/d - 1.0f);

	client_players[p].ball[q].pos.x -= movx;
	client_players[p].ball[q].pos.y -= movy;

	return 1;
}

static int colisions(float *x, float *y, float *vx, float *vy,
	float lastx, float lasty, float radius, float spring, char checkonly)
{
	int r, s;
	char colision = 0;

	/* ball2wall colisions */
	for (r=0; r<client_map->elementsCount; r++)
		if (client_map->elements[r].type == 0 ||
			client_map->elements[r].type == 6)
		{
			/* horizontal lines */
			if (client_map->elements[r].arg[1]
			 == client_map->elements[r].arg[3])
			{
				int lox, hix;
				if (client_map->elements[r].arg[0] >
				    client_map->elements[r].arg[2])
				{
					lox = client_map->elements[r].arg[2];
					hix = client_map->elements[r].arg[0];
				}
				else
				{
					lox = client_map->elements[r].arg[0];
					hix = client_map->elements[r].arg[2];
				}

				/* woo, what a nice conditionals...
				 */

				/* this checks if there was a colision
				 *
				 *          |
				 *          v
				 *       ------- 
				 */
				if  (
				     ((*x) >= lox)
					&&
				     ((*x) <= hix)
					&&
				     (  (*y) <
					client_map->elements[r].arg[1]+radius
				     )
					&&
				     (
					lasty >=
					client_map->elements[r].arg[1]+radius
				     )
				    )
				{
					if (!checkonly)
					{

						(*vy) = spring
							* -(*vy);
					
						(*vx) = spring
							* (*vx);

						(*y) = radius +
						 client_map->elements[r].arg[1];
					}

					colision = 1;
					colIndex = r;
				}

				/* this checks if there was a colision
				 *
				 *       -------
				 *          ^
				 *          |
				 */
				if  (
				     ((*x) >= lox)
					&&
				     ((*x) <= hix)
					&&
				     (  (*y) >
					client_map->elements[r].arg[1]-radius
				     )
					&&
				     (
					lasty <=
					client_map->elements[r].arg[1]-radius
				     )
				    )
				{
					if (!checkonly)
					{
						(*vy) = spring
							* -(*vy);
					
						(*vx) = spring
							* (*vx);

						(*y) =
						 client_map->elements[r].arg[1]
						 - radius;
					}

					colision = 1;
					colIndex = r;
				}
			}
			/* vertical lines */
			else if (client_map->elements[r].arg[0]
			      == client_map->elements[r].arg[2])
			{
				int loy, hiy;
				if (client_map->elements[r].arg[1] >
				    client_map->elements[r].arg[3])
				{
					loy = client_map->elements[r].arg[3];
					hiy = client_map->elements[r].arg[1];
				}
				else
				{
					loy = client_map->elements[r].arg[1];
					hiy = client_map->elements[r].arg[3];
				}

				/* woo, what a nice conditionals...
				 */

				/* this checks if there was a colision
				 *
				 *       |
				 *       |<-----
				 *       |
				 */
				if  (
				     ((*y) >= loy)
					&&
				     ((*y) <= hiy)
					&&
				     (  (*x) <
					client_map->elements[r].arg[0]+radius
				     )
					&&
				     (
					lastx >=
					client_map->elements[r].arg[0]+radius
				     )
				    )
				{
					if (!checkonly)
					{
						(*vy) = spring
							* (*vy);

						(*vx) = spring
							* -(*vx);

						(*x) =
						 client_map->elements[r].arg[0]
						 +radius;
					}

					colision = 1;
					colIndex = r;
				}

				/* this checks if there was a colision
				 *
				 *             |
				 *       ----->|
				 *             |
				 */
				if  (
				     ((*y) >= loy)
					&&
				     ((*y) <= hiy)
					&&
				     (  (*x) >
					client_map->elements[r].arg[0]-radius
				     )
					&&
				     (
					lastx <=
					client_map->elements[r].arg[0]-radius
				     )
				    )
				{
					if (!checkonly)
					{
						(*vy) = spring
							* (*vy);

						(*vx) = spring
							* -(*vx);

						(*x) =
						 client_map->elements[r].arg[0]
						 - radius;
					}

					colision = 1;
					colIndex = r;
				}
			}
			/* diagonal lines */
			else
			{
				int lox, hix;
				float a, b, hib, lob;
				float now, b4;
				if (client_map->elements[r].arg[0] >
				    client_map->elements[r].arg[2])
				{
					lox = client_map->elements[r].arg[2];
					hix = client_map->elements[r].arg[0];
				}
				else
				{
					lox = client_map->elements[r].arg[0];
					hix = client_map->elements[r].arg[2];
				}

				a = (float)(client_map->elements[r].arg[3]
				    -client_map->elements[r].arg[1]) /
				    (float)(client_map->elements[r].arg[2]
				    -client_map->elements[r].arg[0]);
				b = client_map->elements[r].arg[1] -
				    a*client_map->elements[r].arg[0];

				hib = b + (radius / cos(atan(a)));
				lob = b - (radius / cos(atan(a)));

				now = ((*y)
				      -a*(*x));
				b4 = lasty - a*lastx;

				/* this checks if there was a colision
				 *
				 *  \      /
				 *   \    /
				 *    \  /
				 *     V/
				 *     /
				 *    /
				 *   /
				 */
				if  (
				     (
					(*x) >=
					lox - radius*sin(atan(a))
				     )
					&&
				     (
					(*x) <=
					hix - radius*sin(atan(a))
				     )
					&&
				     (now < hib)
					&&
				     (b4 >= hib)
				    )
				{
					if (!checkonly)
					{
						float dotprod;

						(*y) = hib +
							a*(*x) + SLOPEBUGFIX;

						dotprod = (-radius*sin(atan(a))
							* (*vx)
							+
							  (radius*cos(atan(a)))
							* (*vy)
							) / radius;

						(*vx) -=
							2.0f *
							(-radius*sin(atan(a)))
							/ radius * dotprod;

						(*vy) -= 
							2.0f *
							(radius*cos(atan(a)))
							/ radius * dotprod;

						(*vx) *= spring;
						(*vy) *= spring;
					}
					

					colision = 1;
					colIndex = r;
				}
				/* this checks if there was a colision
				 *
				 *         /
				 *        /
				 *       /
				 *      /^
				 *     /  \
				 *    /    \
				 *   /      \
				 */
				if  (
				     (
					(*x) >=
					lox + radius*sin(atan(a))
				     )
					&&
				     (
					(*x) <=
					hix + radius*sin(atan(a))
				     )
					&&
				     (now > lob)
					&&
				     (b4 <= lob)
				    )
				{
					if (!checkonly)
					{
						float dotprod;

						(*y) = lob +
							a*(*x) - SLOPEBUGFIX;

						dotprod = ((radius*sin(atan(a)))
							* (*vx)
							+
							  (-radius*cos(atan(a)))
							* (*vy)
							) / radius;

						(*vx) -=
							2.0f *
							(radius*sin(atan(a)))
							/ radius * dotprod;

						(*vy) -= 
							2.0f *
							(-radius*cos(atan(a)))
							/ radius * dotprod;

						(*vx) *= SPRING;
						(*vy) *= SPRING;
					}
					

					colision = 1;
					colIndex = r;
				}

				
			}

			/* point colisions */
			for (s=0; s<=2; s+=2)
			{
				float dotprod;
				float d = sqrt(
				    pow(client_map->elements[r].arg[s]
				       -(*x), 2)
				    +
				    pow(client_map->elements[r].arg[s+1]
				       -(*y), 2)
				    );
				if (d < radius)
				{
				/* extended version of "colision correcion",
				 * which is more precise.
				 */ if (!checkonly) {
					float xVelDelta
						= ((*x)-lastx);
					float yVelDelta
						= ((*y)-lasty);
					int counter = 0;

					while (fabsf(radius-d)>0.01f
					|| d < radius)
					{
						xVelDelta /= 1.5f;
						yVelDelta /= 1.5f;

						if (d < radius)
						{
							(*x) -=
								xVelDelta;
							(*y) -=
								yVelDelta;

						}
						else if (d > radius)
						{
							(*x) +=
								xVelDelta;
							(*y) +=
								yVelDelta;
						}

						d = sqrt(
					    pow(client_map->elements[r].arg[s]
					       -(*x), 2)
					    +
					    pow(client_map->elements[r].arg[s+1]
					       -(*y), 2)
						);

						/* make sure that it won't be
						 * an infinite loop */
						if (++counter > 1000)
							break;
					}

					dotprod = (
						(*vx)
						* ((*x) -
						client_map->elements[r].arg[s])
						+
						(*vy)
						* ((*y) -
					       client_map->elements[r].arg[s+1])
					       ) / radius;

					(*vx) -=
						2.0f *
						((*x) -
						client_map->elements[r].arg[s])
						/radius * dotprod;

					(*vy) -= 
						2.0f *
						((*y) -
					       client_map->elements[r].arg[s+1])
						/radius * dotprod;

					(*vx) *= spring;
					(*vy) *= spring;

				}
					colision = 1;
					colIndex = r;
				}
			}
		}
	
	if (!colision)
		colIndex = -1;
	return colision;
}

static int physics(int p, int q)
{
	float lastx, lasty;
	int r, s;
	int result = 0;
	char cols;

	if (client_players[p].ball[q].pos.y < 0.0f)
	{
		client_players[p].ball[q].vel.y += WATER
			- WATER2 * (float)client_players[p].ball[q].water;

		misc_minMaxLimit(&client_players[p].ball[q].vel.y, -100.0f,
			100.0f);

		client_players[p].ball[q].vel.x /= WATERX;
	}
	else
		client_players[p].ball[q].vel.y -= GRAVITY;

	lastx = client_players[p].ball[q].pos.x;
	lasty = client_players[p].ball[q].pos.y;
	client_players[p].ball[q].pos.y += client_players[p].ball[q].vel.y
		* CONSTMOVE;
	client_players[p].ball[q].pos.x += client_players[p].ball[q].vel.x
		* CONSTMOVE;

	/* ball2ball colisions */
	for (r=0; r<6; r++) if (client_players[r].status &&
		client_players[r].ingame)
	for (s=0; s<4; s++) if (client_players[r].ball[s].status)
	if (p!=r || q!=s)
		if (ball2ballColisions(p, q, r, s))
			result = 2;
			
	cols = colisions(&client_players[p].ball[q].pos.x,
		&client_players[p].ball[q].pos.y,
		&client_players[p].ball[q].vel.x,
		&client_players[p].ball[q].vel.y,
		lastx, lasty, 10.0f, SPRING, 0);
	
	/* block double colision */
	if (cols && colisions(&client_players[p].ball[q].pos.x,
		&client_players[p].ball[q].pos.y,
		&client_players[p].ball[q].vel.x,
		&client_players[p].ball[q].vel.y,
		lastx, lasty, 10.0f, SPRING, 0))
	{
		client_players[p].ball[q].pos.x = lastx;
		client_players[p].ball[q].pos.y = lasty;
	}
	else if (
		cols &&
		((fabs(client_players[p].ball[q].vel.x) > 50.0f)
		||
		(fabs(client_players[p].ball[q].vel.y) > 50.0f)
		))
	{
		audio_play(client_sndColision2);
	}
		
	if (cols && !result)
		result = 1;

	if (client_players[p].ball[q].pos.y < 0.0f)
	{
		client_players[p].ball[q].health = 0;
		if (!client_players[p].ball[q].water)
			audio_play(client_sndWater);
		client_players[p].ball[q].water ++;
		if (p == client_turn && q == client_turnBall
			&& !game_preparetoend)
		{
			game_preparetoend = 1;
			game_rope = 0;
			game_hooked = 0;
			game_roperep = 1;
		}
	}
	
	if (client_players[p].ball[q].fall)
		client_players[p].ball[q].fall--;

	return result;

}

static int ropePhysics(int p, int q)
{
	float interia = MASS*game_ropelen*game_ropelen;
	float lastx, lasty, lastra;
	int r, s;

	lastx = client_players[p].ball[q].pos.x;
	lasty = client_players[p].ball[q].pos.y;
	lastra = game_ropeang;

	game_omega -= (GRAVITY*MASS*game_ropelen*
			(-cos(misc_deg2rad(game_ropeang)))/interia)
			*CONSTROPE;
	game_ropeang += game_omega;
	game_omega *= 1.0f-RESISTANCE;

	while (game_ropeang>=360.0f)
		game_ropeang -= 360.0f;
	while (game_ropeang<0.0f)
		game_ropeang += 360.0f;

	client_players[p].ball[q].pos.x = game_hookPos.x
				- game_ropelen*cos(misc_deg2rad(game_ropeang));
	client_players[p].ball[q].pos.y = game_hookPos.y
				- game_ropelen*sin(misc_deg2rad(game_ropeang));

	if (colisions(&client_players[p].ball[q].pos.x,
		&client_players[p].ball[q].pos.y,
		&client_players[p].ball[q].vel.x,
		&client_players[p].ball[q].vel.y,
		lastx, lasty, 10.0f, SPRING, 1))
	{
		/* wxw walls */
		if (client_gametype == 2
			&& client_map->elements[colIndex].type == 6
			&& !game_walls[colIndex])
		{
			game_walls[colIndex] = 1;
			game_walls2go--;
			if (game_walls2go > 1)
				console_messagePrint("^1%d^0 more walls to go!",
				game_walls2go);
			else if (game_walls2go == 1)
				console_messagePrint("^1%d^0 more wall to go!",
				game_walls2go);
			else
				console_messagePrint("^1That's all!");
		}
		game_omega = -game_omega*ROPESPRING;
		client_players[p].ball[q].pos.x = lastx;
		client_players[p].ball[q].pos.y = lasty;
		game_ropeang = lastra;
		audio_play(client_sndColision1);
	}

	/* ball2ball colisions */
	for (r=0; r<6; r++) if (client_players[r].status &&
		client_players[r].ingame)
	for (s=0; s<4; s++) if (client_players[r].ball[s].status)
	if (p!=r || q!=s)
		if (checkBall2ballColisions(p, q, r, s))
		{
			game_omega = -game_omega*ROPESPRING;
			client_players[p].ball[q].pos.x = lastx;
			client_players[p].ball[q].pos.y = lasty;
			game_ropeang = lastra;
			audio_play(client_sndColision1);
		}


	game_ropelen = sqrt(pow(client_players[client_turn].
		ball[client_turnBall].pos.x - game_hookPos.x, 2) +
		pow(client_players[client_turn].
		ball[client_turnBall].pos.y - game_hookPos.y, 2));
	
	if (client_players[p].ball[q].pos.y < 0.0f)
	{
		client_players[p].ball[q].health = 0;
		if (p == client_turn && q == client_turnBall
			&& !game_preparetoend)
		{
			game_preparetoend = 1;
			game_rope = 0;
			game_hooked = 0;
			game_roperep = 1;
		}
	}
	
	return 0;
	
}

static void boom(float radius, int w)
{
	int p, q;

	audio_play(client_sndBoom);

	if (!game_weap[w].status)
		return;
	
/* bad coding style, I know I know... :< */	
for (p=0; p<6; p++) if (client_players[p].status && client_players[p].ingame)
for (q=0; q<4; q++) if (client_players[p].ball[q].status)
{
	float d = sqrt(pow(game_weap[w].pos.x-client_players[p].ball[q].pos.x,2)
		+ pow(game_weap[w].pos.y-client_players[p].ball[q].pos.y, 2));


	if (d < radius)
	{
		float boomVel = cos(d/radius*PI/2.0f)*BOOMCONST*radius;

		if (p == client_turn && q == client_turnBall
			&& !game_preparetoend)
		{
			game_preparetoend = 1;
			game_rope = 0;
			game_hooked = 0;
			game_roperep = 1;
		}
		
		client_players[p].ball[q].vel.x +=
			(client_players[p].ball[q].pos.x-game_weap[w].pos.x)
			* boomVel / d;
		/*
		client_players[p].ball[q].vel.y +=
			(client_players[p].ball[q].pos.y-game_weap[w].pos.y)
			* boomVel / d;
		*/

		client_players[p].ball[q].vel.y += boomVel * BOOMYVEL;

		client_players[p].ball[q].health -= (int)(boomVel * DAMAGE);
		if (client_players[p].ball[q].health < 0)
			client_players[p].ball[q].health = 0;

		client_players[p].ball[q].fall = 5;

	}
}
	game_weap[w].status = 0;
	game_weap[w].fuse = 0;
	game_weap[w].boom = 1;

	return;
}

static void weaponPhysics(int w)
{
	float lastx, lasty;

	if (game_dropType != 5) /* no gravity for killer bee */
		game_weap[w].vel.y -= GRAVITY;

	/* wind */
	if (game_dropType == 0)
		game_weap[w].vel.x += (float)game_wind * WINDZOOKA;
	if (game_dropType == 3 && w>0)
		game_weap[w].vel.x += (float)game_wind * WINDCLUSTER;

	lastx = game_weap[w].pos.x;
	lasty = game_weap[w].pos.y;
	game_weap[w].pos.x += game_weap[w].vel.x * CONSTMOVE;
	game_weap[w].pos.y += game_weap[w].vel.y * CONSTMOVE;

	if (game_dropType == 4) /* special movement for hamsters */
	{
		if (game_weap[w].sndtime)       /* some shit to be sure that */
			game_weap[w].sndtime++; /* hamsters won't make silly */
		if (game_weap[w].sndtime > 18)  /* sound every frame         */
			game_weap[w].sndtime = 0;

		/* colisions... */
		if (colisions(&game_weap[w].pos.x, &game_weap[w].pos.y,
			&game_weap[w].vel.x, &game_weap[w].vel.y,
			lastx, lasty, 10.0f, HAMSTERSPRING, 0))
		{
			if (game_weap[w].side)
				game_weap[w].vel.x = HAMSTERSPEED;
			else
				game_weap[w].vel.x = -HAMSTERSPEED;
			game_weap[w].vel.y += HAMSTERJUMP;
			if (!game_weap[w].sndtime)
			{
				switch (rand()%4)
				{
					case 0:
						audio_play(client_sndHamster1);
					break;
					case 1:
						audio_play(client_sndHamster2);
					break;
					case 2:
						audio_play(client_sndHamster3);
					break;
					case 3:
						audio_play(client_sndHamster4);
					break;
				}
				game_weap[w].sndtime++;
			}
		}
	}
	else if (colisions(&game_weap[w].pos.x, &game_weap[w].pos.y,
		&game_weap[w].vel.x, &game_weap[w].vel.y,
		lastx, lasty, 5.0f, SPRING, 0))
	{
		/* colision-booms */
		if (game_dropType == 0) /* zooka boom */
			boom(ZOOKABOOM, w);
		else if (game_dropType == 3 && w > 0) /* clusters boom */
			boom(CLUSTERBOOM2, w);
		else if (game_dropType == 5) /* killer bee boom */
			boom(KILLERBEEBOOM, w);
		/* colision sound */
		else if ((fabs(game_weap[w].vel.x) > 50.0f)
			|| (fabs(game_weap[w].vel.y) > 50.0f))
			audio_play(client_sndColision3);
		
	}

	/* weapon-ball colisions */
	if (game_dropType == 0 || game_dropType == 4 || game_dropType == 5)
	{
		int p, q;
		float dlim = 15.0f;

		if (game_dropType == 4)
			dlim = 20.0f;

		for (p=0; p<6; p++) if (client_players[p].status
			&& client_players[p].ingame)
		for (q=0; q<4; q++) if (client_players[p].ball[q].status)
		{
			float d = sqrt(
				pow(client_players[p].ball[q].pos.x
				- game_weap[w].pos.x, 2)
				+
				pow(client_players[p].ball[q].pos.y
				- game_weap[w].pos.y, 2)
				);
			/* make exception for now playing ball when weapon has
			 * just been released */
			if (p == client_turn && q == client_turnBall)
			{
				/* zooka */
				if (game_dropType == 0 && game_escape < 25)
					continue;
				/* killer bee */
				else if (game_dropType == 5 && game_escape < 25)
					continue;
				/* hamsters */
				else
				{
					if (w == 0 && game_escape < 25)
						continue;
					else if (w == 1 && game_escape < 50)
						continue;
					else if (w == 2 && game_escape < 75)
						continue;
				}
			}
			if (d < dlim)
			{
				if (game_dropType == 0)
					boom(ZOOKABOOM, w);
				else if (game_dropType == 4)
					boom(HAMSTERBOOM, w);
				else if (game_dropType == 5)
					boom(KILLERBEEBOOM, w);

			}
		}			
	}

	if (game_weap[w].fuse)
	{
		game_weap[w].fuse--;
		if (game_weap[w].fuse == 0) /* .... *BOOM!!*    */
		{
			int q;

			if (game_dropType == 1) /* grenade */
				boom(GRENADEBOOM, w);
			else if (game_dropType == 2) /* dynamite */
				boom(DYNAMITEBOOM, w);
			else if (game_dropType == 3) /* cluster bomb */
				boom(CLUSTERBOOM, w);
			else if (game_dropType == 4) /* hamster */
				boom(HAMSTERBOOM, w);
			else if (game_dropType == 5) /* killer bee */
				boom(KILLERBEEBOOM, w);

			if (w == 0 && game_dropType == 3)
			{
				for (q=1; q<5; q++)
				{
					game_weap[q].status = 1;
					game_weap[q].pos.x = game_weap[w].pos.x;
					game_weap[q].pos.y = game_weap[w].pos.y;
					game_weap[q].vel.x =
						50.0f*q - 125.0f;
					game_weap[q].vel.y = 350.0f;
					if (q==2 || q ==3)
						game_weap[q].vel.y += 35.0f;
				}
			}
		}
	}
	if (game_weap[w].pos.y < 0.0f)
	{
		audio_play(client_sndWater);
		game_weap[w].status = 0;
		game_weap[w].fuse = 0;
	}

	game_weap[w].rotation += -ROTATION * game_weap[w].vel.x;
	return;
}

static char fallingDamage(int p, int q)
{
	float vel = sqrt(
	pow(client_players[p].ball[q].vel.x, 2) +
	pow(client_players[p].ball[q].vel.y, 2));

	if (!client_fall)
		return 0;
	if (vel > MINFALL && (!client_players[p].ball[q].fall) )
			/*   ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
			 *    this fixes a falling damage bug
			 */
	{
		client_players[p].ball[q].health -=
			(int)((vel - MINFALL) * FALL);
		if (client_players[p].ball[q].health < 0.0f)
			client_players[p].ball[q].health = 0.0f;
		return 1;
	}

	return 0;
}

/* the main physics funcion */
static void doFrame(int frame)
{
	int p, q;
	char colision = 0;
	float delta_len;

	if (client_cameraFocus == 2 && game_keys[frame][0] != '.')
		client_cameraFocus = 1;

	/* left-right turning */
	if (!game_roperep)
	{
		if (strchr(game_keys[frame], 'L'))
		{
			game_side = 0;
			client_players[client_turn].ball[client_turnBall].eyes
				= 0;
		}
		else if (strchr(game_keys[frame], 'R'))
		{
			game_side = 1;
			client_players[client_turn].ball[client_turnBall].eyes
				= 1;
		}
	}

	/* aiming */
	if (!game_rope && !game_roperep)
	{
		if (strchr(game_keys[frame], 'U'))
			game_aim += AIMSPEED;
		if (strchr(game_keys[frame], 'D'))
			game_aim -= AIMSPEED;
		misc_minMaxLimit(&game_aim, -90.0f, 90.0f);
	}

	/* fuse time changing */
	if (strchr(game_keys[frame], '1') && fuseTime!=1)
	{
		fuseTime = 1;
		console_messagePrint("Fuse time changed to 1");
	}
	if (strchr(game_keys[frame], '2') && fuseTime!=2)
	{
		fuseTime = 2;
		console_messagePrint("Fuse time changed to 2");
	}
	if (strchr(game_keys[frame], '3') && fuseTime!=3)
	{
		fuseTime = 3;
		console_messagePrint("Fuse time changed to 3");
	}
	if (strchr(game_keys[frame], '4') && fuseTime!=4)
	{
		fuseTime = 4;
		console_messagePrint("Fuse time changed to 4");
	}
	if (strchr(game_keys[frame], '5') && fuseTime!=5)
	{
		fuseTime = 5;
		console_messagePrint("Fuse time changed to 5");
	}


	/* movement for now-playing-ball */
	if (!game_hooked)
	{
		colision = physics(client_turn, client_turnBall);
		if (colision == 1)
		{
			if (
				fallingDamage(client_turn, client_turnBall)
				|| (game_roperep && !game_preparetoend)
				)
			{
				game_rope = 0;
				game_hooked = 0;
				game_roperep = 1;

				game_preparetoend = 1;
			}
		}
		else if (colision == 2 && !game_preparetoend)
		{
			game_roperep = 0;
		}
		
	}
	else
	{
		/* rope controls */
		float interia = MASS*game_ropelen*game_ropelen;
		float swing = SWING;

		if (strchr(game_keys[frame], 'U'))
		{
			delta_len = game_ropelen;
			game_ropelen -= ROPEMOVESPEED;
			if (game_ropelen < ROPEMINLEN)
				game_ropelen = ROPEMINLEN;
			delta_len = game_ropelen - delta_len;

			if (delta_len != 0.0f)
				game_omega *= (game_ropelen-delta_len)
						/ game_ropelen;
		}
		if (strchr(game_keys[frame], 'D'))
		{
			delta_len = game_ropelen;
			game_ropelen += ROPEMOVESPEED;
			if (game_ropelen > ROPEMAXLEN)
				game_ropelen = ROPEMAXLEN;
			delta_len = game_ropelen - delta_len;

			if (delta_len != 0.0f)
				game_omega *= (game_ropelen-delta_len)
						/ game_ropelen;
		}

		if (strchr(game_keys[frame], 'U'))
			swing *= SWINGBOOST;
		else if (strchr(game_keys[frame], 'D'))
			swing /= SWINGBOOST;

		if (strchr(game_keys[frame], 'L'))
			game_omega += swing*(-sin(misc_deg2rad(game_ropeang)))
				* game_ropelen/interia;

		if (strchr(game_keys[frame], 'R'))
			game_omega += swing*(sin(misc_deg2rad(game_ropeang)))
				* game_ropelen/interia;

		ropePhysics(client_turn, client_turnBall);

		/* update the velocity (for rope relasing and turn ending) */
		client_players[client_turn].ball[client_turnBall].vel.x =
			sin(misc_deg2rad(game_ropeang))*
			game_omega*game_ropelen;

		client_players[client_turn].ball[client_turnBall].vel.y =
			-cos(misc_deg2rad(game_ropeang))*
			game_omega*game_ropelen;

	}

	/* jumping */
	if (colision
		&& client_players[client_turn].ball[client_turnBall].vel.y
		< GRAVITY
		&& client_players[client_turn].ball[client_turnBall].vel.y
		>= 0.0f
		&& fabsf(
		client_players[client_turn].ball[client_turnBall].vel.x) <
		0.001f)
		if (strchr(game_keys[frame], 'E'))
		{

			client_players[client_turn].ball[client_turnBall].vel.y
				= JUMP;
			if (strchr(game_keys[frame], 'L'))
				client_players[client_turn].
					ball[client_turnBall].vel.x = -XJUMP;
			else if (strchr(game_keys[frame], 'R'))
				client_players[client_turn].
					ball[client_turnBall].vel.x = XJUMP;
			audio_play(client_sndJump);
		}
	
	/* rope fireing */
	if (strchr(game_keys[frame], 'S') && !game_rope)
	{
		game_rope = 1;
		game_ropelen = 0.0f;
		if (game_side)
			game_ropeang = game_aim;
		else
			game_ropeang = 180.0f-game_aim;
		
		while (game_ropeang>=360.0f)
			game_ropeang -= 360.0f;
		while (game_ropeang<0.0f)
			game_ropeang += 360.0f;

		audio_play(client_sndRope1);
	}
	/* rope relasing */
	if (strchr(game_keys[frame], 'S') && game_rope && game_hooked)
	{
		if (game_ropeang>=30.0f && game_ropeang<90.0f)
		{
			game_side = 0;
			game_aim = game_ropeang;
		}
		else if (game_ropeang>=90.0f && game_ropeang<150.0f)
		{
			game_side = 1;
			game_aim = 180.0f - game_ropeang;
		}
		else if (game_ropeang>=150.0f && game_ropeang<270.0f)
		{
			game_side = 1;
			game_aim = 30.0f;
		}
		else
		{
			game_side = 0;
			game_aim = 30.0f;
		}

		game_rope = 0;
		game_hooked = 0;
		game_roperep = 1;
	}
	
escape:
	/* rope elongating */
	if (game_rope && !game_hooked && !game_preparetoend)
	{
		int r;
		float xRope, yRope;
		char done = 0;
		float xHook = 0.0f;
		float yHook = 0.0f;
		float xHookTmp, yHookTmp;

		game_ropelen += ROPESPEED;
		if (game_ropelen > ROPEMAXLEN)
		{
			game_rope = 0;
			game_ropelen = 0.0f;
			goto escape;
		}

		/* misc_deg2rad is not as precise as we'd like to.
		 * That's why we need to do some workaround*/
		if ((game_ropeang == 90.0f) || (game_ropeang == -90.0f))
			xRope = client_players[client_turn].
				ball[client_turnBall].pos.x;
		
		else
			xRope = client_players[client_turn].
				ball[client_turnBall].pos.x
				+ cos(misc_deg2rad(game_ropeang))*game_ropelen;

		yRope = client_players[client_turn].ball[client_turnBall].pos.y
			+ sin(misc_deg2rad(game_ropeang))*game_ropelen;

		/* rope colisions */
		for (r=0; r<client_map->elementsCount; r++)
			if (client_map->elements[r].type == 0
				|| client_map->elements[r].type == 6)
			{
				char res;

				res = misc_segmentsIntersect(
					client_players[client_turn].
					 ball[client_turnBall].pos.x,
					client_players[client_turn].
					 ball[client_turnBall].pos.y,
					xRope,
					yRope,
					(float)client_map->elements[r].arg[0],
					(float)client_map->elements[r].arg[1],
					(float)client_map->elements[r].arg[2],
					(float)client_map->elements[r].arg[3],
					&xHookTmp, &yHookTmp);
				if (res)
				{
					if (done)
					{
						if (pow(client_players
							[client_turn].
							ball[client_turnBall].
							pos.x-xHookTmp, 2)
							+ pow(
							client_players
							[client_turn].
							ball[client_turnBall].
							pos.y-yHookTmp, 2)
							
							<

							pow(client_players
							[client_turn].
							ball[client_turnBall].
							pos.x-xHook, 2)
							+ pow(
							client_players
							[client_turn].
							ball[client_turnBall].
							pos.y-yHook, 2)
							)
						{
							xHook = xHookTmp;
							yHook = yHookTmp;
						}
					}
					else
					{
						xHook = xHookTmp;
						yHook = yHookTmp;
					}
					done = 1;
				}
			}

		if (done)
		{
			audio_play(client_sndRope2);

			game_hookPos.x = xHook;
			game_hookPos.y = yHook;

			/* "little jump" */
			if (colision
		&& client_players[client_turn].ball[client_turnBall].vel.y
		< GRAVITY
		&& client_players[client_turn].ball[client_turnBall].vel.y
		>= 0.0f
		&& fabsf(
		client_players[client_turn].ball[client_turnBall].vel.x) <
		0.001f)
			{
				client_players[client_turn]
				.ball[client_turnBall].vel.y =
					LITTLEJUMP;
			}

			game_ropelen = sqrt(pow(
			client_players[client_turn].
			ball[client_turnBall].pos.x-xHook, 2)+
			pow(client_players[client_turn].
			ball[client_turnBall].pos.y-yHook, 2));

			game_omega =
			client_players[client_turn].
			ball[client_turnBall].vel.y
			* -cos(misc_deg2rad(game_ropeang))
			/ game_ropelen;

			game_omega +=
			client_players[client_turn].
			ball[client_turnBall].vel.x
			* sin(misc_deg2rad(game_ropeang))
			/ game_ropelen;

			game_hooked = 1;
		}
	}

	/* shooting the weapon */
	if (game_hooked && strchr(game_keys[frame], 'E') && !game_drop
		&& !game_weap[0].status && !game_escape && client_gametype != 4
		&& (client_gametype != 2 || !game_walls2go)
		&& (client_gametype != 1 || flight))
	{
		audio_play(client_sndThrow);
		game_weap[0].status = 1;
		game_weap[0].pos.x =
			client_players[client_turn].ball[client_turnBall].pos.x;
		game_weap[0].pos.y =
			client_players[client_turn].ball[client_turnBall].pos.y;

		if (game_dropType == 4 || game_dropType == 5)
		{
			if (client_players[client_turn].
				ball[client_turnBall].vel.x > 0)
				game_weap[0].side = 1;
			else
				game_weap[0].side = 0;
		}

		if (game_dropType == 2 || game_dropType == 4)
		{
			game_weap[0].vel.x = 0.0f;
			game_weap[0].vel.y = 0.0f;
		}
		else
		{
			game_weap[0].vel.x =
				sin(misc_deg2rad(game_ropeang))*
				game_omega*game_ropelen;

			game_weap[0].vel.y =
				-cos(misc_deg2rad(game_ropeang))*
				game_omega*game_ropelen;
		}

		if (game_dropType == 1 || game_dropType == 3
			|| game_dropType == 5)
			game_weap[0].fuse = 50*fuseTime;
		else if (game_dropType == 2 || game_dropType == 4)
			game_weap[0].fuse = 250;

		game_escape = 1;
	}

	/* shooting 2nd and 3rd hamsters */
	if (!game_drop && game_dropType == 4 &&
		(game_escape == 25 || game_escape == 50))
	{
		audio_play(client_sndThrow);
		game_weap[game_escape/25].status = 1;
		game_weap[game_escape/25].pos.x =
			client_players[client_turn].ball[client_turnBall].pos.x;
		game_weap[game_escape/25].pos.y =
			client_players[client_turn].ball[client_turnBall].pos.y;

		if (client_players[client_turn].
			ball[client_turnBall].vel.x > 0)
			game_weap[game_escape/25].side = 1;
		else
			game_weap[game_escape/25].side = 0;

		game_weap[game_escape/25].vel.x = 0.0f;
		game_weap[game_escape/25].vel.y = 0.0f;

		game_weap[game_escape/25].fuse = 250;
	}

	for (p=0; p<MAXCLUSTERS; p++)
		if (game_weap[p].status)
			weaponPhysics(p);

	/* taking the drop */
	if (game_drop)
	{
		float d = sqrt(
		pow(client_players[client_turn].ball[client_turnBall].pos.x -
		(float)game_dropPos.x, 2) +
		pow(client_players[client_turn].ball[client_turnBall].pos.y -
		(float)game_dropPos.y, 2));

		if (d < 20.0f)
		{
			game_drop = 0;
			if (client_gametype == 3)
				client_players[client_turn].
					ball[client_turnBall].health += 20;
			console_messagePrint("Picked up weapon: ^1%s^0.",
				weapons[game_dropType]);
		}
	}

	for (p=0; p<MAXCLUSTERS; p++)
	{
		if (game_weap[p].boom)
			game_weap[p].boom ++;
		if (game_weap[p].boom >= 50)
			game_weap[p].boom = 0;
	}

	/* fly shoppa checkings */
	if (client_gametype == 1 &&
		client_players[client_turn].ball[client_turnBall].pos.x > 0.0f
		&& !flightSide)
	{
		flightSide = 1;
		if (!game_hooked && !flight)
		{
			flight = 1;
			console_messagePrint("^1Flight accepted!");
		}
		
	}
	else if (client_gametype == 1 &&
		client_players[client_turn].ball[client_turnBall].pos.x <= 0.0f
		&& flightSide)
	{
		flightSide = 0;
		if (!game_hooked && !flight)
		{
			flight = 1;
			console_messagePrint("^1Flight accepted!");
		}
	}
	
	/* other balls movement */
	for (p=0; p<6; p++)
		for (q=0; q<4; q++)
			if (client_players[p].status
				&& client_players[p].ingame
				&& client_players[p].ball[q].status
				&& !(p == client_turn && q == client_turnBall))
				if (physics(p, q))
					fallingDamage(p, q);

	/* eyes */
	for (p=0; p<6; p++)
		for (q=0; q<4; q++)
		if (client_players[p].ball[q].vel.x > 0.0f)
			client_players[p].ball[q].eyes = 1;
		else if (client_players[p].ball[q].vel.x < 0.0f)
			client_players[p].ball[q].eyes = 0;

	return;
}

/* every second frame */
static void unpackEss(int sec)
{
	int p, q;
	for (p=0; p<6; p++) for (q=0; q<4; q++)
	{
		client_players[p].ball[q].health =
			client_ess[sec].ball[p][q].health;
		client_players[p].ball[q].pos.x =
			client_ess[sec].ball[p][q].pos.x;
		client_players[p].ball[q].pos.y =
			client_ess[sec].ball[p][q].pos.y;
		client_players[p].ball[q].vel.x =
			client_ess[sec].ball[p][q].vel.x;
		client_players[p].ball[q].vel.y =
			client_ess[sec].ball[p][q].vel.y;
	}
	for (p=0; p<MAXCLUSTERS; p++)
	{
		game_weap[p].status = client_ess[sec].weap[p].status;
		game_weap[p].boom = client_ess[sec].weap[p].boom;
		game_weap[p].fuse = client_ess[sec].weap[p].fuse;
		game_weap[p].side = client_ess[sec].weap[p].side;
		game_weap[p].pos.x = client_ess[sec].weap[p].pos.x;
		game_weap[p].pos.y = client_ess[sec].weap[p].pos.y;
		game_weap[p].vel.x = client_ess[sec].weap[p].vel.x;
		game_weap[p].vel.y = client_ess[sec].weap[p].vel.y;
	}

	game_side = client_ess[sec].side;
	game_aim = client_ess[sec].aim;
	game_rope = client_ess[sec].rope;
	game_ropeang = client_ess[sec].ropeang;
	game_ropelen = client_ess[sec].ropelen;
	game_hooked = client_ess[sec].hooked;
	game_hookPos.x = client_ess[sec].hookPos.x;
	game_hookPos.y = client_ess[sec].hookPos.y;
	game_roperep = client_ess[sec].roperep;
	game_omega = client_ess[sec].omega;
	game_drop = client_ess[sec].drop;
	game_walls2go = client_ess[sec].walls2go;
	return;
}

/* do or play the frame */
int game_do()
{
	if (!client_gameInProgress)
		return 0;

	if (game_prepare && client_doTheJob)
	{
		if (game_prepare == 1)
			prepareTurn();
		game_prepare++;
		if (game_prepare == 120)
			client_cameraFocus = 1;
		if (game_prepare == 250)
			game_prepare = 0;
		return 0;
	}
	if (client_doTheJob)
	{
		if (game_frame == 0)
		{
			prepareTurn();
			end = 0;
			game_preparetoend = 0;
		}
		if (!game_preparetoend)
			processInput(game_frame);
		else
		{
			game_keys[game_frame][0] = '.';
			game_keys[game_frame][1] = 0;
			game_preparetoend++;

			if (game_escape)
			{
				if (game_escape >= 500
					&& game_preparetoend >= 250)
					end = 1;
			}
			else
			{
				if (game_preparetoend >= 250)
					end = 1;
			}
		}
			

		doFrame(game_frame);
		game_frame++;
		if (!(game_frame%50))
			client_second((game_frame/50)-1);
		if (game_escape)
			game_escape++;
		if (game_escape == 500)
		{
			/*game_escape = 0;*/
			game_rope = 0;
			game_hooked = 0;
			game_roperep = 1;
			if (!game_preparetoend)
				game_preparetoend = 1;
		}
		if (game_frame >= (50*client_timelimit) && !game_preparetoend
			&& !game_escape)
		{
			game_rope = 0;
			game_hooked = 0;
			game_roperep = 1;
			game_preparetoend = 1;
		}
		if (game_frame >= 4000 || (!(game_frame%50) && end))
		{
			client_doTheJob = 0;
			client_sync();
			end = 0;
			game_preparetoend = 0;
		}
	}
	else if (client_playTheJob)
	{
		if (game_frame == 0)
		{
			prepareTurn();
			game_preparetoend = 0;
		}
		if (client_secondsRecieved*50 > game_frame)
		{
			client_lag = 0;
			doFrame(game_frame);
			game_frame++;
			if (game_frame && !(game_frame%50))
				unpackEss(game_frame/50 - 1);
			if (game_escape)
				game_escape++;
			if (game_escape == 500)
			{
				/*game_escape = 0;*/
				game_rope = 0;
				game_hooked = 0;
				game_roperep = 1;
				if (!game_preparetoend)
					game_preparetoend = 1;
			}
			if (game_frame >= (50*client_timelimit) &&
				!game_preparetoend
				&& !game_escape)
			{
				game_rope = 0;
				game_hooked = 0;
				game_roperep = 1;
				game_preparetoend = 1;
			}
		}
		else
			client_lag = 1;
	}

	return 0;
}
