/*
 *  (C) Copyright 2007-2009 by FreeShoppa Development Team
 *                        (http://chomzee.ethernet.pl/freeshoppa/)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include "freeshoppa.h"
#include <openssl/md5.h>

#define MAPBUFLEN 65536 /* 64K skould be enough */

Map *map_load(char *mapname, char quiet)
{
	FILE *map;
	char filenameBuf[256];

	Map *newmap;

	if (!quiet)
		console_print("  Loading map: %s...", mapname);

	newmap = (Map *)malloc(sizeof(Map));
	if (newmap == NULL)
	{
		if (!quiet)
		{
			console_print("  map: failed to alloc memory.");
			console_print("  ^1FAILED!!!");
		}
		return NULL;
	}
	newmap->ready = 0;

#ifdef WIN32
	snprintf(filenameBuf, 256, "maps/%s.fsm", mapname);
	map = fopen(filenameBuf, "r");
#else
	snprintf(filenameBuf, 256, "%s/.freeshoppa/maps/%s.fsm",
		getenv("HOME"), mapname);
	map = fopen(filenameBuf, "r");
	if (!map)
	{
		snprintf(filenameBuf, 256, "maps/%s.fsm", mapname);
		map = fopen(filenameBuf, "r");
		if (!map)
		{
			snprintf(filenameBuf, 256, "%s/maps/%s.fsm",
				SHOPPADIR, mapname);
			map = fopen(filenameBuf, "r");
		}
	}
#endif

	if (map)
	{
		char buffer[MAPBUFLEN];
		unsigned char md5buf[16];
		unsigned char md5buf_c[33];
		int len;
		int p = 0;
		int q;
		char *first, *second;
		char byte;

		for (p=0; (byte=fgetc(map)) != EOF; p++)
		{
			if (p == MAPBUFLEN)
			{
				if (!quiet)
				{
					console_print(
					"  map: failed to load %s.fsm. "
					"File too big.", mapname);
					console_print("  ^1FAILED!!!");
				}
				free(newmap);
				return NULL;

			}
			buffer[p] = byte;

		}
		buffer[p] = 0;
		len = p;

		/* calculate md5 */
		MD5((unsigned char *)buffer, len, md5buf);
		for (p=0; p<16; p++)
			sprintf((char *)&md5buf_c[2*p], "%02x", md5buf[p]);


		/* copy md5 to newmap */
		memcpy(newmap->md5, (char *)md5buf_c, 33);

		/* copy map name and map description */
		first = strchr(buffer, '\n');
		if (!first || (int)(first-buffer)>80)
		{
			if (!quiet)
			{
				console_print("  map: bad format.");
				console_print("  ^1FAILED!!!");
			}
			free(newmap);
			return NULL;
		}
		second = strchr((char*)(first+1), '\n');
		if (!second || (second-first-1)>80)
		{
			if (!quiet)
			{
				console_print("  map: bad format.");
				console_print("  ^1FAILED!!!");
			}
			free(newmap);
			return NULL;
		}

		strncpy(newmap->name, buffer, (first-buffer));
		newmap->name[first-buffer] = 0;

		strncpy(newmap->desc, &buffer[first-buffer+1],
			(second-first-1));
		newmap->desc[second-first-1] = 0;

		/* copy count of elements */
		fseek(map, second-buffer+1, SEEK_SET);
		fscanf(map, "%d", &newmap->elementsCount);
		newmap->spawnsCount = 0;
		newmap->wallsCount = 0;

		if (newmap->elementsCount<1
			|| newmap->elementsCount > MAXELEMENTSCOUNT)
		{
			if (!quiet)
			{
				console_print("  map: bad format.");
				console_print("  ^1FAILED!!!");
			}
			free(newmap);
			return NULL;
		}

		/* allocate memory for elements */
		newmap->elements = (Element *)malloc(
				sizeof(Element)*(newmap->elementsCount));
		if (newmap->elements == NULL)
		{
			if (!quiet)
			{
				console_print("  map: failed to alloc memory.");
				console_print("  ^1FAILED!!!");
			}
			free(newmap);
			return NULL;
		}
		
		for (p=0; p<newmap->elementsCount; p++)
		{
			fscanf(map, "%d%d%d%d%d",
				(int*)&newmap->elements[p].type,
				&newmap->elements[p].arg[0],
				&newmap->elements[p].arg[1],
				&newmap->elements[p].arg[2],
				&newmap->elements[p].arg[3]);

			if (newmap->elements[p].type == 1)
				newmap->spawnsCount ++;
			if (newmap->elements[p].type == 2)
			{
				if (newmap->elements[p].arg[3])
				{
					newmap->gametypes[0] = 0;
					newmap->gametypes[1] = 0;
					newmap->gametypes[2] = 0;
					newmap->gametypes[3] = 0;
					newmap->gametypes[4] = 1;
				}
				else
				{
					newmap->gametypes[0] = 1;
					newmap->gametypes[1] =
						newmap->elements[p].arg[0];
					newmap->gametypes[2] =
						newmap->elements[p].arg[1];
					newmap->gametypes[3] =
						newmap->elements[p].arg[2];
					newmap->gametypes[4] = 0;

				}
			}
			if (newmap->elements[p].type == 3)
			{
				newmap->xDropMin = newmap->elements[p].arg[0];
				newmap->yDropMin = newmap->elements[p].arg[1];
				newmap->xDropMax = newmap->elements[p].arg[2];
				newmap->yDropMax = newmap->elements[p].arg[3];
			}
			if (newmap->elements[p].type == 4)
			{
				newmap->xMin = newmap->elements[p].arg[0];
				newmap->xMax = newmap->elements[p].arg[2];
				newmap->yMax = newmap->elements[p].arg[3];
			}
			if (newmap->elements[p].type == 6)
				newmap->wallsCount++;
				
		}
		
		fclose(map);

		newmap->spawns = (Element *)malloc(
				sizeof(Element)*(newmap->spawnsCount));
		if (newmap->spawns == NULL)
		{
			if (!quiet)
			{
				console_print("  map: failed to alloc memory.");
				console_print("  ^1FAILED!!!");
			}
			free(newmap->elements);
			free(newmap);
			return NULL;
		}
		q=0;
		for (p=0; p<newmap->elementsCount; p++)
		{
			if (newmap->elements[p].type == 1)
			{
				newmap->spawns[q].type = 1;
				newmap->spawns[q].arg[0] =
					newmap->elements[p].arg[0];
				newmap->spawns[q].arg[1] =
					newmap->elements[p].arg[1];
				newmap->spawns[q].arg[2] = 0;
				newmap->spawns[q].arg[3] = 0;
				q++;
			}
		}

		newmap->ready = 1;
		if (!quiet)
			console_print("  ^2DONE");
		return newmap;
	}
	else
	{
		if (!quiet)
		{
			console_print("  map: failed to load %s.fsm.",
				mapname);
			console_print("  ^1FAILED!!!");
		}
		free(newmap);
		return NULL;
	}

}

void map_unload(Map *map)
{
	if (map)
	{
		if (map->elements)
			free(map->elements);
		if (map->spawns)
			free(map->spawns);
		free(map);
	}
	return;
}
