/*
 *  (C) Copyright 2007-2009 by FreeShoppa Development Team
 *                        (http://chomzee.ethernet.pl/freeshoppa/)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include "freeshoppa.h"

/* This funcion ensures us, that physics and rendering will be done in constant
 * peroid of time on every computers. */
void misc_waitFrame(int ms)
{
	static int next_tick = 0;
	int this_tick;

	this_tick = SDL_GetTicks();
	while (this_tick < next_tick)
		this_tick = SDL_GetTicks();
	next_tick = this_tick + (ms);

	return;
}       

float misc_deg2rad(float deg)
{
	return deg * PI / 180.0f;
}

float misc_rad2deg(float rad)
{
	return rad * 180.0f / PI;
}

void misc_minMaxLimit(float *alpha, float min, float max)
{
	if (*alpha <= min)
		*alpha = min;
	else if (*alpha >= max)
		*alpha = max;

	return;
}

void misc_keyboardInput(SDLKey key, char buffer[], int *current, int maxlen,
	char string)
{
	if (*current < maxlen)
	{
		if (key >= '0' && key <= '9')
		{
			if ((SDL_GetModState() & KMOD_SHIFT) && string)
			{
				char const numshift[] =
				{')','!','@','#','$','%','^','&','*','('};

				key = numshift[key - '0'];
			}
			buffer[*current] = key;
			(*current) ++;
		}
		if ((key >= 'a' && key <= 'z') && string)
		{
			if ((SDL_GetModState() & KMOD_SHIFT))
				key += 'A' - 'a';
			buffer[*current] = key;
			(*current) ++;
		}

		if ((key == ',' || key == '.' || key == '/' || key == ';'
			|| key == '\'' || key == '[' || key == ']'
			|| key == '\\' || key == '-' || key == '=')
			&& !(SDL_GetModState() & KMOD_SHIFT) && string)
		{
			buffer[*current] = key;
			(*current) ++;
		}
		if ((key == ',' || key == '.' || key == '/' || key == ';'
			|| key == '\'' || key == '[' || key == ']'
			|| key == '\\' || key == '-' || key == '=')
			&& (SDL_GetModState() & KMOD_SHIFT) && string)
		{
			if (key == ',')
				key = '<';
			if (key == '.')
				key = '>';
			if (key == '/')
				key = '?';
			if (key == ';')
				key = ':';
			if (key == '\'')
				key = '"';
			if (key == '[')
				key = '{';
			if (key == ']')
				key = '}';
			if (key == '\\')
				key = '|';
			if (key == '-')
				key = '_';
			if (key == '=')
				key = '+';

			buffer[*current] = key;
			(*current) ++;
		}
		if ((key == SDLK_SPACE) && string)
		{
			buffer[*current] = key;
			(*current) ++;
		}
	}
	if (key == SDLK_BACKSPACE)
		if (*current > 0)
			buffer[--(*current)] = 0;
	
	return;
}

/* check if segments do intersect. If they do, give a point of intersection */
int misc_segmentsIntersect(float x1, float y1, float x2, float y2,
			float x3, float y3, float x4, float y4,
			float *xr, float *yr)
{
	float z1, z2, z3, z4;

	if (fmaxf(x1, x2) < fminf(x3, x4)
	 || fmaxf(x3, x4) < fminf(x1, x2)
	 || fmaxf(y1, y2) < fminf(y3, y4)
	 || fmaxf(y3, y4) < fminf(y1, y2))
		return 0;
	
	z1 = (x3 - x1)*(y2 - y1) - (y3 - y1)*(x2 - x1);
	z2 = (x4 - x1)*(y2 - y1) - (y4 - y1)*(x2 - x1);
	z3 = (x1 - x3)*(y4 - y3) - (y1 - y3)*(x4 - x3);
	z4 = (x2 - x3)*(y4 - y3) - (y2 - y3)*(x4 - x3);

	if (z1 == 0.0f
	 || z2 == 0.0f
	 || (z1 > 0.0f && z2 < 0.0f && z3 < 0.0f && z4 > 0.0f)
	 || (z1 < 0.0f && z2 > 0.0f && z3 > 0.0f && z4 < 0.0f))
	{
		float a1, b1, a2, b2;
		
		/* Now we just need to check where the segments do intersect */

		if (x1 == x2 && x3 == x4) /* Two vertical segments: ABORT! */
			return 0;
		else if (x1 == x2) /* prepare for vertical segment */
		{
			a2 = (y4-y3)/(x4-x3);
			b2 = y3 - a2*x3;

			*xr = x1;
			*yr = ((*xr)*a2) + b2;
		}
		else if (x3 == x4) /* prepare for vertical segment */
		{
			a1 = (y2-y1)/(x2-x1);
			b1 = y1 - a1*x1;

			*xr = x3;
			*yr = ((*xr)*a1) + b1;
		}
		else /* if there are no vertical segments */
		{
			a1 = (y2-y1)/(x2-x1);
			b1 = y1 - a1*x1;

			a2 = (y4-y3)/(x4-x3);
			b2 = y3 - a2*x3;

			*xr = (b1-b2)/(a2-a1);
			*yr = ((*xr)*a1) + b1;
		}
		/* End of checking */

		return 1;
	}
	else
		return 0;
}
