/*
 *  (C) Copyright 2007-2009 by FreeShoppa Development Team
 *                        (http://chomzee.ethernet.pl/freeshoppa/)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <sys/stat.h>
#include <sys/types.h>

#include "freeshoppa.h"

static TCPsocket tcpsock;
static SDL_Thread *tickThread;
static SDL_mutex *mut;
SDL_sem *server_semUI;

static char dedicated;
static char practice;
static char end;
static char rcRestart;

typedef struct
{
	char status;
	char connecting;
	int id;
	TCPsocket sock;
	SDL_Thread *thread;
	Uint8 buf[BUFLEN];
	int len;
	Uint8 *argv[MAX_LINES_PACKET];
	int argc;
	Uint32 connectTime;
	char host;
} Client;

static Client *clients;
static int max_clients;

typedef struct
{
	char status;
	char name[81];
	char game_version[81];
	char protocol_version[81];
	char ready;
	char ingame;
	char currentBall;
	Ball ball[4];
	char cow;
	Client *ptr;
} Player;

static Player players[6];
static int max_players;
static int count_players;
static int gametype;
static int balls;
static int health;

static char gameInProgress;
static Map *map;
static int turn; /* contains index of player currently doing the turn */
static int turnBall; /* contains index of ball which is in use now */
static int secondPending;

static Uint32 secondTime;

static int cowTeams[6];

static void randomizeDrop(int *x, int *y, int *type)
{
	int p;
onemoretime:
	*x = rand()%(map->xDropMax-map->xDropMin + 1) + map->xDropMin + 1;
	*y = rand()%(map->yDropMax-map->yDropMin + 1) + map->yDropMin + 1;

	for (p=0; p<map->elementsCount; p++)
		if (map->elements[p].type == 5)
		{
			if ((*x) >= map->elements[p].arg[0] &&
				(*y) >= map->elements[p].arg[1] &&
				(*x) <= map->elements[p].arg[2] &&
				(*y) <= map->elements[p].arg[3])
				goto onemoretime;
		}


	if (gametype == 3)
		*type = rand()%3; /* only 3 first weapons for proper */
	else
		*type = rand()%6;

	return;
}

/* save the status file */
static int saveStatusFile(void)
{
	FILE *status;
	char filenameBuf[256];

	if (!commands_sv_status)
		return 0;

#ifdef WIN32
	snprintf(filenameBuf, 256, commands_sv_statusfile);
	status = fopen(filenameBuf, "w");
#else
	snprintf(filenameBuf, 256, "%s/.freeshoppa/%s",
		getenv("HOME"), commands_sv_statusfile);
	status = fopen(filenameBuf, "w");
	if (!status)
	{
		snprintf(filenameBuf, 256, "%s/.freeshoppa",
			getenv("HOME"));
		mkdir(filenameBuf, 0755);

		snprintf(filenameBuf, 256, "%s/.freeshoppa/%s",
			getenv("HOME"), commands_sv_statusfile);
		status = fopen(filenameBuf, "w");
	}
#endif

	if (status)
	{
		int p;

		fprintf(status, "Server Name:      %s\n", commands_sv_name);
		fprintf(status, "Address:          %s\n", commands_sv_hostname);
		fprintf(status, "Port:             %d\n", commands_sv_port);
		fprintf(status, "Game Type:        ");
		switch (commands_sv_gametype)
		{
			case 0:
				fprintf(status, "Shoppa\n");
				break;
			case 1:
				fprintf(status, "FlyShoppa\n");
				break;
			case 2:
				fprintf(status, "WxW\n");
				break;
			case 3:
				fprintf(status, "Proper\n");
				break;
			case 4:
				fprintf(status, "RopeRace\n");
				break;
			default:
				fprintf(status, "Unknown\n");
		}
		fprintf(status, "Map:              %s\n", commands_sv_map);
		fprintf(status, "Server version:   %s\n", VERSION);
		fprintf(status, "Protocol version: %s\n", PROTOCOL_VERSION);
		fprintf(status, "Players:          %d/%d\n",
			count_players, max_players);

		for (p=0; p<max_players; p++)
			if (players[p].status)
				fprintf(status, "    %s\n", players[p].name);
		fclose(status);
		return 0;
	}
	else
	{
		console_print("  server: failed to save status file.");
		return 1;
	}
}

/* erase the status file */
static int eraseStatusFile(void)
{
	FILE *status;
	char filenameBuf[256];

	if (!commands_sv_status)
		return 0;

#ifdef WIN32
	snprintf(filenameBuf, 256, commands_sv_statusfile);
	status = fopen(filenameBuf, "w");
#else
	snprintf(filenameBuf, 256, "%s/.freeshoppa/%s",
		getenv("HOME"), commands_sv_statusfile);
	status = fopen(filenameBuf, "w");
	if (!status)
	{
		snprintf(filenameBuf, 256, "%s/.freeshoppa",
			getenv("HOME"));
		mkdir(filenameBuf, 0755);

		snprintf(filenameBuf, 256, "%s/.freeshoppa/%s",
			getenv("HOME"), commands_sv_statusfile);
		status = fopen(filenameBuf, "w");
	}
#endif

	if (status)
	{
		fprintf(status, "Server Name: %s\n", commands_sv_name);
		fprintf(status, "Server Offline\n");
		fclose(status);
		return 0;
	}
	else
	{
		console_print("  server: failed to save status file.");
		return 1;
	}
}

static void sendTimeout(Client *client)
{
	char buf[BUFLEN];

	memset(buf, 0, BUFLEN);
	sprintf(&buf[strlen(buf)], "%s\r\n", PACKET_ERROR);
	sprintf(&buf[strlen(buf)], "Timeout\r\n\r\n");

	SDLNet_TCP_Send(client->sock, buf, strlen(buf));
	return;
}
static void sendError(Client *client, char *reason)
{
	char buf[BUFLEN];

	memset(buf, 0, BUFLEN);
	sprintf(&buf[strlen(buf)], "%s\r\n", PACKET_ERROR);
	sprintf(&buf[strlen(buf)], "%s\r\n\r\n", reason);

	SDLNet_TCP_Send(client->sock, buf, strlen(buf));
	return;
}
static void sendRCWrongPass(Client *client)
{
	char buf[BUFLEN];

	memset(buf, 0, BUFLEN);
	sprintf(&buf[strlen(buf)], "%s\r\n\r\n", PACKET_RCON_WRONG_PASSWORD);

	SDLNet_TCP_Send(client->sock, buf, strlen(buf));
	return;
}
static void sendServerState(Client *client)
{
	int p;
	char buf[BUFLEN];

	memset(buf, 0, BUFLEN);
	sprintf(&buf[strlen(buf)], "%s\r\n", PACKET_SERVER_STATE);
	sprintf(&buf[strlen(buf)], "%s\r\n", commands_sv_name);
	sprintf(&buf[strlen(buf)], "%d\r\n", gametype);
	sprintf(&buf[strlen(buf)], "%d\r\n", commands_sv_timelimit);
	sprintf(&buf[strlen(buf)], "%d\r\n", commands_sv_fall);
	sprintf(&buf[strlen(buf)], "%s\r\n", commands_sv_map);
	if (commands_sv_passwd[0])
		sprintf(&buf[strlen(buf)], "SECURED\r\n");
	else
		sprintf(&buf[strlen(buf)], "OPEN\r\n");
	sprintf(&buf[strlen(buf)], "%d\r\n", balls);
	sprintf(&buf[strlen(buf)], "%d\r\n", health);
	sprintf(&buf[strlen(buf)], "%d\r\n", max_players);
	sprintf(&buf[strlen(buf)], "%d\r\n", count_players);
	for (p=0; p<max_players; p++)
		if (players[p].status)
			sprintf(&buf[strlen(buf)], "%s\r\n", players[p].name);
	sprintf(&buf[strlen(buf)], "\r\n");


	SDLNet_TCP_Send(client->sock, buf, strlen(buf));

	return;
}
static void sendWelcome(Client *client)
{
	char buf[BUFLEN];

	memset(buf, 0, BUFLEN);

	sprintf(&buf[strlen(buf)], "%s\r\n", PACKET_WELCOME);
	if (!strlen(commands_sv_motd1))
		sprintf(&buf[strlen(buf)], " \r\n");
	else
		sprintf(&buf[strlen(buf)], "%s\r\n", commands_sv_motd1);

	if (!strlen(commands_sv_motd2))
		sprintf(&buf[strlen(buf)], " \r\n");
	else
		sprintf(&buf[strlen(buf)], "%s\r\n", commands_sv_motd2);

	if (!strlen(commands_sv_motd3))
		sprintf(&buf[strlen(buf)], " \r\n");
	else
		sprintf(&buf[strlen(buf)], "%s\r\n", commands_sv_motd3);

	if (!strlen(commands_sv_motd4))
		sprintf(&buf[strlen(buf)], " \r\n");
	else
		sprintf(&buf[strlen(buf)], "%s\r\n", commands_sv_motd4);

	if (!strlen(commands_sv_motd5))
		sprintf(&buf[strlen(buf)], " \r\n");
	else
		sprintf(&buf[strlen(buf)], "%s\r\n", commands_sv_motd5);

	if (!strlen(commands_sv_motd6))
		sprintf(&buf[strlen(buf)], " \r\n");
	else
		sprintf(&buf[strlen(buf)], "%s\r\n", commands_sv_motd6);

	if (!strlen(commands_sv_motd7))
		sprintf(&buf[strlen(buf)], " \r\n");
	else
		sprintf(&buf[strlen(buf)], "%s\r\n", commands_sv_motd7);

	if (!strlen(commands_sv_motd8))
		sprintf(&buf[strlen(buf)], " \r\n");
	else
		sprintf(&buf[strlen(buf)], "%s\r\n", commands_sv_motd8);

	if (!strlen(commands_sv_motd9))
		sprintf(&buf[strlen(buf)], " \r\n");
	else
		sprintf(&buf[strlen(buf)], "%s\r\n", commands_sv_motd9);
		
	sprintf(&buf[strlen(buf)], "\r\n");

	SDLNet_TCP_Send(client->sock, buf, strlen(buf));
	return;
}
static void sendSay(Client *client, char *message)
{
	int p;
	char buf[BUFLEN];

	if (practice)
		return;

	memset(buf, 0, BUFLEN);

	sprintf(&buf[strlen(buf)], "%s\r\n", PACKET_SAY);
	sprintf(&buf[strlen(buf)], "%s\r\n", players[client->id].name);
	sprintf(&buf[strlen(buf)], "%s\r\n\r\n", message);

	for (p=0; p<max_players; p++)
		if (players[p].status)
			SDLNet_TCP_Send(players[p].ptr->sock, buf, strlen(buf));
	return;
}
static void sendName(char *prev, char *now)
{
	int p;
	char buf[BUFLEN];

	if (practice)
		return;

	memset(buf, 0, BUFLEN);

	sprintf(&buf[strlen(buf)], "%s\r\n", PACKET_NAME);
	sprintf(&buf[strlen(buf)], "%s\r\n", prev);
	sprintf(&buf[strlen(buf)], "%s\r\n\r\n", now);

	for (p=0; p<max_players; p++)
		if (players[p].status)
			SDLNet_TCP_Send(players[p].ptr->sock, buf, strlen(buf));
	return;
}
static void sendState(Client *client)
{
	int p;
	char buf[BUFLEN];

	sprintf(buf, PACKET_STATE);
	sprintf(&buf[strlen(buf)], "\r\n");
	if (gameInProgress)
		sprintf(&buf[strlen(buf)], "IN PROGRESS\r\n");
	else
		sprintf(&buf[strlen(buf)], "WAITING FOR PLAYERS\r\n");

	for (p=0; p<max_players; p++)
		if (players[p].status)
		{
			sprintf(&buf[strlen(buf)], "%d\r\n", p);
			if (players[p].ready)
				sprintf(&buf[strlen(buf)], "READY\r\n");
			else
				sprintf(&buf[strlen(buf)], "UNREADY\r\n");
			if (players[p].ingame)
				sprintf(&buf[strlen(buf)], "INGAME\r\n");
			else
				sprintf(&buf[strlen(buf)], "SPECTATOR\r\n");
			sprintf(&buf[strlen(buf)], "%s\r\n", players[p].name);
		}
	sprintf(&buf[strlen(buf)], "\r\n");

	if (client == NULL)
	{
		for (p=0; p<max_players; p++)
			if (players[p].status)
			{
				SDLNet_TCP_Send(players[p].ptr->sock, buf,
					strlen(buf));
			}
	}
	else
		SDLNet_TCP_Send(client->sock, buf, strlen(buf));
		
	return;
}
static void sendSpec(Client *client)
{
	char buf[BUFLEN];

	sprintf(buf, PACKET_SPEC);
	sprintf(&buf[strlen(buf)], "\r\n\r\n");

	SDLNet_TCP_Send(client->sock, buf, strlen(buf));

	return;
}
static void sendConnected(Client *client)
{
	int p;
	char buf[BUFLEN];

	if (practice)
		return;

	memset(buf, 0, BUFLEN);

	sprintf(&buf[strlen(buf)], "%s\r\n", PACKET_CONNECTED);
	sprintf(&buf[strlen(buf)], "%s\r\n\r\n", players[client->id].name);

	for (p=0; p<max_players; p++)
		if ((players[p].status) && (p != client->id))
			SDLNet_TCP_Send(players[p].ptr->sock, buf, strlen(buf));
	return;
}
static void sendDisconnected(Client *client)
{
	int p;
	char buf[BUFLEN];

	if (practice)
		return;

	memset(buf, 0, BUFLEN);

	sprintf(&buf[strlen(buf)], "%s\r\n", PACKET_DISCONNECTED);
	sprintf(&buf[strlen(buf)], "%s\r\n\r\n", players[client->id].name);

	for (p=0; p<max_players; p++)
		if ((players[p].status) && (p != client->id))
			SDLNet_TCP_Send(players[p].ptr->sock, buf, strlen(buf));
	return;
}
static void sendIngame(Client *client)
{
	int p;
	char buf[BUFLEN];

	memset(buf, 0, BUFLEN);

	sprintf(&buf[strlen(buf)], "%s\r\n", PACKET_INGAME);
	sprintf(&buf[strlen(buf)], "%s\r\n\r\n", players[client->id].name);

	for (p=0; p<max_players; p++)
		if (players[p].status)
			SDLNet_TCP_Send(players[p].ptr->sock, buf, strlen(buf));
	return;
}
static void sendSpectator(Client *client)
{
	int p;
	char buf[BUFLEN];

	memset(buf, 0, BUFLEN);

	sprintf(&buf[strlen(buf)], "%s\r\n", PACKET_SPECTATOR);
	sprintf(&buf[strlen(buf)], "%s\r\n\r\n", players[client->id].name);

	for (p=0; p<max_players; p++)
		if (players[p].status)
			SDLNet_TCP_Send(players[p].ptr->sock, buf, strlen(buf));
	return;
}
static void sendReady(Client *client)
{
	int p;
	char buf[BUFLEN];

	memset(buf, 0, BUFLEN);

	sprintf(&buf[strlen(buf)], "%s\r\n", PACKET_READY);
	sprintf(&buf[strlen(buf)], "%s\r\n\r\n", players[client->id].name);

	for (p=0; p<max_players; p++)
		if (players[p].status)
			SDLNet_TCP_Send(players[p].ptr->sock, buf, strlen(buf));
	return;
}
static void sendUnready(Client *client)
{
	int p;
	char buf[BUFLEN];

	memset(buf, 0, BUFLEN);

	sprintf(&buf[strlen(buf)], "%s\r\n", PACKET_UNREADY);
	sprintf(&buf[strlen(buf)], "%s\r\n\r\n", players[client->id].name);

	for (p=0; p<max_players; p++)
		if (players[p].status)
			SDLNet_TCP_Send(players[p].ptr->sock, buf, strlen(buf));
	return;
}

/* shuffles an array */
static void shuffle(int *tab, int len)
{
	int p;

	for (p=0; p<len; p++)
		tab[p] = -1;

	for (p=0; p<len; p++)
	{
		int q = rand()%len;

		while (tab[q] != -1)
		{
			q++;
			if (q==len)
				q=0;
		}
		tab[q] = p;
	}
	return;
}

static void sendGameStart()
{
	int p;
	char buf[BUFLEN];

	memset(buf, 0, BUFLEN);

	sprintf(&buf[strlen(buf)], "%s\r\n\r\n", PACKET_GAME_START);

	if (practice)
		SDLNet_TCP_Send(players[0].ptr->sock, buf, strlen(buf));
	else
		for (p=0; p<max_players; p++)
			if (players[p].status)
				SDLNet_TCP_Send(players[p].ptr->sock,
					buf, strlen(buf));
	return;
}
static void sendGameEnd(int won)
{
	int p;
	if (practice)
	{
		sendError(&clients[0], "Game has ended");
		return;
	}

	for (p=0; p<max_players; p++)
		if (players[p].status)
		{
			char buf[BUFLEN];
			memset(buf, 0, BUFLEN);

			sprintf(buf, PACKET_GAME_END);
			sprintf(&buf[strlen(buf)], "\r\n");
			sprintf(&buf[strlen(buf)], "%d\r\n\r\n", won);
			SDLNet_TCP_Send(players[p].ptr->sock, buf, strlen(buf));
		}
	return;
}

static void sendCoords(int readyPlayers)
{
	int p;
	char buf[BUFLEN];
	memset(buf, 0, BUFLEN);

	sprintf(buf, PACKET_COORDS);
	sprintf(&buf[strlen(buf)], "\r\n");
	sprintf(&buf[strlen(buf)], "%d\r\n", readyPlayers);
	sprintf(&buf[strlen(buf)], "%d\r\n", balls);
	for (p=0; p<max_players; p++)
		if (players[p].status && players[p].ingame && players[p].ready)
		{
			int q;

			sprintf(&buf[strlen(buf)], "%d\r\n", p);
			for (q=0; q<balls; q++)
			{
				int tmp;
				sprintf(&buf[strlen(buf)], "%d\r\n",
					players[p].ball[q].health);

				memcpy(&tmp, &(players[p].ball[q].pos.x), 4);
				sprintf(&buf[strlen(buf)], "%d\r\n", tmp);
				memcpy(&tmp, &(players[p].ball[q].pos.y), 4);
				sprintf(&buf[strlen(buf)], "%d\r\n", tmp);
			}
			
		}

	sprintf(&buf[strlen(buf)], "\r\n");

	for (p=0; p<max_players; p++)
	{
		if (players[p].status)
			SDLNet_TCP_Send(players[p].ptr->sock, buf, strlen(buf));
		if (practice)
			break;
	}
	return;
}
static void sendTurn()
{
	int p;
	int xDrop = 0;
	int yDrop = 0;
	int type = 0;
	int wind;
	char buf[BUFLEN];



	secondTime = SDL_GetTicks();

	/* randomize the wind */
	wind = rand()%201 - 100;

	if (gametype != 4) /* no drop for RR */
		randomizeDrop(&xDrop, &yDrop, &type);

	memset(buf, 0, BUFLEN);
	sprintf(&buf[strlen(buf)], "%s\r\n", PACKET_TURN);
	sprintf(&buf[strlen(buf)], "%d\r\n", turn);
	sprintf(&buf[strlen(buf)], "%d\r\n", turnBall);
	sprintf(&buf[strlen(buf)], "%d\r\n", wind);
	if (gametype != 4)
		sprintf(&buf[strlen(buf)], "%d\r\n%d\r\n%d\r\n",
			xDrop, yDrop, type);
	sprintf(&buf[strlen(buf)], "\r\n");


	for (p=0; p<max_players; p++)
		if (players[p].status && p != turn && !practice)
			SDLNet_TCP_Send(players[p].ptr->sock, buf, strlen(buf));
	
	memset(buf, 0, BUFLEN);
	sprintf(&buf[strlen(buf)], "%s\r\n", PACKET_GO_TURN);
	sprintf(&buf[strlen(buf)], "%d\r\n", turn);
	sprintf(&buf[strlen(buf)], "%d\r\n", turnBall);
	sprintf(&buf[strlen(buf)], "%d\r\n", wind);
	if (gametype != 4)
		sprintf(&buf[strlen(buf)], "%d\r\n%d\r\n%d\r\n",
			xDrop, yDrop, type);
	sprintf(&buf[strlen(buf)], "\r\n");


	/* players[turn].state must be positive, because it was checked
	 * before */
	SDLNet_TCP_Send(players[turn].ptr->sock, buf, strlen(buf));
	return;
}
static void sendCow(Client *client)
{
	int p;
	char buf[BUFLEN];

	memset(buf, 0, BUFLEN);

	sprintf(&buf[strlen(buf)], "%s\r\n", PACKET_COW);
	sprintf(&buf[strlen(buf)], "%s\r\n\r\n", players[client->id].name);

	if (!practice)
	{
		for (p=0; p<max_players; p++)
			if (players[p].status)
				SDLNet_TCP_Send(players[p].ptr->sock, buf,
					strlen(buf));
	}
	else
		SDLNet_TCP_Send(clients[0].sock, buf, strlen(buf));
	
	return;
}

static void startGame(int readyPlayers)
{
	int p, q, r;
	int *tab; /* an array used for assigning spawnpoints to balls */
	gameInProgress = 1;

	tab = (int*)malloc(sizeof(int)*map->spawnsCount);
	if (!tab)
	{
		console_print("  server: Could not allocate memory");
		return;
	}
	shuffle(tab, map->spawnsCount);

	for (p=0; p<6; p++)
		for (q=0; q<4; q++)
			players[p].ball[q].status = 0;

	/* assign a spawnpoint to every ball */
	q=0;
	for (p=0; p<max_players; p++)
		if (players[p].status && players[p].ingame)
		{
			players[p].cow = 0;
			players[p].currentBall = -1;
			for (r=0; r<balls; r++)
			{
				players[p].ball[r].status = 1;
				players[p].ball[r].health = health;
				players[p].ball[r].pos.x =
					(float)map->spawns[tab[q]].arg[0];
				players[p].ball[r].pos.y =
					(float)map->spawns[tab[q]].arg[1];
				q++;
			}
		}
	free(tab);
	sendGameStart();
	SDL_Delay(3000);
	sendCoords(readyPlayers);
	sendState(NULL);

	memset(cowTeams, 0, sizeof(cowTeams));
	for (p=0; p<max_players; p++)
		if (players[p].status && players[p].ingame)
			cowTeams[p] = balls*health;

	/* draw player whitch starts the game */
	p = rand()%6;
	while (!(players[p].status && players[p].ingame))
		p = rand()%6;
	turn = p;

	players[turn].currentBall++;
	turnBall = 0;
	secondPending = 0;

	sendTurn();

	return;
}
static char alive(int p)
{
	int q;

	for (q=0; q<balls; q++)
		if (players[p].ball[q].status)
			return 1;
	return 0;
}

static int cow()
{
	int p, q;
	int min=-1;
	int minval=1000;
	char twomin=0;
	int ncowTeams[6];
	int playing=0;

	for (p=0; p<max_players; p++)
		if (players[p].status && players[p].ingame && players[p].ready
			&& alive(p))
		{
			ncowTeams[p] = 0;
			for (q=0; q<balls; q++)
				ncowTeams[p] += players[p].ball[q].health;
		}
		else
			ncowTeams[p] = 0;

	for (p=0; p<max_players; p++)
	{
		if (cowTeams[p]==0)
			continue;
		playing++;
		if (cowTeams[p]<minval)
		{
			min=p;
			minval=cowTeams[p];
			twomin=0;
		}
		else if (cowTeams[p] == minval)
		{
			twomin=1;
		}
	}
	if (playing <= 2)
	{
		memcpy(cowTeams, ncowTeams, sizeof(cowTeams));
		return 0;
	}


	if (twomin)
	{
		memcpy(cowTeams, ncowTeams, sizeof(cowTeams));
		return 0;
	}
	if (min == turn)
	{
		memcpy(cowTeams, ncowTeams, sizeof(cowTeams));
		return 0;
	}

	if (ncowTeams[min] >= cowTeams[min])
	{
		memcpy(cowTeams, ncowTeams, sizeof(cowTeams));
		return 0;
	}

	for (p=0; p<max_players; p++)
		if (cowTeams[p] && p!=min && p!=turn)
		{
			if (ncowTeams[p] < cowTeams[p])
			{
				memcpy(cowTeams, ncowTeams, sizeof(cowTeams));
				return 0;
			}
		}
	
	sendCow(players[turn].ptr);
	memcpy(cowTeams, ncowTeams, sizeof(cowTeams));
	return 1;
}

static void startNextTurn()
{
	int p;
	int ready = 0;
	int playing = 0;
	char rr = 0;
	int won = -1;
		

	/* check number of ready players */
	for (p=0; p<6; p++)
		if (players[p].status && players[p].ingame
				&& players[p].ready)
					ready++;
	/* check number of still playing players */
	for (p=0; p<6; p++)
		if (players[p].status && players[p].ingame
				&& players[p].ready && alive(p))
					playing++;

	/* check if rr has finished */
	if (gametype == 4)
	{
		int q;
		for (p=0; p<6; p++) if (players[p].status)
		for (q=0; q<4; q++) if (players[p].ball[q].status)
		{
			if (players[p].ball[q].pos.x >= map->xDropMin
				&& players[p].ball[q].pos.x <= map->xDropMax
				&& players[p].ball[q].pos.y >= map->yDropMin
				&& players[p].ball[q].pos.y <= map->yDropMax)
				{
					won = p;
					rr = 1;
				}
		}
	}

	/* It is end of a game if there are less than 2 players playing 
	 * or rope race has ended */
	if (playing < 2 ||  rr)
	{
		/* check who has won */
		if (playing == 0)
			won = -1;
		else if (playing == 1)
		{
			int p;
			for (p=0; p<6; p++)
				if (players[p].status && players[p].ingame
					&& players[p].ready && alive(p))
					won = p;

		}

		for (p=0; p<6; p++)
			players[p].ready = 0;

		gameInProgress = 0;
		sendState(NULL);
		sendGameEnd(won);
		return;
	}

	sendCoords(ready);
	sendState(NULL);

	if (gametype != 4)
		players[turn].cow = cow();


	lostTurn:;
	p = turn+1;
	if (p == 6)
		p = 0;
	while (!(players[p].status && players[p].ingame && alive(p)))
	{
		p++;
		if (p == 6)
			p = 0;
	}
	turn = p;
	if (players[p].cow)
	{
		players[p].cow = 0;
		goto lostTurn;
	}


	players[turn].currentBall++;
	if (players[turn].currentBall == balls)
		players[turn].currentBall = 0;
	while (!players[turn].ball[(int)players[turn].currentBall].status)
	{
		players[turn].currentBall++;
		if (players[turn].currentBall == balls)
			players[turn].currentBall = 0;

	}

	turnBall = players[turn].currentBall;
	secondPending = 0;

	sendTurn();
	return;
}

static void sendSecond()
{
	int p;
	char buf[BUFLEN];

	if (practice)
		return;

	memset(buf, 0, BUFLEN);
	for (p=0; p<players[turn].ptr->argc; p++)
	{
		sprintf(&buf[strlen(buf)], (char*)players[turn].ptr->argv[p]);
		sprintf(&buf[strlen(buf)], "\r\n");

	}
	sprintf(&buf[strlen(buf)], "\r\n");

	for (p=0; p<max_players; p++)
		if (players[p].status && p != turn)
			SDLNet_TCP_Send(players[p].ptr->sock, buf, strlen(buf));
	
	return;
}


/* use INSIDE client thread */
static void disconnectClient2(Client *client)
{
	SDLNet_TCP_Close(client->sock);

	if (client->host && !dedicated)
		end = 1;

	if (client->connecting == 2 && players[client->id].status)
	{
	
		if (!practice)
			console_print("  server: %s^0 (%s^0) disconnected",
				players[client->id].name, colors[client->id]);
		sendDisconnected(client);
		memset(&players[client->id], 0, sizeof(Player));
		count_players--;
		if (!gameInProgress)
			sendState(NULL);
		else
		{
			if (turn == client->id)
				startNextTurn();
		}
		saveStatusFile();
	}

	memset(client, 0, sizeof(Client));
	SDL_mutexV(mut);

	/* a bit of debug
	console_print("  server: client disconnected.");
	*/
	return;
}

/* use OUTSIDE client thread */
static void disconnectClient(Client *client)
{
	if (client->thread)
	{
		SDL_KillThread(client->thread);
		client->thread = NULL;
	}
	disconnectClient2(client);
	return;
}


/* timeout thread */
static int tick(void *ptr)
{
	for (;;)
	{
		int p;

		if (practice)
		{
			SDL_Delay(1000);
			continue;
		}

		SDL_mutexP(mut);
		for (p=0; p<max_clients; p++)
		{
			if (clients[p].status && clients[p].connecting < 2)
				if (SDL_GetTicks()-clients[p].connectTime
					> 10000)
				{
					sendTimeout(&clients[p]);
					disconnectClient(&clients[p]);
				}
			if (clients[p].status && clients[p].connecting == 2
				&& gameInProgress && turn == p)
				if (SDL_GetTicks()-secondTime > 30000)
				{
					sendTimeout(&clients[p]);
					disconnectClient(&clients[p]);
				}
		}
		SDL_mutexV(mut);
		SDL_Delay(1000);
	}
	return 0;
}

/* remember" disconnect client with 'disconnectClient2(); return1;' !! */
static int preparePacket(Client *client)
{
	int p;

	client->argc = 1;
	client->argv[0] = &client->buf[0];

	for (p=0; p<client->len; p++)
	{
		if (p>0 && p<(client->len-1)
			&& client->buf[p] == '\n'
			&& client->buf[p-1] == '\r')
		{
			client->argv[client->argc] = &client->buf[p+1];
			client->argc++;

			client->buf[p-1] = 0;

			if (client->argc>=MAX_LINES_PACKET)
			{
				disconnectClient2(client);
				return 1;
			}
		}
	}

	return 0;
}

/* remember: disconnect client with 'disconnectClient2(); return1;' !! */
static int processPacket(Client *client)
{
	/* just a debug
	int p;
	console_print("Got packet, %d lines:", client->argc);
	for (p=0; p<client->argc; p++)
		console_print("Line %d: %s", p, client->argv[p]);
	console_print("End of packet");
	*/

	if (strcmp("HELLO", (char*)client->argv[0]) == 0
		&& client->argc == 1
		&& client->connecting == 0)
	{
		sendServerState(client);
		if (max_players == count_players)
		{
			disconnectClient2(client);
			return 1;
		}
		else
			client->connecting = 1;
	}
	else if (strcmp("SCAN", (char*)client->argv[0]) == 0
		&& client->argc == 1
		&& client->connecting == 0)
	{
		sendServerState(client);
		disconnectClient2(client);
		return 1;
	}
	else if (strcmp("JOIN", (char*)client->argv[0]) == 0
		&& (client->argc == 5 || client->argc == 6)
		&& client->connecting == 1)
	{
		int p;
		char done = 0;

		if (strcmp((char*)client->argv[4], PROTOCOL_VERSION))
		{
			char buf[128];
			sprintf(buf, "Wrong protocol version, "
				"you use %s, server uses %s. Use FreeShoppa-%s",
				(char*)client->argv[4], PROTOCOL_VERSION,
				VERSION);
			sendError(client, buf);
			disconnectClient2(client);
			return 1;
		}

		if (commands_sv_passwd[0] && client->argc == 5)
		{
			sendError(client, "Password requied");
			disconnectClient2(client);
			return 1;
		}
		if (commands_sv_passwd[0] && client->argc == 6)
		{
			if (strcmp(commands_sv_passwd, (char*)client->argv[5]))
			{
				sendError(client, "Wrong password");
				disconnectClient2(client);
				return 1;
			}
		}
		if (strlen((char*)client->argv[1]) > 80)
		{
			sendError(client, "Name too long");
			disconnectClient2(client);
			return 1;
		}

		if (
			strlen((char*)client->argv[2]) != 32
			|| strcmp((char*)client->argv[2], map->md5)
			)
		{
			sendError(client, "Wrong md5 (your map differs "
				"from server's map)");
			disconnectClient2(client);
			return 1;
		}

		for (p=0; p<max_players; p++)
		{
			if (!players[p].status)
			{
				client->id = p;
				players[p].status = 1;
				snprintf(players[p].name, 81, "%s",
					client->argv[1]);
				players[p].ptr = client;

				client->connecting = 2;
				count_players++;
				done = 1;
				if (practice)
				{
					players[p].ingame = 1;
					players[p].ready = 1;
				}
				else
					break;
			}
		}
		if (done)
		{
			sendWelcome(client);
			sendConnected(client);
			if (!gameInProgress)
				sendState(NULL);
			else
			{
				sendSpec(client);
				sendState(client);
			}
			if (!practice)
				console_print(
				"  server: %s^0 (%s^0) connected",
				players[p].name, colors[p]);
			else
				startGame(p);
			saveStatusFile();
		}
		else
		{
			sendError(client, "Server full");
			disconnectClient2(client);
			return 1;
		}
	}
	else if (strcmp("SAY", (char*)client->argv[0]) == 0
		&& client->argc == 2
		&& client->connecting == 2)
	{
		sendSay(client, (char*)client->argv[1]);
	}
	else if (strcmp("NAME", (char*)client->argv[0]) == 0
		&& client->argc == 2
		&& client->connecting == 2)
	{
		char prevName[81];
		if (!practice)
		{
			snprintf(prevName, 81, players[client->id].name);
			snprintf(players[client->id].name, 81,
				(char*)client->argv[1]);
			sendName(prevName, players[client->id].name);
			if (!gameInProgress)
				sendState(NULL);
			saveStatusFile();
		}
	}
	else if (strcmp("INGAME", (char*)client->argv[0]) == 0
		&& client->argc == 1
		&& client->connecting == 2)
	{
		if (!players[client->id].ingame
			&& !gameInProgress)
		{
			players[client->id].ingame = 1;
			players[client->id].ready = 0;
			sendIngame(client);
			sendState(NULL);
		}
	}
	else if (strcmp("SPECTATOR", (char*)client->argv[0]) == 0
		&& client->argc == 1
		&& client->connecting == 2)
	{
		if (players[client->id].ingame
			&& !gameInProgress)
		{
			players[client->id].ingame = 0;
			players[client->id].ready = 0;
			sendSpectator(client);
			sendState(NULL);
		}
	}
	else if (strcmp("READY", (char*)client->argv[0]) == 0
		&& client->argc == 1
		&& client->connecting == 2)
	{
		if (!players[client->id].ready && players[client->id].ingame
			&& !gameInProgress)
		{
			int p;
			int ready=0;
			int unready=0;

			players[client->id].ready = 1;
			sendReady(client);
			sendState(NULL);

			/* check number of ready and unready players */
			for (p=0; p<6; p++)
			{
				if (players[p].status && players[p].ingame
					&& players[p].ready)
					ready++;
				if (players[p].status && players[p].ingame
					&& !players[p].ready)
					unready++;
			}
			/* start the game if there are more than 1 ready
			 * player and no unready players */
			if (ready > 1 && unready == 0)
				startGame(ready);
		}
	}
	else if (strcmp("UNREADY", (char*)client->argv[0]) == 0
		&& client->argc == 1
		&& client->connecting == 2)
	{
		if (players[client->id].ready && players[client->id].ingame
			&& !gameInProgress)
		{
			players[client->id].ready = 0;
			sendUnready(client);
			sendState(NULL);
		}
	}
	else if (strcmp("SECOND", (char*)client->argv[0]) == 0
		&& client->argc == 224
		&& client->connecting == 2)
	{
		int sec;

		sscanf((char*)client->argv[1], "%d", &sec);
		if (client->id == turn && secondPending == sec)
		{
			secondTime = SDL_GetTicks();
			sendSecond();
			secondPending++;
		}
	}
	else if (strcmp("SYNC", (char*)client->argv[0]) == 0
		&& client->argc == 73
		&& client->connecting == 2)
	{
		int p, q, tmp;

		if (client->id == turn || practice)
		{
			for (p=0; p<6; p++)
				for (q=0; q<4; q++)
				{
					sscanf((char*)client->argv
						[1+p*12 + q*3], "%d",
						&(players[p].ball[q].health));
					if (players[p].ball[q].health)
						players[p].ball[q].status = 1;
					else
						players[p].ball[q].status = 0;

					sscanf((char*)client->argv
						[1+p*12 + q*3 + 1],
						"%d", &tmp);
					memcpy(&(players[p].ball[q].pos.x),
						&tmp, 4);

					sscanf((char*)client->argv
						[1+p*12 + q*3 + 2],
						"%d", &tmp);
					memcpy(&(players[p].ball[q].pos.y),
						&tmp, 4);

				}
			startNextTurn();
		}
	}
	else if (strcmp("RCON RESTART", (char*)client->argv[0]) == 0
		&& client->argc > 1
		&& client->connecting == 2)
	{
		if (!strcmp(commands_rc_svpasswd, (char*)client->argv[1])
			&& dedicated)
		{
			int p;
			for (p=2; p<client->argc; p++)
			{
				commands_process((char*)client->argv[p]);
			}
			rcRestart = 1;
			end = 1;
			console_print("Restarting server...");
		}
		else
		{
			sendRCWrongPass(client);
		}
	}
	else if (strcmp("RCON KICK", (char*)client->argv[0]) == 0
		&& client->argc == 3
		&& client->connecting == 2)
	{
		if (!strcmp(commands_rc_svpasswd, (char*)client->argv[1])
			&& dedicated)
		{
			int index;
			sscanf((char*)client->argv[2], "%d", &index);
			if (index<0 || index>5)
			{
				/* wtf? */
			}
			else
			{
				if (players[index].status
					&& client->id != index)
					disconnectClient(players[index].ptr);
			}
		}
		else
		{
			sendRCWrongPass(client);
		}
	}
	else
	{
		sendError(client, "Protocol incompatibility");
		disconnectClient2(client);
		return 1;
	}

	return 0;
}


static int serveClient(void *ptr)
{
	Client *client = (Client *)ptr;
	int result;
	int p = 0;

	srand((int)time(NULL));

	for (;;)
	{
		if (p >= BUFLEN)
		{
			disconnectClient2(client);
			return 0;
		}
		result = SDLNet_TCP_Recv(client->sock, &client->buf[p], 1);
		if (result<1)
		{
			disconnectClient2(client);
			return 0;
		}
		p++;
		if (p>=4 && client->buf[p-1]=='\n' && client->buf[p-2]=='\r'
			 && client->buf[p-3]=='\n' && client->buf[p-4]=='\r')
		{
			client->buf[p-4] = 0;
			client->len = p-4;
			p = 0;

			if (preparePacket(client))
				return 0;
			SDL_mutexP(mut);
			if (processPacket(client))
				return 0;
			SDL_mutexV(mut);
		}

	}
	return 0;

}

int server_init()
{
	IPaddress ip;

	console_print("Initializing server...");

	end = 0;

	mut = NULL;
	mut = SDL_CreateMutex();
	if (mut == NULL)
	{
		console_print("  server: Couldn't create mutex!");
		console_print("^1FAILED!!!");
		return 1;
	}

	max_players = commands_sv_max_players;
	count_players = 0;
	health = commands_sv_health;

	balls = commands_sv_balls;

	map = map_load(commands_sv_map, 0);
	if (!map)
	{
		console_print("  server: couldn't load map: %s.",
			commands_sv_map);
		console_print("^1FAILED!!!");
		return 1;
	}
	if (map->spawnsCount < max_players*balls)
	{
		console_print("  server: There are too less spawnpoints!");
		console_print("^1FAILED!!!");
		return 1;
	}

	
	gametype = commands_sv_gametype;
	if (!map->gametypes[gametype])
	{
		console_print(
			"  server: you can't play this game on this map!");
		console_print("^1FAILED!!!");
		return 1;
	}

	if (dedicated)
	{
		if (SDLNet_ResolveHost(&ip, NULL, commands_sv_port) == -1)
		{
			console_print("  server: couldn't resolve hostname.");
			console_print("^1FAILED!!!");
			return 1;
		}
	}
	else
	{
		if (SDLNet_ResolveHost(&ip, NULL, 22200) == -1)
		{
			console_print("  server: couldn't resolve hostname.");
			console_print("^1FAILED!!!");
			return 1;
		}
	}

	tcpsock = SDLNet_TCP_Open(&ip);
	if (!tcpsock)
	{
		console_print("  server: couldn't open socket.");
		console_print("^1FAILED!!!");
		return 1;
	}
	else
		console_print("  server: listening.");
	
	if (practice)
	{
		clients = (Client *)
			malloc(sizeof(Client));
		max_clients = 1;
	}
	else
	{
		max_clients = commands_sv_max_clients;
		clients = (Client *)
			malloc(sizeof(Client)*max_clients);
	}

	if (gametype == 3 || gametype == 4) /* one ball for proper and rr */
		balls = 1;

	if (clients == NULL)
	{
		console_print("  server: couldn't alloc memory.");
		console_print("^1FAILED!!!");
		return 1;
	}

	memset(clients, 0, sizeof(Client)*max_clients);
	memset(players, 0, sizeof(Player)*6);
	
	gameInProgress = 0;

	tickThread = SDL_CreateThread(tick, NULL);
	if (tickThread == NULL)
	{
		console_print("  server: couldn't create thread for tick.");
		console_print("^1FAILED!!!");
		return 1;
	}


	console_print("^2DONE");
	saveStatusFile();

	return 0;
}

int server_run()
{
	TCPsocket tmp;
	IPaddress *remoteIP;

	for (;;)
	{
		/* accept connections for clients and create threads for them */
		if	(
			((tmp = SDLNet_TCP_Accept(tcpsock)))
			&&
			((remoteIP = SDLNet_TCP_GetPeerAddress(tmp)))
			)
		{
			int p;
			int done = 0;

			/* a bit of debug
			console_print("  server: host connected: %x %d",
				SDLNet_Read32(&remoteIP->host),
				SDLNet_Read16(&remoteIP->port));
			*/

			SDL_mutexP(mut);
			for (p=0; p<max_clients; p++)
			{
				if (clients[p].status == 0)
				{
					/* another debug
					console_print(
						"  server: client accepted.");
					*/

					memset(&clients[p], 0, sizeof(Client));
					clients[p].connecting = 0;
					clients[p].connectTime = SDL_GetTicks();
					clients[p].status = 1;
					memcpy(&clients[p].sock, &tmp,
						sizeof(TCPsocket));
					if (!dedicated && p == 0)
						clients[p].host = 1;

					clients[p].thread = SDL_CreateThread(
						serveClient, &clients[p]);

					if (clients[p].thread == NULL)
					{
						console_print(
				"  server: couldn't create thread for client.");
						disconnectClient(&clients[p]);

						goto nextClient;
					}
					else
						done = 1;
					break;
					
				}
			}
			SDL_mutexV(mut);
			if (done == 0)
			{
				console_print("  server: no place for client, "
					"closing socket.");
				SDLNet_TCP_Close(tmp);
			}
		}
nextClient:
		SDL_Delay(1);
		if (end)
			return 0;
	}
	return 0;
}

void server_cleanup()
{
	int p;
	console_print("Cleaning server...");

	SDL_mutexP(mut);
	if (tickThread)
	{
		SDL_KillThread(tickThread);
		tickThread = NULL;
	}

	for (p=0; p<max_clients; p++)
		if (clients[p].status)
			disconnectClient(&clients[p]);

	if (clients)
	{
		free(clients);
		clients = NULL;
	}

	if (tcpsock)
	{
		SDLNet_TCP_Close(tcpsock);
		tcpsock = NULL;
	}

	SDL_mutexV(mut);
	SDL_DestroyMutex(mut);

	eraseStatusFile();

	console_print("^2DONE");
	return;
}

int server_thread(void *ptr)
{
	if (ptr == NULL)
		dedicated = 1;
	else
		dedicated = 0;

	if (ptr == (void*)2)
		practice = 1;
	else
		practice = 0;

	if (dedicated)
		commands_loadConfig();

	doitagain:;
	rcRestart = 0;

	if (server_init())
	{
		if (!dedicated)
			SDL_SemPost(server_semUI);
		return 1;
	}
	
	if (!dedicated)
		SDL_SemPost(server_semUI);
	
	server_run();
	server_cleanup();

	if (dedicated)
		commands_saveConfig();

	if (rcRestart)
		goto doitagain; /* I <3 goto :D */

	return 0;
}
