/*
 *  (C) Copyright 2007-2009 by FreeShoppa Development Team
 *                        (http://chomzee.ethernet.pl/freeshoppa/)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <GL/gl.h>
#ifdef WIN32
#include <gl\glext.h>
#endif


#include "freeshoppa.h"

typedef struct _bitmapFile
{
	int width, height;
	unsigned char *data;
} bitmapFile;


int textures_loadBMP(char *filename)
{
	FILE *file;
	int bitmapSize;
	int i;
	bitmapFile bitmap;
	unsigned int texture;

	file = fopen(filename, "rb");
#ifndef WIN32
	if (!file)
	{
		char buf[256];
		snprintf(buf, 256, "%s/%s",
			SHOPPADIR, filename);
		file = fopen(buf, "rb");
	}
#endif
	if (file == NULL)
	{
		console_print("  textures: could not load %s.", filename);
		return 0;
	}

	fseek(file, 18, SEEK_SET);
	fread(&bitmap.width, 4, 1, file);

	fseek(file, 22, SEEK_SET);
	fread(&bitmap.height, 4, 1, file);

	fseek(file, 34, SEEK_SET);
	fread(&bitmapSize, 4, 1, file);

	bitmap.data = (unsigned char*)malloc(bitmapSize);
	fseek(file, 54, SEEK_SET);
	fread(bitmap.data, bitmapSize, 1, file);

	for (i = 0; i < bitmapSize; i += 3)
	{
		int tempByte = bitmap.data[i];
		bitmap.data[i] = bitmap.data[i+2];
		bitmap.data[i+2] = tempByte;
	}

	fclose(file);

	glGenTextures(1, &texture);
	glBindTexture(GL_TEXTURE_2D, texture);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexImage2D( GL_TEXTURE_2D, 0, GL_RGB, bitmap.width, bitmap.height,
		0, GL_RGB, GL_UNSIGNED_BYTE, bitmap.data );
	

	if (bitmap.data != NULL)
	{
		free(bitmap.data);
		bitmap.data = NULL;
	}

	return texture;

}

int textures_loadBMPAlpha(char *filename)
{
	FILE *file;
	int bitmapSize;
	int i, j;
	bitmapFile bitmap;
	unsigned int texture;

	bitmapFile newBitmap;

	file = fopen(filename, "rb");
#ifndef WIN32
	if (!file)
	{
		char buf[256];
		snprintf(buf, 256, "%s/%s",
			SHOPPADIR,filename);
		file = fopen(buf, "rb");
	}
#endif
	if (file == NULL)
	{
		console_print("  textures: could not load %s.", filename);
		return 0;
	}

	fseek(file, 18, SEEK_SET);
	fread(&bitmap.width, 4, 1, file);

	fseek(file, 22, SEEK_SET);
	fread(&bitmap.height, 4, 1, file);

	fseek(file, 34, SEEK_SET);
	fread(&bitmapSize, 4, 1, file);

	bitmap.data = (unsigned char*)malloc(bitmapSize);
	fseek(file, 54, SEEK_SET);
	fread(bitmap.data, bitmapSize, 1, file);

	for (i = 0; i < bitmapSize; i += 3)
	{
		int tempByte = bitmap.data[i];
		bitmap.data[i] = bitmap.data[i+2];
		bitmap.data[i+2] = tempByte;
	}

	fclose(file);




	newBitmap.data = (unsigned char*)malloc(bitmap.width*bitmap.height*4);
	newBitmap.width = bitmap.width;
	newBitmap.height = bitmap.height;

	for (i = 0, j = 0; i < newBitmap.width*newBitmap.height*3;
		i += 3, j+= 4)
	{
		newBitmap.data[j] = bitmap.data[i];
		newBitmap.data[j+1] = bitmap.data[i+1];
		newBitmap.data[j+2] = bitmap.data[i+2];

		newBitmap.data[j+3] = newBitmap.data[j];
	}

	if (bitmap.data != NULL)
	{
		free(bitmap.data);
		bitmap.data = NULL;
	}



	glGenTextures(1, &texture);
	glBindTexture(GL_TEXTURE_2D, texture);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA,
		newBitmap.width, newBitmap.height, 0,
		GL_BGRA, GL_UNSIGNED_BYTE, newBitmap.data );
	

	if (newBitmap.data != NULL)
	{
		free(newBitmap.data);
		newBitmap.data = NULL;
	}

	return texture;

}
int textures_loadBMPWithAlpha(char *filename, char *mask)
{
	FILE *file;
	int bitmapSize;
	int i, j;
	bitmapFile bitmap;
	bitmapFile bitmapMask;
	unsigned int texture;

	bitmapFile newBitmap;

	file = fopen(filename, "rb");
#ifndef WIN32
	if (!file)
	{
		char buf[256];
		snprintf(buf, 256, "%s/%s",
			SHOPPADIR,filename);
		file = fopen(buf, "rb");
	}
#endif
	if (file == NULL)
	{
		console_print("  textures: could not load %s.", filename);
		return 0;
	}

	fseek(file, 18, SEEK_SET);
	fread(&bitmap.width, 4, 1, file);

	fseek(file, 22, SEEK_SET);
	fread(&bitmap.height, 4, 1, file);

	fseek(file, 34, SEEK_SET);
	fread(&bitmapSize, 4, 1, file);

	bitmap.data = (unsigned char*)malloc(bitmapSize);
	fseek(file, 54, SEEK_SET);
	fread(bitmap.data, bitmapSize, 1, file);

	fclose(file);

	file = fopen(mask, "rb");
#ifndef WIN32
	if (!file)
	{
		char buf[256];
		snprintf(buf, 256, "%s/%s",
			SHOPPADIR,mask);
		file = fopen(buf, "rb");
	}
#endif
	if (file == NULL)
	{
		console_print("  textures: could not load %s.", mask);
		return 0;
	}

	fseek(file, 18, SEEK_SET);
	fread(&bitmapMask.width, 4, 1, file);

	fseek(file, 22, SEEK_SET);
	fread(&bitmapMask.height, 4, 1, file);

	fseek(file, 34, SEEK_SET);
	fread(&bitmapSize, 4, 1, file);

	bitmapMask.data = (unsigned char*)malloc(bitmapSize);
	fseek(file, 54, SEEK_SET);
	fread(bitmapMask.data, bitmapSize, 1, file);

	fclose(file);

	newBitmap.data = (unsigned char*)malloc(bitmap.width*bitmap.height*4);
	newBitmap.width = bitmap.width;
	newBitmap.height = bitmap.height;

	for (i = 0, j = 0; i < newBitmap.width*newBitmap.height*3;
		i += 3, j+= 4)
	{
		newBitmap.data[j] = bitmap.data[i];
		newBitmap.data[j+1] = bitmap.data[i+1];
		newBitmap.data[j+2] = bitmap.data[i+2];

		newBitmap.data[j+3] = bitmapMask.data[i];
	}

	if (bitmap.data != NULL)
	{
		free(bitmap.data);
		bitmap.data = NULL;
	}
	if (bitmapMask.data != NULL)
	{
		free(bitmapMask.data);
		bitmapMask.data = NULL;
	}



	glGenTextures(1, &texture);
	glBindTexture(GL_TEXTURE_2D, texture);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA,
		newBitmap.width, newBitmap.height, 0,
		GL_BGRA, GL_UNSIGNED_BYTE, newBitmap.data );
	

	if (newBitmap.data != NULL)
	{
		free(newBitmap.data);
		newBitmap.data = NULL;
	}

	return texture;

}



void textures_unloadBMP(int *texture)
{
	if (texture && *texture)
	{
		glDeleteTextures(1, (unsigned int *)texture);
		*texture = 0;
	}

	return;
}
