/*
 *  (C) Copyright 2007-2009 by FreeShoppa Development Team
 *                        (http://chomzee.ethernet.pl/freeshoppa/)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <GL/gl.h>
#ifdef WIN32
#include <gl\glext.h>
#endif

#include <sys/stat.h>
#include <dirent.h>

#include "freeshoppa.h"

#define MAXMAPS 100

char ui_clientRunning = 0;
char ui_quitNow = 0;
char ui_place = 0;

int ui_texMainMenu = 0;
int ui_texPractice = 0;
int ui_texHost = 0;
int ui_texJoin = 0;
int ui_texOptions = 0;

int ui_texCursor = 0;

static char maps[MAXMAPS][81];
static int mapsCount;

static SDL_Thread *serverthread = NULL;

/* Main Menu */
static void buttonPractice(void);
static void buttonHost(void);
static void buttonJoin(void);
static void buttonOptions(void);
static void buttonExit(void);

/* Return to Main Menu */
static void buttonCancel(void);

/* Practice */
static void buttonStartPractice(void);
static void buttonPracticeDelSvPasswd(void);
static void buttonPracticeShoppa(void);
static void buttonPracticeFlyShoppa(void);
static void buttonPracticeWxW(void);
static void buttonPracticeProper(void);
static void buttonPracticeRR(void);

/* Host */
static void buttonHostHost(void);
static void buttonHostDelSvPasswd(void);
static void buttonHostShoppa(void);
static void buttonHostFlyShoppa(void);
static void buttonHostWxW(void);
static void buttonHostProper(void);
static void buttonHostRR(void);
static void buttonNextMap(void);
static void buttonPrevMap(void);

/* Options */
static void buttonApplyVideo(void);

/* Join */
static void buttonJoinJoin(void);
static void buttonDelClPasswd(void);

Buttons ui_buttonsMainMenu[] = {
/*  x1   x2   y1   y2     ptr  */
  {200, 600, 450, 500, buttonPractice},
  {200, 600, 350, 400, buttonHost},
  {200, 600, 250, 300, buttonJoin},
  {200, 600, 150, 200, buttonOptions},
  {200, 600,  50, 100, buttonExit},
};
const int ui_buttonsMainMenuCount =
	sizeof(ui_buttonsMainMenu)/sizeof(Buttons);

Buttons ui_buttonsPractice[] = {
/*  x1   x2   y1   y2     ptr  */
  {20, 120, 530, 550, buttonPracticeShoppa},
  {130, 230, 530, 550, buttonPracticeFlyShoppa},
  {240, 340, 530, 550, buttonPracticeWxW},
  {75, 175, 500, 520, buttonPracticeProper},
  {185, 280, 500, 520, buttonPracticeRR},
  {675, 750, 395, 410, buttonPracticeDelSvPasswd},
  {450, 525, 335, 350, buttonNextMap},
  {575, 650, 335, 350, buttonPrevMap},
  {550, 650, 10, 50, buttonCancel},
  {690, 790, 10, 50, buttonStartPractice}
};
const int ui_buttonsPracticeCount =
	sizeof(ui_buttonsPractice)/sizeof(Buttons);

Buttons ui_buttonsHost[] = {
/*  x1   x2   y1   y2     ptr  */
  {20, 120, 530, 550, buttonHostShoppa},
  {130, 230, 530, 550, buttonHostFlyShoppa},
  {240, 340, 530, 550, buttonHostWxW},
  {75, 175, 500, 520, buttonHostProper},
  {185, 280, 500, 520, buttonHostRR},
  {675, 750, 395, 410, buttonHostDelSvPasswd},
  {450, 525, 335, 350, buttonNextMap},
  {575, 650, 335, 350, buttonPrevMap},
  {550, 650, 10, 50, buttonCancel},
  {690, 790, 10, 50, buttonHostHost}
};
const int ui_buttonsHostCount =
	sizeof(ui_buttonsHost)/sizeof(Buttons);

Buttons ui_buttonsJoin[] = {
/*  x1   x2   y1   y2     ptr  */
  {300, 380, 250, 290, buttonCancel},
  {420, 500, 250, 290, buttonJoinJoin},
  {475, 550, 310, 325, buttonDelClPasswd}
};
const int ui_buttonsJoinCount =
	sizeof(ui_buttonsJoin)/sizeof(Buttons);

Buttons ui_buttonsOptions[] = {
/*  x1   x2   y1   y2     ptr  */
  {550, 700, 450, 465, buttonApplyVideo},
  {690, 790, 10, 50, buttonCancel}
};
const int ui_buttonsOptionsCount =
	sizeof(ui_buttonsOptions)/sizeof(Buttons);



Edits ui_editsPractice[] = {
  {150, 250, 435, 450, "sv_gametype", 0, 0, &commands_sv_gametype, 0, 1},
  {150, 250, 415, 430, "sv_max_players", 0, 1, &commands_sv_max_players, 0, 1},
  {150, 250, 395, 410, "sv_balls", 0, 1, &commands_sv_balls, 0, 1},
  {150, 250, 375, 390, "sv_timelimit", 0, 1, &commands_sv_timelimit, 0, 2},
  {150, 250, 355, 370, "sv_fall", 0, 1, &commands_sv_fall, 0, 1},
  {150, 250, 335, 350, "sv_health", 0, 1, &commands_sv_health, 0, 3},
  {450, 750, 435, 450, "sv_name", 1, 1, &commands_sv_name, 0, 29},
  {450, 550, 415, 430, "sv_port", 0, 1, &commands_sv_port, 0, 5},
  {450, 650, 395, 410, "sv_passwd", 1, 1, &commands_sv_passwd, 0, 19},
  {450, 650, 355, 370, "sv_map", 1, 1, &commands_sv_map, 0, 19}
};
const int ui_editsPracticeCount =
	sizeof(ui_editsPractice)/sizeof(Edits);
int ui_activeEditPractice = -1;

Edits ui_editsHost[] = {
  {150, 250, 435, 450, "sv_gametype", 0, 0, &commands_sv_gametype, 0, 1},
  {150, 250, 415, 430, "sv_max_players", 0, 1, &commands_sv_max_players, 0, 1},
  {150, 250, 395, 410, "sv_balls", 0, 1, &commands_sv_balls, 0, 1},
  {150, 250, 375, 390, "sv_timelimit", 0, 1, &commands_sv_timelimit, 0, 2},
  {150, 250, 355, 370, "sv_fall", 0, 1, &commands_sv_fall, 0, 1},
  {150, 250, 335, 350, "sv_health", 0, 1, &commands_sv_health, 0, 3},
  {450, 750, 435, 450, "sv_name", 1, 1, &commands_sv_name, 0, 29},
  {450, 550, 415, 430, "sv_port", 0, 1, &commands_sv_port, 0, 5},
  {450, 650, 395, 410, "sv_passwd", 1, 1, &commands_sv_passwd, 0, 19},
  {450, 650, 355, 370, "sv_map", 1, 1, &commands_sv_map, 0, 19}
};
const int ui_editsHostCount =
	sizeof(ui_editsHost)/sizeof(Edits);
int ui_activeEditHost = -1;

Edits ui_editsJoin[] = {
  {250, 600, 350, 365, "cl_hostname", 1, 1, &commands_cl_hostname, 0, 34},
  {250, 350, 330, 345, "cl_port",     0, 1, &commands_cl_port,     0, 5},
  {250, 450, 310, 325, "cl_passwd",   1, 1, &commands_cl_passwd,   0, 19}
};
const int ui_editsJoinCount =
	sizeof(ui_editsJoin)/sizeof(Edits);
int ui_activeEditJoin = -1;

Edits ui_editsOptions[] = {
  {100, 400, 500, 515, "name", 1, 1, &commands_name, 0, 29},
  {600, 650, 500, 515, "vd_width", 0, 1, &commands_vd_width, 0, 4},
  {675, 725, 500, 515, "vd_height", 0, 1, &commands_vd_height, 0, 4},
  {600, 650, 480, 495, "vd_fullscreen", 0, 1, &commands_vd_fullscreen, 0, 1}
};
const int ui_editsOptionsCount =
	sizeof(ui_editsOptions)/sizeof(Edits);
int ui_activeEditOptions = -1;


static void *sndClick;
static void *sndClonk;

static char isMap(char *name)
{
	int p;

	for (p=0; p<mapsCount; p++)
		if (!strcmp(maps[p], name))
			return p;

	return -1;
}

static void addMap(char *name)
{
	/* check if filename ends with '.fsm' */
	if (strlen(name) > 4
		&& name[strlen(name)-4]=='.'
		&& name[strlen(name)-3]=='f'
		&& name[strlen(name)-2]=='s'
		&& name[strlen(name)-1]=='m')
	{
		name[strlen(name)-4] = 0;
		if (isMap(name) == -1)
		{
			snprintf(maps[mapsCount], 81, "%s", name);
			mapsCount++;
		}
	}
	return;
}

static void editCheckout(Edits *edit, int index)
{
	if (edit[index].type)
		snprintf(edit[index].text, 81, "%s",
			(char *)(edit[index].data));
	else
		snprintf(edit[index].text, 81, "%d",
			*((int *)(edit[index].data)));
	edit[index].len = strlen(edit[index].text);
	return;
}

static void editCommit(Edits *edit, int index)
{
	char buf[81];

	snprintf(buf, 81, "%s %s", edit[index].cmd, edit[index].text);
	commands_process(buf);
	editCheckout(edit, index);
	return;
}


static void buttonPractice(void)
{
	int p;
	for (p=0; p<ui_editsPracticeCount; p++)
		editCheckout(ui_editsPractice, p);
	ui_place = 1;
	return;
}
static void buttonHost(void)
{
	int p;
	for (p=0; p<ui_editsHostCount; p++)
		editCheckout(ui_editsHost, p);
	ui_place = 2;
	return;
}
static void buttonJoin(void)
{
	int p;
	for (p=0; p<ui_editsJoinCount; p++)
		editCheckout(ui_editsJoin, p);
	ui_place = 3;
	return;
}
static void buttonOptions(void)
{
	int p;
	for (p=0; p<ui_editsOptionsCount; p++)
		editCheckout(ui_editsOptions, p);
	ui_place = 4;
	return;
}
static void buttonExit(void)
{
	ui_quitNow = 1;
	return;
}
static void buttonCancel(void)
{
	ui_place = 0;
	return;
}
static void buttonStartPractice(void)
{
	server_semUI = NULL;
	server_semUI = SDL_CreateSemaphore(0);
	if (server_semUI == NULL)
	{
		console_print("ui: Couldn't create semaphore!");
		return;
	}

	memcpy(commands_cl_passwd, commands_sv_passwd, 81);
	serverthread = SDL_CreateThread(server_thread, (void*)2);

	SDL_SemWait(server_semUI);
	SDL_DestroySemaphore(server_semUI);

	client_local = 1;
	client_init();
	return;
}
static void buttonJoinJoin(void)
{
	client_local = 0;
	client_init();
	return;
}
static void buttonDelClPasswd(void)
{
	int p;
	commands_cl_passwd[0] = 0;
	for (p=0; p<ui_editsJoinCount; p++)
		editCheckout(ui_editsJoin, p);
	return;
}
static void buttonHostDelSvPasswd(void)
{
	int p;
	commands_sv_passwd[0] = 0;
	for (p=0; p<ui_editsHostCount; p++)
		editCheckout(ui_editsHost, p);
	return;
}
static void buttonHostHost(void)
{
	server_semUI = NULL;
	server_semUI = SDL_CreateSemaphore(0);
	if (server_semUI == NULL)
	{
		console_print("ui: Couldn't create semaphore!");
		return;
	}

	memcpy(commands_cl_passwd, commands_sv_passwd, 81);
	serverthread = SDL_CreateThread(server_thread, (void*)1);

	SDL_SemWait(server_semUI);
	SDL_DestroySemaphore(server_semUI);

	client_local = 1;
	client_init();
	return;
}

static void buttonPracticeDelSvPasswd(void)
{
	int p;
	commands_sv_passwd[0] = 0;
	for (p=0; p<ui_editsPracticeCount; p++)
		editCheckout(ui_editsPractice, p);
	return;
}
static void buttonPracticeShoppa(void)
{
	int p;
	ui_editsPractice[2].editable = 1;
	commands_process("sv_gametype 0");
	commands_process("sv_max_players 2");
	commands_process("sv_balls 4");
	commands_process("sv_timelimit 45");
	commands_process("sv_fall 1");
	commands_process("sv_health 100");
	commands_process("sv_map shoppa1");
	for (p=0; p<ui_editsPracticeCount; p++)
		editCheckout(ui_editsPractice, p);
	return;
}
static void buttonPracticeFlyShoppa(void)
{
	int p;
	ui_editsPractice[2].editable = 1;
	commands_process("sv_gametype 1");
	commands_process("sv_max_players 2");
	commands_process("sv_balls 4");
	commands_process("sv_timelimit 45");
	commands_process("sv_fall 1");
	commands_process("sv_health 100");
	commands_process("sv_map fly1");
	for (p=0; p<ui_editsPracticeCount; p++)
		editCheckout(ui_editsPractice, p);
	return;
}
static void buttonPracticeWxW(void)
{
	int p;
	ui_editsPractice[2].editable = 1;
	commands_process("sv_gametype 2");
	commands_process("sv_max_players 2");
	commands_process("sv_balls 4");
	commands_process("sv_timelimit 45");
	commands_process("sv_fall 1");
	commands_process("sv_health 100");
	commands_process("sv_map wxw1");
	for (p=0; p<ui_editsPracticeCount; p++)
		editCheckout(ui_editsPractice, p);
	return;
}
static void buttonPracticeProper(void)
{
	int p;
	ui_editsPractice[2].editable = 0;
	commands_process("sv_gametype 3");
	commands_process("sv_max_players 2");
	commands_process("sv_balls 1");
	commands_process("sv_timelimit 15");
	commands_process("sv_fall 1");
	commands_process("sv_health 200");
	commands_process("sv_map proper1");
	for (p=0; p<ui_editsPracticeCount; p++)
		editCheckout(ui_editsPractice, p);
	return;
}
static void buttonPracticeRR(void)
{
	int p;
	ui_editsPractice[2].editable = 0;
	commands_process("sv_gametype 4");
	commands_process("sv_max_players 2");
	commands_process("sv_balls 1");
	commands_process("sv_timelimit 30");
	commands_process("sv_fall 0");
	commands_process("sv_health 1");
	commands_process("sv_map rr1");
	for (p=0; p<ui_editsPracticeCount; p++)
		editCheckout(ui_editsPractice, p);
	return;
}

static void buttonHostShoppa(void)
{
	int p;
	ui_editsHost[2].editable = 1;
	commands_process("sv_gametype 0");
	commands_process("sv_max_players 6");
	commands_process("sv_balls 4");
	commands_process("sv_timelimit 45");
	commands_process("sv_fall 1");
	commands_process("sv_health 100");
	commands_process("sv_map shoppa1");
	for (p=0; p<ui_editsHostCount; p++)
		editCheckout(ui_editsHost, p);
	return;
}
static void buttonHostFlyShoppa(void)
{
	int p;
	ui_editsHost[2].editable = 1;
	commands_process("sv_gametype 1");
	commands_process("sv_max_players 6");
	commands_process("sv_balls 4");
	commands_process("sv_timelimit 45");
	commands_process("sv_fall 1");
	commands_process("sv_health 100");
	commands_process("sv_map fly1");
	for (p=0; p<ui_editsHostCount; p++)
		editCheckout(ui_editsHost, p);
	return;
}
static void buttonHostWxW(void)
{
	int p;
	ui_editsHost[2].editable = 1;
	commands_process("sv_gametype 2");
	commands_process("sv_max_players 6");
	commands_process("sv_balls 4");
	commands_process("sv_timelimit 45");
	commands_process("sv_fall 1");
	commands_process("sv_health 100");
	commands_process("sv_map wxw1");
	for (p=0; p<ui_editsHostCount; p++)
		editCheckout(ui_editsHost, p);
	return;
}
static void buttonHostProper(void)
{
	int p;
	ui_editsHost[2].editable = 0;
	commands_process("sv_gametype 3");
	commands_process("sv_max_players 6");
	commands_process("sv_balls 1");
	commands_process("sv_timelimit 15");
	commands_process("sv_fall 1");
	commands_process("sv_health 200");
	commands_process("sv_map proper1");
	for (p=0; p<ui_editsHostCount; p++)
		editCheckout(ui_editsHost, p);
	return;
}
static void buttonHostRR(void)
{
	int p;
	ui_editsHost[2].editable = 0;
	commands_process("sv_gametype 4");
	commands_process("sv_max_players 6");
	commands_process("sv_balls 1");
	commands_process("sv_timelimit 30");
	commands_process("sv_fall 0");
	commands_process("sv_health 1");
	commands_process("sv_map rr1");
	for (p=0; p<ui_editsHostCount; p++)
		editCheckout(ui_editsHost, p);
	return;
}
static void buttonNextMap(void)
{
	int p;
	Map *tmpmap;
	int counter = 0;

	p = isMap(commands_sv_map);

nextmap:
	counter++;
	if (counter == 10000)
		return;

	p++;
	if (p == mapsCount)
		p = 0;
	
	tmpmap = map_load(maps[p], 1);
	if (tmpmap)
	{
		if (tmpmap->gametypes[commands_sv_gametype])
			map_unload(tmpmap);
		else
		{
			map_unload(tmpmap);
			goto nextmap;
		}
	}
	else
		goto nextmap;


	snprintf(commands_sv_map, 81, maps[p]);
	for (p=0; p<ui_editsHostCount; p++)
		editCheckout(ui_editsHost, p);
	for (p=0; p<ui_editsPracticeCount; p++)
		editCheckout(ui_editsPractice, p);

	return;
}
static void buttonPrevMap(void)
{
	int p;
	Map *tmpmap;
	int counter = 0;

	p = isMap(commands_sv_map);

prevmap:
	counter++;
	if (counter == 10000)
		return;

	p--;
	if (p == -1)
		p = mapsCount-1;
	
	tmpmap = map_load(maps[p], 1);
	if (tmpmap)
	{
		if (tmpmap->gametypes[commands_sv_gametype])
			map_unload(tmpmap);
		else
		{
			map_unload(tmpmap);
			goto prevmap;
		}
	}
	else
		goto prevmap;


	snprintf(commands_sv_map, 81, maps[p]);
	for (p=0; p<ui_editsHostCount; p++)
		editCheckout(ui_editsHost, p);
	for (p=0; p<ui_editsPracticeCount; p++)
		editCheckout(ui_editsPractice, p);

	return;
}
static void buttonApplyVideo(void)
{
	commands_process("vd_reload");
	return;
}


void ui_loadTextures()
{
	ui_texMainMenu = textures_loadBMP("graphics/ui/mainmenu.bmp");
	ui_texPractice = textures_loadBMP("graphics/ui/practice.bmp");
	ui_texHost     = textures_loadBMP("graphics/ui/host.bmp");
	ui_texJoin     = textures_loadBMP("graphics/ui/join.bmp");
	ui_texOptions  = textures_loadBMP("graphics/ui/options.bmp");

	ui_texCursor   = textures_loadBMPAlpha("graphics/ui/cursor.bmp");

	return;
}
void ui_unloadTextures()
{
	textures_unloadBMP(&ui_texMainMenu);
	textures_unloadBMP(&ui_texPractice);
	textures_unloadBMP(&ui_texHost);
	textures_unloadBMP(&ui_texJoin);
	textures_unloadBMP(&ui_texOptions);

	textures_unloadBMP(&ui_texCursor);
	return;
}


int ui_init()
{
	int p;
	DIR *dir;
	struct dirent *mapf;

	console_print("Initializing ui...");

	ui_loadTextures();

	for (p=0; p<ui_editsPracticeCount; p++)
	{
		memset(ui_editsPractice[p].text, 0, 81);
	}

	/* load maps */
	memset(maps, 0, 81*MAXMAPS);
	mapsCount = 0;

	dir = opendir("maps"); /* search in 'maps' directory */
	if (dir)
	{
		while ((mapf = readdir(dir)) != NULL)
			addMap(mapf->d_name);
		closedir(dir);
	}

#ifndef WIN32 /* UNIXes, oh yeah! */
	{
		char dirnameBuf[256];

		snprintf(dirnameBuf, 256, "%s/.freeshoppa/maps",
			getenv("HOME")); /* search also in ~/.freeshoppa/maps */
		dir = opendir(dirnameBuf);
		if (dir)
		{
			while ((mapf = readdir(dir)) != NULL)
				addMap(mapf->d_name);
			closedir(dir);
		}

		snprintf(dirnameBuf, 256, "%s/maps", SHOPPADIR);
			/* search in $PREFIX/share/maps */
		dir = opendir(dirnameBuf);
		if (dir)
		{
			while ((mapf = readdir(dir)) != NULL)
				addMap(mapf->d_name);
			closedir(dir);
		}
	}
#endif

	sndClick = audio_loadWAV("sounds/click.wav");
	sndClonk = audio_loadWAV("sounds/clonk.wav");
	console_print("^2DONE");
	return 0;
}

void ui_cleanup()
{
	console_print("Cleaning ui...");

	ui_unloadTextures();
	audio_unloadWAV(sndClick);
	audio_unloadWAV(sndClonk);

	console_print("^2DONE");
	return;
}

/* forgive me the code which is not as nice as it should be */
void ui_click(float x, float y)
{
	int p;
	switch (ui_place)
	{
		case 0: /* main menu */
			for (p=0; p<ui_buttonsMainMenuCount; p++)
			{
				if (x>=ui_buttonsMainMenu[p].x1 &&
				    x<=ui_buttonsMainMenu[p].x2 &&
				    y>=ui_buttonsMainMenu[p].y1 &&
				    y<=ui_buttonsMainMenu[p].y2)
				{
					audio_play(sndClick);
				    	ui_buttonsMainMenu[p].ptr();
				}
			}
			break;
		case 1: /* practice */
			if (ui_activeEditPractice >= 0)
			{
				editCommit(ui_editsPractice,
					ui_activeEditPractice);
				ui_activeEditPractice = -1;
			}
			for (p=0; p<ui_buttonsPracticeCount; p++)
			{
				if (x>=ui_buttonsPractice[p].x1 &&
				    x<=ui_buttonsPractice[p].x2 &&
				    y>=ui_buttonsPractice[p].y1 &&
				    y<=ui_buttonsPractice[p].y2)
				{
					audio_play(sndClick);
				    	ui_buttonsPractice[p].ptr();
				}
			}
			for (p=0; p<ui_editsPracticeCount; p++)
			{
				if (x>=ui_editsPractice[p].x1 &&
				    x<=ui_editsPractice[p].x2 &&
				    y>=ui_editsPractice[p].y1 &&
				    y<=ui_editsPractice[p].y2)
				{
					audio_play(sndClick);
				    	ui_activeEditPractice = p;
				}
			}
			break;
		case 2: /* host */
			if (ui_activeEditHost >= 0)
			{
				editCommit(ui_editsHost,
					ui_activeEditHost);
				ui_activeEditHost = -1;
			}
			for (p=0; p<ui_buttonsHostCount; p++)
			{
				if (x>=ui_buttonsHost[p].x1 &&
				    x<=ui_buttonsHost[p].x2 &&
				    y>=ui_buttonsHost[p].y1 &&
				    y<=ui_buttonsHost[p].y2)
				{
					audio_play(sndClick);
				    	ui_buttonsHost[p].ptr();
				}
			}
			for (p=0; p<ui_editsHostCount; p++)
			{
				if (x>=ui_editsHost[p].x1 &&
				    x<=ui_editsHost[p].x2 &&
				    y>=ui_editsHost[p].y1 &&
				    y<=ui_editsHost[p].y2)
				{
					audio_play(sndClick);
				    	ui_activeEditHost = p;
				}
			}
			break;
		case 3: /* join */
			if (ui_activeEditJoin >= 0)
			{
				editCommit(ui_editsJoin,
					ui_activeEditJoin);
				ui_activeEditJoin = -1;
			}
			for (p=0; p<ui_buttonsJoinCount; p++)
			{
				if (x>=ui_buttonsJoin[p].x1 &&
				    x<=ui_buttonsJoin[p].x2 &&
				    y>=ui_buttonsJoin[p].y1 &&
				    y<=ui_buttonsJoin[p].y2)
				{
					audio_play(sndClick);
				    	ui_buttonsJoin[p].ptr();
				}
			}
			for (p=0; p<ui_editsJoinCount; p++)
			{
				if (x>=ui_editsJoin[p].x1 &&
				    x<=ui_editsJoin[p].x2 &&
				    y>=ui_editsJoin[p].y1 &&
				    y<=ui_editsJoin[p].y2)
				{
					audio_play(sndClick);
				    	ui_activeEditJoin = p;
				}
			}
			break;
		case 4: /* options */
			if (ui_activeEditOptions >= 0)
			{
				editCommit(ui_editsOptions,
					ui_activeEditOptions);
				ui_activeEditOptions = -1;
			}
			for (p=0; p<ui_buttonsOptionsCount; p++)
			{
				if (x>=ui_buttonsOptions[p].x1 &&
				    x<=ui_buttonsOptions[p].x2 &&
				    y>=ui_buttonsOptions[p].y1 &&
				    y<=ui_buttonsOptions[p].y2)
				{
					audio_play(sndClick);
				    	ui_buttonsOptions[p].ptr();
				}
			}
			for (p=0; p<ui_editsOptionsCount; p++)
			{
				if (x>=ui_editsOptions[p].x1 &&
				    x<=ui_editsOptions[p].x2 &&
				    y>=ui_editsOptions[p].y1 &&
				    y<=ui_editsOptions[p].y2)
				{
					audio_play(sndClick);
				    	ui_activeEditOptions = p;
				}
			}
			break;
	}
	
	return;
}

void ui_keyDown(SDLKey key)
{
	if (key == SDLK_ESCAPE)
		switch (ui_place)
		{
			case 0:
				ui_quitNow = 1;
				break;
			default:
				ui_place = 0;
				audio_play(sndClick);
		}

	/* dirty code again... */

	/* practice */
	if (ui_place == 1)
		if (ui_activeEditPractice >= 0
			&& ui_editsPractice[ui_activeEditPractice].editable)
		{
			misc_keyboardInput(key,
				ui_editsPractice[ui_activeEditPractice].text,
				&ui_editsPractice[ui_activeEditPractice].len,
				ui_editsPractice[ui_activeEditPractice].maxlen,
				ui_editsPractice[ui_activeEditPractice].type);
			audio_play(sndClonk);
			if (key == SDLK_RETURN)
			{
				editCommit(ui_editsPractice,
					ui_activeEditPractice);
				ui_activeEditPractice = -1;
			}
		}
	/* host */
	if (ui_place == 2)
		if (ui_activeEditHost >= 0
			&& ui_editsHost[ui_activeEditHost].editable)
		{
			misc_keyboardInput(key,
				ui_editsHost[ui_activeEditHost].text,
				&ui_editsHost[ui_activeEditHost].len,
				ui_editsHost[ui_activeEditHost].maxlen,
				ui_editsHost[ui_activeEditHost].type);
			audio_play(sndClonk);
			if (key == SDLK_RETURN)
			{
				editCommit(ui_editsHost,
					ui_activeEditHost);
				ui_activeEditHost = -1;
			}
		}
	/* join */
	if (ui_place == 3)
		if (ui_activeEditJoin >= 0
			&& ui_editsJoin[ui_activeEditJoin].editable)
		{
			misc_keyboardInput(key,
				ui_editsJoin[ui_activeEditJoin].text,
				&ui_editsJoin[ui_activeEditJoin].len,
				ui_editsJoin[ui_activeEditJoin].maxlen,
				ui_editsJoin[ui_activeEditJoin].type);
			audio_play(sndClonk);
			if (key == SDLK_RETURN)
			{
				editCommit(ui_editsJoin,
					ui_activeEditJoin);
				ui_activeEditJoin = -1;
			}
		}
	/* options */
	if (ui_place == 4)
		if (ui_activeEditOptions >= 0
			&& ui_editsOptions[ui_activeEditOptions].editable)
		{
			misc_keyboardInput(key,
				ui_editsOptions[ui_activeEditOptions].text,
				&ui_editsOptions[ui_activeEditOptions].len,
				ui_editsOptions[ui_activeEditOptions].maxlen,
				ui_editsOptions[ui_activeEditOptions].type);
			audio_play(sndClonk);
			if (key == SDLK_RETURN)
			{
				editCommit(ui_editsOptions,
					ui_activeEditOptions);
				ui_activeEditOptions = -1;
			}
		}
	
	return;
}
