/*
 *  (C) Copyright 2007-2009 by FreeShoppa Development Team
 *                        (http://chomzee.ethernet.pl/freeshoppa/)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <GL/gl.h>
#ifdef WIN32
#include <gl\glext.h>
#endif

#include "freeshoppa.h"

/* counter for blinking cursor */
static int cursor = 0;

static void drawCursor()
{
	glEnable(GL_TEXTURE_2D);
	glColor3f(1.0f, 1.0f, 1.0f);

	glBindTexture(GL_TEXTURE_2D, ui_texCursor);

	glBegin(GL_TRIANGLES);

	glTexCoord2f(0.0f, 1.0f);
	glVertex2f(input_mouse.x, input_mouse.y);
	glTexCoord2f(0.0f, 0.0f);
	glVertex2f(input_mouse.x, input_mouse.y-20.0f);
	glTexCoord2f(1.0f, 0.0f);
	glVertex2f(input_mouse.x+20.0f, input_mouse.y-20.0f);

	glTexCoord2f(0.0f, 1.0f);
	glVertex2f(input_mouse.x, input_mouse.y);
	glTexCoord2f(1.0f, 0.0f);
	glVertex2f(input_mouse.x+20.0f, input_mouse.y-20.0f);
	glTexCoord2f(1.0f, 1.0f);
	glVertex2f(input_mouse.x+20.0f, input_mouse.y);

	glEnd();
	glDisable(GL_TEXTURE_2D);
	return;
}

static void drawInterface(int tex, Buttons *buttons, const int buttonsCount,
	Edits *edits, const int editsCount, int activeEdit)
{
	int p, q;

	glEnable(GL_TEXTURE_2D);
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	glBindTexture(GL_TEXTURE_2D, tex);
	glBegin( GL_QUADS );
	glTexCoord2f( 0.0f, 0.25f );
	glVertex2f(0.0f, 0.0f);

	glTexCoord2f( 1.0f, 0.25f );
	glVertex2f(800.0f, 0.0f);

	glTexCoord2f( 1.0f, 1.0f );
	glVertex2f(800.0f, 600.0f);

	glTexCoord2f( 0.0f, 1.0f );
	glVertex2f(0.0f, 600.0f);

	glEnd();
	glDisable(GL_TEXTURE_2D);

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	for (p=0; p<buttonsCount; p++)
	{
		glBegin(GL_LINE_STRIP);

		glVertex2f(buttons[p].x1,
			buttons[p].y1);
		glVertex2f(buttons[p].x2,
			buttons[p].y1);
		glVertex2f(buttons[p].x2,
			buttons[p].y2);
		glVertex2f(buttons[p].x1,
			buttons[p].y2);
		glVertex2f(buttons[p].x1,
			buttons[p].y1);

		glEnd();
	}

	if (edits == NULL)
		return;

	for (p=0; p<editsCount; p++)
	{
		if (activeEdit == p)
			glColor4f(1.0f, 0.0f, 0.0f, 1.0f);
		else
			glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
		glBegin(GL_LINE_STRIP);

		glVertex2f(edits[p].x1,
			edits[p].y1);
		glVertex2f(edits[p].x2,
			edits[p].y1);
		glVertex2f(edits[p].x2,
			edits[p].y2);
		glVertex2f(edits[p].x1,
			edits[p].y2);
		glVertex2f(edits[p].x1,
			edits[p].y1);

		glEnd();

		/* draw chars */
		glEnable(GL_TEXTURE_2D);
		glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
		for (q=0; q<edits[p].len; q++)
		{
			fonts_drawChar(console_texFont,
				edits[p].text[q],
				edits[p].x1 + q*10.0f,
				edits[p].x1 + (q+1)*10.0f,
				edits[p].y1,
				edits[p].y1 + 15.0f);
		}

		/* blinking cursor */
		if (
			(activeEdit == p)
			&&
			edits[p].editable
			&&
			((commands_co_blink == 0) ||
			(!((cursor/commands_co_blink) % 2)))
		)
			fonts_drawChar(console_texFont, '_',
				edits[p].x1 + q*10.0f,
				edits[p].x1 + (q+1)*10.0f,
				edits[p].y1,
				edits[p].y1 + 15.0f);

		glDisable(GL_TEXTURE_2D);
	}
	cursor ++;

	return;
}

void ui_render_draw(void)
{
	/* prepare everything... */
	glDisable(GL_DEPTH_TEST);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);

	glOrtho(0.0-video_xOffset, 800.0+video_xOffset,
		0.0-video_yOffset, 600.0+video_yOffset, -1.0f, 1.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLineWidth(1.0f);

	/* let's draw! */
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();


	switch (ui_place)
	{
		case 0: /* main menu */
			drawInterface(ui_texMainMenu, ui_buttonsMainMenu,
				ui_buttonsMainMenuCount, NULL, 0, 0);
			break;
		case 1: /* practice */
			drawInterface(ui_texPractice, ui_buttonsPractice,
				ui_buttonsPracticeCount, ui_editsPractice,
				ui_editsPracticeCount, ui_activeEditPractice);
			break;
		case 2: /* host */
			drawInterface(ui_texHost, ui_buttonsHost,
				ui_buttonsHostCount, ui_editsHost,
				ui_editsHostCount, ui_activeEditHost);
			break;
		case 3: /* join */
			drawInterface(ui_texJoin, ui_buttonsJoin,
				ui_buttonsJoinCount, ui_editsJoin,
				ui_editsJoinCount, ui_activeEditJoin);
			break;
		case 4: /* options */
			drawInterface(ui_texOptions, ui_buttonsOptions,
				ui_buttonsOptionsCount, ui_editsOptions,
				ui_editsOptionsCount, ui_activeEditOptions);

			break;
	}

	drawCursor();


	return;
}

