/*
 *  (C) Copyright 2007-2009 by FreeShoppa Development Team
 *                        (http://chomzee.ethernet.pl/freeshoppa/)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <GL/gl.h>
#ifdef WIN32
#include <gl\glext.h>
#endif

#include "freeshoppa.h"

float video_xOffset;
float video_yOffset;

static int swidth = 0;
static int sheight = 0;

int video_init()
{
	/*const SDL_VideoInfo* info = NULL;*/
	/*int bpp = 0;*/
	int flags;
	
	console_print("Initializing video...");

	if(SDL_Init(SDL_INIT_VIDEO) < 0)
	{
		console_print("^1FAILED!!!");
		return 1;
	}
	SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY,
		SDL_DEFAULT_REPEAT_INTERVAL);


	/*
	info = SDL_GetVideoInfo();

	if(!info)
	{
		console_print("^1FAILED!!!");
		return 1;
	}
	*/
	
	
	/*bpp = info->vfmt->BitsPerPixel;*/

	SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 5);
	SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 5);
	SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 5);
	SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 16);
	SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);

	if (commands_vd_vsync)
		SDL_GL_SetAttribute(SDL_GL_SWAP_CONTROL, 1);
	else
		SDL_GL_SetAttribute(SDL_GL_SWAP_CONTROL, 0);
	
	if (commands_vd_fullscreen)
		flags = SDL_OPENGL | SDL_FULLSCREEN;
	else
		flags = SDL_OPENGL;

	
	if (SDL_SetVideoMode(commands_vd_width,
		commands_vd_height, 0, flags) == NULL)
	{
		console_print("^1FAILED!!!");
		console_print("Trying 800x600 windowed mode...");
		commands_process("vd_width 800");
		commands_process("vd_height 600");
		commands_process("vd_fullscreen 0");
		flags = SDL_OPENGL;
		
		if (SDL_SetVideoMode(commands_vd_width,
			commands_vd_height, 0, flags) == NULL)
		{
			console_print("^1FAILED!!!");
			return 1;
		}
	}
	video_setupOGL(commands_vd_width, commands_vd_height);
	
	SDL_ShowCursor(SDL_DISABLE);
	SDL_WM_GrabInput(SDL_GRAB_ON);
	
	console_print("^2DONE");

	/* Everything's okay. Let's init ui now */
	if (ui_init())
		return 1;
	return 0;
}

int video_run()
{
	int counter = 0;
	int theTick = SDL_GetTicks();

	/* "Infinity - Where do we go from here?
	 *  Infinity - Where do we go from here?
	 *  Infinity - Where do we go?
	 *  Infinity - Where do we go from here?"
	 *			Stratovarius - "Infinity"
	 *
	 * I mean... Inifite loop ;)
	 */
	for (;;)
	{
		if (ui_quitNow)
		{
			/* oh, we have to quit :( */
			if (ui_clientRunning)
				client_cleanup();
			return 0;
		}
		misc_waitFrame(20);
		
		if (commands_db_frametime)
			theTick = SDL_GetTicks();

		input_processEvents();
		
		/* if there's no client running, only ui */
		if (!ui_clientRunning)
			ui_render_draw();
		/* if there's client running, do the drawing */
		else
		{
			game_do(); /* do the game's engine */
			client_render_draw();
		}

		if (console_active)
			console_draw();
		SDL_GL_SwapBuffers();

		if (commands_vd_bmpmovie)
		{
			static int counter = 0;

			if (!(counter%2))
			{
				char buf[16];
				snprintf(buf, 16, "movie%.5d.bmp", counter/2);
				video_takeScreenshot(buf);
			}
			counter++;
		}

		if (commands_db_frametime)
		{
			counter++;
			if (counter == 60)
			{
				counter = 0;
				console_print("frametime: %d",
					SDL_GetTicks()-theTick);
			}
		}


	}

	return 0;
}

void video_setupOGL(int width, int height)
{
	swidth = width;
	sheight = height;

	glShadeModel(GL_SMOOTH);
	glCullFace(GL_BACK);
	glFrontFace(GL_CCW);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_CULL_FACE);
	glViewport(0, 0, width, height);

	if ((float)width/(float)height > 4.0/3.0)
	{
		video_xOffset = ((float)width - (float)height*4.0/3.0)/2.0;
		video_xOffset *= 600.0f/(float)height;
		video_yOffset = 0.0;
	}
	else if ((float)width/(float)height < 4.0/3.0)
	{
		video_xOffset = 0.0;
		video_yOffset = ((float)height - (float)width*3.0/4.0)/2.0;
		video_yOffset *= 800.0f/(float)width;
	}
	else
	{
		video_xOffset = 0.0;
		video_yOffset = 0.0;
	}

	return;
}

int video_takeScreenshot(char *filename)
{
	int p;
	SDL_Surface *screen = SDL_CreateRGBSurface(SDL_SWSURFACE, 
		swidth, sheight, 32,
		0x0000ff, 0x00ff00, 0xff0000, 0x000000);

	unsigned char *pixels;
	unsigned char *line;

	pixels = (unsigned char*)malloc(swidth * sheight * 4);
	if (!pixels)
	{
		console_print("video: failed to alloc memory for screenshot");
		return 1;
	}

	line = (unsigned char*)malloc(swidth * 4);
	if (!line)
	{
		console_print("video: failed to alloc memory for screenshot");
		free(pixels);
		return 1;
	}

	glReadPixels(0, 0, swidth, sheight, GL_RGBA, GL_UNSIGNED_BYTE, pixels);

	/* flip the image */
	for (p=0; p<sheight/2; p++)
	{

		memcpy(line, &pixels[swidth*4*p], swidth*4);
		memcpy(&pixels[swidth*4*p],
			&pixels[swidth*4*(sheight-p-1)], swidth*4);
		memcpy(&pixels[swidth*4*(sheight-p-1)], line, swidth*4);
	}
	free(line);

	screen->pixels = pixels;
	SDL_SaveBMP(screen, filename);
	SDL_FreeSurface(screen);

	return 0;
}

void video_cleanup()
{
	
	if (!ui_clientRunning)
		ui_cleanup();
	else
	{
		/* TODO: Do something else... */
	}

	console_print("Cleaning video...");

	SDL_ShowCursor(SDL_ENABLE);
	SDL_Quit();
	
	console_print("^2DONE");
	return;
}

